# dans ASI

# Simple example:
# - Generate data (as a cheap way of getting some)
# - Perform statistical analysis to describe the data
#
#package require math::statistics
# 
source [pwd]/math/statistics.tcl

# Procedure LFORMAT # debut
#
proc lformat {ainsi liste} {
    foreach item $liste {
        set item [format $ainsi $item]
        lappend fliste $item
    }
    return $fliste
}
#
# Procedure LFORMAT # fin

# procedure DEBUG
#
proc debug {output} {
    # fichier pour debugging
    global flag memo bugatti
    append bugatti "\n$output\n"
    set flag 1
    # text widget
    if {[info exists .debug]} {
        $memo insert end $bugatti
    } else  {
        destroy .debug .non
        toplevel .debug
        wm geometry .debug 710x600+205+66
        set memo [text .debug.t -wrap word -height 22 -font "courier 13" \
                -bg grey88 \
                -fg black\
                -tabs "3c left"\
                -yscrollcommand ".debug.v_scroll set" \
                -xscrollcommand ".debug.h_scroll set"]
        
        scrollbar .debug.v_scroll -command ".debug.t yview"
        scrollbar .debug.h_scroll -command ".debug.t xview" -orient horizontal
        pack .debug.v_scroll -side right -fill y
        pack .debug.h_scroll -side bottom -fill x
        pack .debug.t -side bottom -fill both -expand 1
        
        button .debug.exit -fg red -text "Exit" -command {
            global find_last_indx pointf ici
            destroy .debug .head
        }
        button .debug.wrap -fg red -text "Wrap" -command {
            global flag
            if {$flag == 1} {
                .debug.t configure -wrap none
                set flag 0
            } else  {
                .debug.t configure -wrap word
                set flag 1
            }
        }
        #
        $memo insert end $bugatti
        pack .debug.exit .debug.wrap -side left -padx 5 -fill x
    }
}
#
# Procedure DEBUG fin

#
# Two auxiliary procs
#
proc pause {time} {
   set wait 0
   after [expr {$time*1000}] {set ::wait 1}
   vwait wait
}
proc print-histogram {counts limits} {
   foreach count $counts limit $limits {
      if { $limit != {} } {
	  debug [format "<%12.4g\t%d" $limit $count]
         set prev_limit $limit
      } else {
         debug [format ">%12.4g\t%d" $prev_limit $count]
      }
   }
}
#
# Our source of arbitrary data
#
proc generateData { data1 data2 } {
   upvar 1 $data1 _data1
   upvar 1 $data2 _data2
   set d1 0.0
   set d2 0.0
   for { set i 0 } { $i < 100 } { incr i } {
      set d1 [expr {10.0-2.0*cos(2.0*3.1415926*$i/24.0)+3.5*rand()}]
      set d2 [expr {0.7*$d2+0.3*$d1+0.7*rand()}]
      lappend _data1 $d1
      lappend _data2 $d2
   }
   return {}
}
#
# The analysis session
#
package require Tk
#console show
canvas .plot1
canvas .plot2
pack   .plot1 .plot2 -fill both -side top
generateData data1 data2
debug "Basic statistics:"
set b1 [::math::statistics::basic-stats $data1]
set b2 [::math::statistics::basic-stats $data2]
foreach label {mean min max number stdev var} v1 $b1 v2 $b2 {
   debug "$label\t$v1\t$v2"
}
debug "Plot the data as function of \"time\" and against each other"
::math::statistics::plot-scale .plot1  0 100  0 20
::math::statistics::plot-scale .plot2  0 20   0 20
::math::statistics::plot-tline .plot1 $data1
::math::statistics::plot-tline .plot1 $data2
::math::statistics::plot-xydata .plot2 $data1 $data2
debug "Correlation coefficient:"
debug [::math::statistics::corr $data1 $data2]
pause 3
debug "Plot histograms"
.plot2 delete all
::math::statistics::plot-scale .plot2  0 20 0 100
set limits         [::math::statistics::minmax-histogram-limits 7 16]
set histogram_data [::math::statistics::histogram $limits $data1]
::math::statistics::plot-histogram .plot2 $histogram_data $limits
debug "First series:"
print-histogram $histogram_data $limits
pause 3
set limits         [::math::statistics::minmax-histogram-limits 0 15 10]
set histogram_data [::math::statistics::histogram $limits $data2]
::math::statistics::plot-histogram .plot2 $histogram_data $limits d2
.plot2 itemconfigure d2 -fill red
debug "Second series:"
print-histogram $histogram_data $limits
debug "Autocorrelation function (green):"
set  autoc [::math::statistics::autocorr $data1]
#debug [::math::statistics::map $autoc {[format "%.2f" $x]}] <- c'est mon "lformat"
set autoc [lformat "%.2f" $autoc];# <- ?
debug "... $autoc"
#
debug "Crosscorrelation function (orange):"
set  crossc [::math::statistics::crosscorr $data1 $data2]
#debug [::math::statistics::map $crossc {[format "%.2f" $x]}]  <- c'est mon "lformat"
set crossc [lformat "%.2f" $crossc];# <- ?
debug "... $crossc"
::math::statistics::plot-scale .plot1  0 100 -1  4
::math::statistics::plot-tline .plot1  $autoc "autoc"
::math::statistics::plot-tline .plot1  $crossc "crossc"
.plot1 itemconfigure autoc  -fill "dark green"
.plot1 itemconfigure crossc -fill orange
debug "Quantiles: 0.1, 0.2, 0.5, 0.8, 0.9"
debug "First:  [::math::statistics::quantiles $data1 {0.1 0.2 0.5 0.8 0.9}]"
debug "Second: [::math::statistics::quantiles $data2 {0.1 0.2 0.5 0.8 0.9}]"