
package require Tk
#
source math/statistics.tcl

# procedure DEBUG
#
proc debug {output} {
    # fichier pour debugging
    global flag memo bugatti
    append bugatti "\n$output\n"
    set flag 1
    # text widget
    if {[info exists .debug]} {
        $memo insert end $bugatti
    } else  {
        destroy .debug .non
        toplevel .debug
        wm geometry .debug 710x600+205+66
        set memo [text .debug.t -wrap word -height 22 -font "courier 13" \
                -bg grey88 \
                -fg black\
                -tabs "3c left"\
                -yscrollcommand ".debug.v_scroll set" \
                -xscrollcommand ".debug.h_scroll set"]
        
        scrollbar .debug.v_scroll -command ".debug.t yview"
        scrollbar .debug.h_scroll -command ".debug.t xview" -orient horizontal
        pack .debug.v_scroll -side right -fill y
        pack .debug.h_scroll -side bottom -fill x
        pack .debug.t -side bottom -fill both -expand 1
        
        button .debug.exit -fg red -text "Exit" -command {
            global find_last_indx pointf ici
            destroy .debug .head
        }
        button .debug.wrap -fg red -text "Wrap" -command {
            global flag
            if {$flag == 1} {
                .debug.t configure -wrap none
                set flag 0
            } else  {
                .debug.t configure -wrap word
                set flag 1
            }
        }
        #
        $memo insert end $bugatti
        pack .debug.exit .debug.wrap -side left -padx 5 -fill x
    }
}
#
# Procedure DEBUG fin

proc pnorm {x} {
    #
    # cumulative distribution function (cdf)
    # for the standard normal distribution like in the statistical software 'R'
    # (mean=0 and sd=1)
    #
    # x -> value for which the cdf should be calculated
    #
    set sum [expr {double($x)}]
    set oldSum 0.0
    set i 1
    set denom 1.0
    while {$sum != $oldSum} {
        set oldSum $sum
        incr i 2
        set denom [expr {$denom*$i}]
        #puts "$i - $denom"
        set sum [expr {$oldSum + pow($x,$i)/$denom}]
    }
    return [expr {0.5 + $sum * exp(-0.5 * $x*$x - 0.91893853320467274178)}]
}

proc LillieforsFit {values} {
    #
    # calculate the goodness of fit according to Lilliefors
    # (goodness of fit to a normal distribution)
    #
    # values -> list of values to be tested for normality
    # (these values are sampled counts)
    #
    debug "Basic statistics:"
    set b [::math::statistics::basic-stats $values]
    foreach label {mean min max number stdev var} v $b {
	debug "$label\t$v"
    }
    #
    set mean [lindex $b 0]
    set min [lindex $b 1]
    set max [lindex $b 2]
    set n [lindex $b 3]
    set sd [lindex $b 4]
    set var [lindex $b 5]
    # calculate standard deviation and mean of the sample:
    #set n [llength $values]
    if {$n < 5} {error "insufficient data (n<5)"}
    #set sd [sd $values]
    #set mean [mean $values]
    
    # sort the sample for further processing:
    set values [lsort -real $values]
            
    # standardize the sample data (Z-scores):
    foreach x $values {lappend stdData [expr {($x - $mean)/double($sd)}]}

    # compute the value of the distribution function at every sampled point:
    foreach x $stdData {lappend expData [pnorm $x]}

    # compute D+:
    set i 0
    foreach x $expData {
        incr i
        lappend dplus [expr {$i/double($n)-$x}]
    }
    set dplus [lindex [lsort -real $dplus] end]

    # compute D-:
    set i 0
    foreach x $expData {
        incr i
        lappend dminus [expr {$x-($i-1)/double($n)}]
    }
    set dminus [lindex [lsort -real $dminus] end]
    
    # calculate the test statistic D
    # by finding the maximal vertical difference
    # between the samle and the expectation:
    set D [expr {$dplus > $dminus ? $dplus : $dminus}]
    # we now use the modified statistic Z,
    # because D is only reliable 
    # if the p-value is smaller than 0.1
    ###if {$n < 30} {puts "warning: n=$n<30"}
    return [expr {$D * (sqrt($n) - 0.01 + 0.831/sqrt($n))}]
}

set llf "72 112 111 107 119 92 126 80 81 84 115 118 128 128 123 116 125 126 122 126 127 86 142 132 87 123 133 106 103 118 114 94"

debug [::math::statistics::lillieforsFit $llf]

debug "Basic statistics:"
    set b [::math::statistics::basic-stats $llf]
    foreach label {mean min max number stdev var} v $b {
	debug "$label\t$v"
    }
    #
    set mean [lindex $b 0]
    set min [lindex $b 1]
    set max [lindex $b 2]
    set n [lindex $b 3]
    set sd [lindex $b 4]
    set var [lindex $b 5]


set llf [LillieforsFit $llf]
# A VERIFIER
if {$llf < 0.895} {debug "$llf not significant at : P = 0.05"} else {debug "$llf : significant at P < 0.05"}
if {$llf < 1.035} {debug "$llf not significant at : P = 0.001"} else {debug "$llf : significant at P < 0.001"}


