##+##########################################################################
#
# Magnify.tcl -- Aims a magnifying glass at an image
# by Keith Vetter, January 2006
#

package require Tk
#package require Img

lappend auto_path [file dirname [info script]]

switch $tcl_platform(platform) {
    "unix" {
        lappend auto_path [file join [file dirname [info script]] +/L]
        load ./+/L/Img1.3/libtkimg1.3.so
        load ./+/L/Img1.3/libtkimgjpeg1.3.so
        load ./+/L/Img1.3/libtkimgpng1.3.so
        load ./+/L/Img1.3/libtkimgbmp1.3.so
        load ./+/L/Img1.3/libtkimgppm1.3.so
        load ./+/L/Img1.3/libtkimgps1.3.so
        set separateur "/"
        set separateur_loc 0
    }
    "windows" {
        lappend auto_path [file join [file dirname [info script]] +/W]
        load +/W/Img1.3/tkimg13.dll
        load +/W/Img1.3/tkimgjpeg13.dll
        load +/W/Img1.3/tkimgpng13.dll
        load +/W/Img1.3/tkimgppm13.dll
        load +/W/Img1.3/tkimgbmp13.dll
        load +/W/Img1.3/tkimgps13.dll
        set separateur "/"
        set separateur_loc 2
    }
    "macintosh" {}
    default {}
}


set S(sz) 140                                    ;# Size of magnifying square
set S(sz2) [expr {$S(sz)/2}]
set S(zoom) 2
set S(zsize) [expr {$S(sz)/2/$S(zoom)}]

proc DoDisplay {} {
    global S
    
    wm title . "Magnifying Glass"
    canvas .c -width $S(w) -height $S(h) -highlightthickness 0
    
    #.c create image 0 0 -image ::img::img -anchor nw
    .c create image 0 0 -image fotoa -anchor nw;# tres important le '-anchor nw'
    .c create image 0 0 -image fotoasmall -anchor c -tag small
    
    frame .buttons -bd 2 -relief ridge
    scale .zoom -label "Zoom" -from 2 -to 8 -showvalue 0 \
            -variable S(zoom) -orient h -command FMove -relief raised
    .zoom configure  -font "[font actual [.zoom cget -font]] -weight bold"
    option add *font [.zoom cget -font]
    button .about -text "About" -command About
    pack .c .buttons -side top -fill both -expand 1
    pack .zoom .about -side left -expand 1 -in .buttons
    .c create rect -100 -100 -100 -100 -tag box -fill {} -outline black -width 2
    
    bind .c <Motion> [list MMove %x %y]
    bind .c <Leave> [list MLeave]
    update
    MLeave
}
proc MLeave {} {
    .c coords box -100 -100 -100 -100
    .c coords small -1000 -1000
}
proc MMove {x y} {
    global S
    
    .c coords small $x $y
    
    set xy [list [expr {$x-$S(sz2)}] [expr {$y-$S(sz2)}] \
            [expr {$x+$S(sz2)}] [expr {$y+$S(sz2)}]]
    .c coords box $xy
    
    # Bug here when xy extends past edge of image. Too lazy to fix it.
    set xy [list [expr {$x-$S(zsize)}] [expr {$y-$S(zsize)}] \
            [expr {$x+$S(zsize)}] [expr {$y+$S(zsize)}]]
    catch {
        eval fotoasmall copy fotoa -from $xy -zoom $S(zoom) $S(zoom)
    }
}
proc FMove {args} {
    set ::S(zsize) [expr {$::S(sz)/2/$::S(zoom)}]
    .zoom config -label "Zoom: $::S(zoom)"
    
    foreach var {x y} val [.c coords small] {
        set $var [expr {int($val)}]
    }
    MMove $x $y
}

proc About {} {
    set msg "Magnifying Glass\nby Keith Vetter, January 2006"
    tk_messageBox -message $msg
}

image create photo fotoa
fotoa configure -file /media/ext/mytcl/Developpement/COOl/coo_data/p1010024.jpg

image create photo fotoasmall -width $S(sz) -height $S(sz)
set S(w) [image width fotoa]
set S(h) [image height fotoa]
DoDisplay
#return