

# MICROGRAAL
# cfr. procedure DECOUPAGE
# # Des rectangles sont definis dans l'image initiale, 'fotoa', autour de chaque LM
#    # On cree une image de type photo, appelee 'dec' (pour decoupe)
#    # On copie dans l'image 'dec' le rectangle défini dans la grande image 'fotoa'
#
#
# cfr. procedure BIN2GRAY
# # Cette procedure transforme une image couleur (un rectangle) en noir et blanc (la moyenne des RGB de chaque pixel)
#
#
# # Apres chgt des pixels, on copie l'image source dans l'image de depart (cfr. INSTRUCTION "fotoa copy $res")
# # # Ce n'est d'ailleurs pas indispensable, on peut travailler d'emblee sur le tableau (liste de listes, cfr. ci-dessous)
#
#
# cfr. procedure ONCALCVEC
# # On place alors dans une liste de listes (vector2) la premiere valeur de chaque pixel -
#

# ------------------------------------------------------------------------------------------------------------------------
# Image sampling
# --------------
#
# La vitesse de la procedure (GR)AAL sera fonction du nombre et de la taille des fenetres (rectangles)

# User configuration
# ------------------
# cote_rect
# rgb_max
# distanciation
# signal/fond
# controle_technique
# ------------------------------------------------------------------------------------------------------------------------

global p plaque_aal

destroy .t
toplevel .t
set scb [scrollbar .t.scrollA -command ".t.debut yview"]
pack $scb -side left -fill y
set p [text .t.debut -background white -font "helvetica 12" \
        -width 180 -height 60 -wrap word -yscrollcommand ".t.scrollA set"]
pack $p -side right -fill both -expand 1

# procedure MICROGRAAL
#
proc micrograal {coord_obs} {
    
    global S p plaque_aal
    
    # PROCEDURE bin2gray -- start
    #
    proc bin2gray {image subfactor lm} {
        
        global p
        
        set iota [lrange {- 0 1 2 3 4 5 6 7 8 9} 1 $subfactor]
        set h0 [image height $image]
        set w0 [image width $image]
        set th [expr {round(1.*$h0/$subfactor)}]
        set tw [expr {round(1.*$w0/$subfactor)}]
        set res [image create photo -height $th -width $tw]
        for {set tx 0} {$tx<$tw} {incr tx} {
            set pix ""
            for {set ty 0} {$ty<$th} {incr ty} {
                set y [expr {$ty*$subfactor}]
                set sum 0
                foreach i $iota {
                    set x [expr {$tx*$subfactor}]
                    foreach j $iota {
                        incr sum [lindex [$image get $x $y] 0]
                        incr x
                        if {$x>=$w0} break
                    }
                    incr y
                    if {$y>=$h0} break
                }
                # R = G = B = moyenne des pixels
                set g [expr {round(1.0*$sum/($subfactor*$subfactor))}]
                lappend pix [
                format #%02x%02x%02x $g $g $g
                ]
                
            }
            # On rentre les valeurs RGB de chaque pixel de l'image
            $res put $pix -to $tx 0
        }
        
        img_show $res $lm
        
        return $res
    }
    #
    # PROCEDURE bin2gray -- end
    
    
    
    # Procedure     presentation d'une image
    #
    # PRESENTATION de l'IMAGE GRISE
    
    proc  img_show {figure numero} {
        destroy .tog($numero)
        toplevel .tog($numero)
        set horz 150
        set vert [expr {$numero*90}]
        wm geometry .tog($numero) 250x65+$horz+$vert
        
        # scrolling canvas
        destroy .tog($numero).can .tog($numero).v_scroll .tog($numero).h_scroll
        #canvas .tog($numero).can -width $tw -height $th
        canvas .tog($numero).can \
                -yscrollcommand ".tog($numero).v_scroll set" \
                -xscrollcommand ".tog($numero).h_scroll set"
        
        scrollbar .tog($numero).v_scroll -command ".tog($numero).can yview"
        scrollbar .tog($numero).h_scroll -command ".tog($numero).can xview" -orient horizontal
        .tog($numero).can create image 0 0 -image $figure -anchor nw
        #
        # A present l'astuce pour configurer automatiquement le deroulement du canvas
        .tog($numero).can configure -scrollregion [.tog($numero).can bbox all]
        #
        pack .tog($numero).v_scroll -side right -fill y
        pack .tog($numero).h_scroll -side bottom -fill x
        pack .tog($numero).can -side left -expand 1
    }
    #
    #
    #
    #    Procedure     presentation d'une image
    
    
    # procedure ONCALCVEC # debut
    #
    # ... sur le bloc d'image grise (le bloc est grise, pas l'image)
    # On y peche les valeurs RBG pour chaque pixel
    # R = G = B parce que bin2gray
    # On en prend la premiere
    # On la divise par 25.5 (pour avoir qch entre 0 et 10 - Note: 0 = noir) <- par 12.75 si on veut plus de nuances
    # le tout est place dans une liste de listes
    #
    proc oncalcvec {dec wi he} {
        
        global p rgb_max
        
        # set rgb_max 1
        # rgb_max 25.5  Pixels simplifies de 0 a 10
        # rgb_max 12.75 Pixels simplifies de 0 a 20
        # rgb_max 6.375 Pixels simplifies de 0 a 40
        # rgb_max 3.1875 Pixels simplifies de 0 a 80
        # rgb_max 1.59375 Pixels simplifies de 0 a 160
        # rgb_max 1 Pixels simplifies de 0 a 255
        
        $p insert end "\nprocedure ONCALCVEC hauteur de l'image $he et largeur $wi"
        
        for {set i 0} {$i < [expr $he]} {incr i} {
            #$p insert end "\nprocedure ONCALCVEC ligne suivante"
            for {set j 0} {$j < [expr $wi]} {incr j} {
                #$p insert end "... [$dec get $j $i] "
                set point [lindex [$dec get $j $i] 0]
                set point [expr {round($point * 1.0/$rgb_max)}]
                append vector "$point "
            }
            update
            append vector "\n"
        }
        #$p insert end "\n$vector"
        return $vector
    }
    #
    # Procedure ONCALCVEC # fin
    
    
    
    # Procedure DECOUPAGE - debut
    #
    # On y procede au decoupage de l'image en rectangles
    # Ces rectangles sont supposes contenir le point recherche (sans quoi ...)
    #
    proc decoupage {lm x y cote} {
        
        #
        # appelee par la procedure SAMPLING
        #
        
        global p rectangle
        
        if {[llength $cote] == 1} {
            set bas $cote
            set ht $cote
            set gch $cote
            set dt $gch
            set th [expr {int($ht + $bas)}]
            set tw [expr {$dt + $gch}]
        } else  {
            set bas [lindex $cote 1]
            set ht [lindex $cote 1]
            set gch [lindex $cote 0]
            set dt [lindex $cote 0]
            set th [expr {int($ht + $bas)}]
            set tw [expr {int($dt + $gch)}]
        }
        
        set xgch [expr {$x - $gch}]
        set xdt [expr {$x + $dt}]
        set yht [expr {$y - $bas}]
        set ybas [expr {$y + $ht}]
        #debug
        #$p insert end "\n-+- -+- -+- -+- -+- -+-\nProcedure decoupage gch = \
        #        $gch, dt = $dt, ht = $ht, bas = $bas, \
        #        image width $tw and height $th\
        #        \nrectangle or SQUARE is $xgch $yht $xdt $ybas\
        #\n-+- -+- -+- -+- -+- -+-\n"
        set rectangle($lm) [list $xgch $yht $xdt $ybas]
        
        # On cree une image de type photo, appelee 'dec' (pour decoupe)
        set dec [image create photo -height $th -width $tw]
        # On copie dans l'image 'dec' le rectangle défini dans la grande image 'fotoa'
        $dec copy fotoa -from  $xgch $yht $xdt $ybas
        
        return $dec
    }
    #
    # Procedure DECOUPAGE - FIN
    
    
    
    
    
    proc oui_ou_non {vector seuil signal_fond i} {
        
        global p nobibloc nocontrast onretaille solutions cote_rect
        
        # Procedure menant aux CHOIX apres nettoyage et couture des elements de la table
        set choix_plusieurs [menant_aux_choix $vector $i]
        $p insert end "\
                \nCHOIX_PLUSIEURS soit $choix_plusieurs, avec le seuil '$seuil' et fenetre $cote_rect($i):\
                \t[lindex $choix_plusieurs 0] et [lindex $choix_plusieurs 1], \
                \t[lindex $choix_plusieurs 2] et [lindex $choix_plusieurs 3]\
                \tnobibloc = $nobibloc, nocontrast = $nocontrast, onretaille $onretaille\n"
        # Appel a la procedure DISCUSSION
        #
        set prem_choix_lig($i) [lindex $choix_plusieurs 0]
        set prem_choix_col($i) [lindex $choix_plusieurs 1]
        set deux_choix_lig($i) [lindex $choix_plusieurs 2]
        set deux_choix_col($i) [lindex $choix_plusieurs 3]
        #
        set retour [discussion $prem_choix_lig($i) $prem_choix_col($i) $deux_choix_lig($i) $deux_choix_col($i) $nocontrast $nobibloc $onretaille $i]
        #
        #
        if {$solutions == 1} {
            # S'il y a une solution, on sort par la force (break), on ne change pas de seuil pour recommencer
            if {$signal_fond > 0.05 && $signal_fond < 0.70} {
                set retour [list $prem_choix_lig($i) $prem_choix_col($i)]
            } else  {
                set retour "none none"
            }
        }
        return $retour
    }
    
    
    
    
    
    
    # Procedure SB
    #
    # verifie si le signal ne devient pas du bruit
    #
    proc sb {vector_table lm} {
        #
        global p cote_rect
        #
        #$p insert end "\nvector_table \n$vector_table"
        set x [expr {[lindex $cote_rect($lm) 0] * 2}]
        set y [expr {[lindex $cote_rect($lm) 1] * 2}]
        #
        # HUME matrix -  liste des listes ('vector_table') transformee en matrice de Hume ('hume_table')
        set hume_table "2 $x $y "
        foreach item $vector_table {if {$item != "\n"} {append hume_table "$item "}}
        #
        set nbtot [expr {[mcols $hume_table]*[mrows $hume_table]}]
        set totsum [msum [msum $hume_table]]
        set signal_bruit [expr {$totsum * 1.0 / $nbtot}]
        return $signal_bruit
    }
    
    # Procedure TRESHOLD - start
    #
    # appelee par SAMPLING
    #
    #
    proc treshold {vector_table lm} {
        #
        # Qui appelle treshold ?? (Rep/ sampling)
        #
        global p cote_rect tucol tulig landmark rgb_max candidats
        #
        set x [expr {[lindex $cote_rect($lm) 0] * 2}]
        set y [expr {[lindex $cote_rect($lm) 1] * 2}]
        
        # HUME matrix -  liste des listes ('vector_table') transformee en matrice de Hume ('hume_table')
        set hume_table "2 $x $y "
        foreach item $vector_table {if {$item != "\n"} {append hume_table "$item "}}
        
        #
        # HISTOGRAMME des pixels - debut
        #
        set data_pix [vtrim $hume_table]
        set tot_pix [llength $data_pix]
        #debug
        #$p insert end "total $tot_pix"
        #
        # Valeur maximum d'un pixel r=g=b
        set max [expr {round(255 * 1.0/$rgb_max)}]
        #
        set epaisseur 1
        for {set i 0} {$i < [expr {$max + $epaisseur}]} {incr i $epaisseur} {
            lappend limits $i
        }
        #debug
        #$p insert end "\nProcedure TRESHOLD max = $max\nLimits \t$limits"
        set limits_correction [lrange $limits 0 end-1]
        # Calcul des hauteurs des barres de l'histogramme pour les limites choisies
        set counts [::math::statistics::histogram $limits_correction $data_pix]
        #debug
        #$p insert end "\nCounts \t$counts"
        foreach item $counts {append freq_counts "[expr {round(($item * 1.0 / $tot_pix) * 100)}] "}
        #
        # ASCII visualization of HISTOGRAM - start
        #
        $p insert end "\n\n"
        set compteur 0
        foreach item $freq_counts {
            if {$item > 0} {
                for {set i 0} {$i < $item} {incr i} {
                    $p insert end "|"
                }
                $p insert end "\t\t\t\t[lindex $limits $compteur]\n"
            }
            incr compteur
        }
        #
        # ASCII visualization of HISTOGRAM - end
        
        #
        for {set i 0} {$i < [expr {$max + 1}]} {incr i} {
            set a($i) [lindex $freq_counts $i]
        }
        #
        # Histogramme des pixels - fin
        
        #
        # Moyenne, a tout hasard (ne sert pas a l'algo du seuil)
        #
        set nbtot [expr {[mcols $hume_table]*[mrows $hume_table]}]
        set totsum [msum [msum $hume_table]]
        set moy [expr {$totsum * 1.0 / $nbtot}]
        #debug
        #$p insert end  "$totsum / $nbtot = ? $moy"
        
        # PREFILTRAGE, qui devrait enlever les toutes petites zones (trop petites pour etre des veines)
        #
        # On elimine les R=G=B dont la frequence est nulle. On elimine aussi ceux dont la frequence est proche de zero
        # Elimination des valeurs dont la freq < 0.005 - ce choix est fixe, il est pour l'instant subjectif
        set stat_rgb ""
        for {set i 0} {$i < [expr {$max + 1}]} {incr i} {
            set freq [expr {$a($i) * 1.0 / 100}]
            if {$freq > 0.005} {lappend stat_rgb "$i $freq"}
        }
        #debug
        #$p insert end "RECHERCHE DU SEUIL - deux algoritmes produisant seuil_1 et/ou seuil_2\nALGO 1 :\t"
        #
        # 'stat-rgb' contient la valeur d'un pixel et sa frequence (> 0.005)
        # Les valeurs informatives ET le fond sont a present dans la liste 'stat_rgb'
        
        
        # ALGO DU SEUIL num 1 - ... seuil = min + (max-min)/2) ... - debut
        #
        set seuil_1 [seuil_1 $stat_rgb]
        #
        #debug
        #$p insert end "\nalgo 1 : seuil = min + (max-min)/2)\t$seuil_1"
        #
        # ALGO DU SEUIL num 1 - fin
        
        # debug
        $p insert end "\nMoyenne (ne sert pas a l'algo du seuil)\t[expr {round($moy)}]"
        
        
        # ALGO DU SEUIL num 2 - ... le premier maximum - debut
        #
        set seuil_2 [seuil_2 $stat_rgb]
        # debug
        $p insert end "\n---------------------------------------------------------------algo 2 : seuil\t$seuil_2\n"
        # debug - fin
        #
        # ALGO DU SEUIL num 2 - fin
        # Il faudrait garder tous les maxiumms, avec le pixel correspondant, pour revenir au deuxieme, troisieme, etc. max
        # au fur et a mesure que la methode verte flanche ... YES !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        
        # Correction si le seuil est superieur a la moyenne (tres peu de contraste, ou nervure plus claire que le fond)
        if {$seuil_2 > $moy} {set seuil_2 [expr {int($moy)}]}
        if {$seuil_1 > $moy} {set seuil_1 [expr {int($moy-1)}]}
        
        # Les deux seuils seront utilises dans la procedure TO_Z_O, mais la on revient au SAMPLING
        return [list $seuil_1 $seuil_2]
    }
    #
    # Procedure TRESHOLD - end
    
    
    
    
    # PROCEDURE seuil_1
    #
    # appelee par la procedure TRESHOLD (situee ci-dessus)
    #
    proc seuil_1 {stat_rgb} {
        #
        # 'stat_rgb' contient une liste de binomes - valeur pixel - freq pixel -
        # c'est la meme liste que 'pix-freq' traitee dans seuil_2
        #
        global p landmark nocontrast rgb_min
        # Valeurs ordonnees par les frequences R=G=B, afin de faire ensuite la diff entre max et min
        set ordre [lsort -real -index 1 $stat_rgb]
        # On retire les accolades
        regsub -all {\{||\}} $ordre "" ordre
        # On range R=G=B et sa freq dans des var separees (rgb et qte); 'qte' ne sert a rien, pour le moment
        foreach {hip hop} $ordre {append rgb "$hip "; append qte "$hop "}
        # Valeurs R=G=B min et max
        set rgb_min [lindex [lsort -real $rgb] 0]
        set rgb_max [lindex [lsort -real $rgb] end]
        # S'il n'y a pas de contraste, alerte
        if {$rgb_min == $rgb_max} {
            set nocontrast [expr {$nocontrast + 1}]
            tk_messageBox -message "No contrast for LM $landmark" -icon warning -type ok
        }
        # seuil = min + (max-min)/2)
        #
        set seuil [expr {$rgb_min + ($rgb_max-$rgb_min)/2}]
        return $seuil
        
    }
    #
    # PROCEDURE seuil_1
    
    
    # PROCEDURE seuil_2
    #
    # appelee par la procedure TRESHOLD (ci-dessus)
    #
    proc seuil_2 {pix_freq} {
        
        # Bug a corriger: si 2 candidats, les valeurs de pixels sont dans l'ordre decroissant ! A FAIRE A FAIRE A FAIRE
        
        #
        # 'pix-freq' contient une liste de binomes - valeur pixel - freq pixel -
        # c'est la meme liste que 'stat_rgb' traitee dans seuil_1
        #
        
        global p landmark candidats nocontrast
        # 'candidats' liste des pixels de freq max
        # On retire les accolades
        regsub -all {\{||\}} $pix_freq "" pix_freq
        
        set pix_nb [expr {[llength $pix_freq]/2}]
        #debug
        #$p insert end "\n\nALGO 2 :\t$pix_freq et nbre diff de r=g=b $pix_nb "
        # attention, que se passe-t-il si pas de 'candidats' ???
        set candidats ""
        set candidature ""
        set candidature_autre "";# pour micrograal_X_candidats
        set descente 0
        for {set i 3} {$i < [expr {$pix_nb*2}]} {incr i 2} {
            set a [lindex $pix_freq $i]
            set b [lindex $pix_freq [expr {$i-2}]]
            if {$a >= $b} {
                set descente 0
            } else {
                if {$descente == 0} {
                    append candidature "[lindex $pix_freq [expr {$i-3}]] "
                    lappend candidature_autre "[lindex $pix_freq [expr {$i-3}]] [lindex $pix_freq [expr {$i-2}]]"
                    set descente 1
                }
            }
        }
        if {[lindex $pix_freq [expr {($pix_nb*2)-3}]] < [lindex $pix_freq [expr {($pix_nb*2)-1}]]} {
            append candidature "[lindex $pix_freq [expr {($pix_nb * 2)-2}]] "
            lappend candidature_autre "[lindex $pix_freq [expr {($pix_nb * 2)-2}]] [lindex $pix_freq [expr {($pix_nb * 2)-1}]]"
        }
        
        # S'il n'y a qu'un maximum, ou si ce maximum n'a pas ete detecte (pas de baisse)
        # cela veut dire que le contraste est insuffisant, on est probablement dans une zone homogene, informe
        # et dans ce cas il vaut mieux agrandir la fenetre
        if {[llength $candidature] < 2} {
            # un lindex avec valeur negative ne genere pas d'erreur
            # Si ce maximum n'est pas le dernier pixel
            if {[lindex $pix_freq [expr {($pix_nb*2)-3}]] < [lindex $pix_freq [expr {($pix_nb*2)-1}]]} {
                append candidats "[lindex $pix_freq [expr {($pix_nb * 2)-4}]] "
            }
        }
        
        if {[llength $candidature] > 1} {
            set candidats_nb [expr {[llength $candidature]/2}]
            set compteur 0
            foreach item $candidature {
                incr compteur
                if {$compteur <= $candidats_nb} {append candidats "$item "} else {break}
            }
        }
        
        #debug
        #$p insert end "\nProcedure seuil_2: candidats $candidats"
        if {$candidats != ""} {
            set choix [lindex $candidats 0]
        } else  {
            set choix ""
        }
        return $choix
    }
    #
    # PROCEDURE seuil_2
    
    
    
    
    
    # Procedure TO_Z_0 - start
    #
    # "0" et "1" separent le bruit du signal ... Cette procedure est critique; en fait ce qui est critique c'est le choix du seuil.
    # Elle est apelee par
    #
    proc to_z_o {vector_table seuil} {
        global p
        set seuil_1 [lindex $seuil 0];# je ne l'utilise que si seuil_2 est vide
        set seuil_2 [lindex $seuil 1]
        # Mise a zero initiale
        set zero_un ""
        # Values to zero un - Attention, l'algo 2 peut ne pas generer de seuil_2 !
        if {$seuil_2 != ""} {
            foreach line [split $vector_table "\n"] {
                foreach item $line {
                    if {$seuil_2 > 2} {
                        if {$item <= $seuil_2} {
                            append zero_un "1 "
                        } else {
                            append zero_un "0 "
                        }
                    } else  {
                        if {$item == $seuil_2} {
                            append zero_un "1 "
                        } else {
                            append zero_un "0 "
                        }
                    }
                }
                append zero_un "\n"
            }
        } else  {
            foreach line [split $vector_table "\n"] {
                foreach item $line {
                    if {$item <= $seuil_1} {
                        append zero_un "1 "
                    } else {
                        append zero_un "0 "
                    }
                }
                append zero_un "\n"
            }
        }
        return $zero_un
    }
    #
    # Procedure TO_Z_0 - end
    
    
    
    
    # Procedure CB_RC - start
    #
    # une liste de liste en 'curved brackets' transformee en 'retour charriot'
    #
    proc cb_rc {cb} {
        foreach item $cb {
            foreach truc $item {
                append rc "$truc "
            }
            append rc "\n"
        }
        return $rc
    }
    #
    # Procedure CB_RC - end
    
    
    # Procedure CB_RC - start
    #
    # une liste de liste en 'retour charriot' transformee en 'curved brackets'
    #
    proc rc_cb {rc} {
        foreach item [split $rc "\n"] {if {$item != ""} {lappend cb $item}}
        return $cb
    }
    #
    # Procedure CB_RC - end
    
    # Procedure MENANT_AUX_CHOIX - debut
    #
    proc menant_aux_choix {vector_propre lm} {
        
        global p
        
        # Pour poser un choix, cette procedure fait appel aux procedures 'bloc_stat' + 'cibles' (+ 'future target') ##### ##### #####
        #
        # Appel a la procedure BLOC_STAT, etat des lieux en ce qui concerne les monoblocs
        #
        # Cette procedure renvoie deux listes (lignes et colonnes)
        #
        set lecture [bloc_stat $vector_propre $lm]
        set local_mono_lig [lindex $lecture 0]
        set local_mono_col [lindex $lecture 1]
        #
        # Presentation tres importante - debut
        $p insert end "\n-------------------------------------------"
        $p insert end "\nLignes et Colonnes pourvues d'un monobloc :"
        $p insert end "\nLignes:\t\t"
        foreach item $local_mono_lig {$p insert end "[lindex $item 0] "}
        $p insert end "\nColonnes:\t"
        foreach item $local_mono_col {$p insert end "[lindex $item 0] "}
        $p insert end "\n-------------------------------------------"
        $p insert end "\nlocal_mono_lig \n$local_mono_lig \nlocal_mono_col \n$local_mono_col"
        $p insert end "\n-------------------------------------------"
        # Presentation tres importante - fin
        
        # Appel a la procedure CIBLES qui va nous rapporter en principe deux choix possibles,
        #     l'un deduit de l'etude des lignes
        #     l'autre deduit de celle des colonnes
        set choix_plusieurs [cibles $local_mono_lig $local_mono_col $lm]
        #
        return $choix_plusieurs
    }
    #
    # Procedure MENANT_AUX_CHOIX - fin
    
    
    # --------------------------------------------------------------------------------------
    #
    # Procedure POSITION - debut
    #
    proc position {vector xy lm} {
        global p
        #$p insert end "\nxy = $xy pour lm $lm"
        set pos ""
        set vector [rc_cb $vector]
        set compteur 0
        #$p insert end "\nPosition :  [lindex $vector $xy]"
        foreach item [lindex $vector $xy] {
            if {$item == 1} {append pos "$compteur "}
            incr compteur
        }
        return $pos
    }
    #
    # Procedure POSITION - fin
    
    # Procedure LMOY - debut
    #
    proc lmoy {liste} {
        set l [llength $liste]
        set m [expr {[lsum $liste] * 1.0 / $l}]
        return $m
    }
    #
    # Procedure LMOY - fin
    
    
    
    
    # Procedure TARGET - debut
    #
    proc target {ll_rc choix_aal lm} {
        #
        # Attention: il ne faut pas comptabiliser les non-monoblocs A FAIRE A FAIRE A FAIRE Prendre le plus long MOINS les zeros ... ?
        #
        global p
        
        set lig [lindex $choix_aal 0]
        set col [lindex $choix_aal 1]
        
        set liste_col "$col "
        set liste_lig "$lig "
        
        $p insert end "\nTARGET\nLa ligne choisie $lig et la colonne choisie $col"
        
        set lignes [position $ll_rc $lig $lm]
        set t_ll_rc [vector_transp $ll_rc]
        set colonnes  [position $t_ll_rc $col $lm]
        $p insert end "\nTARGET\nLa ligne choisie $lig passe sur les colonnes suivantes:\t$lignes \
                \nLa colonne choisie $col passe sur les lignes suivantes :\t$colonnes"
        
        set pref_lig ""
        set pref_col ""
        #$p insert end "\nRaisonnement sur la ligne choisie dans la colonne de choix Sur cette ligne, ou sont les (autres) gdes colonnes ? rep/ 5, 6, 7"
        foreach x $lignes {
            if {$x != $col} {
                # Attention: il ne faut pas comptabiliser les non-monoblocs A FAIRE pitet A FAIRE Prendre le plus long 'pitet' MOINS les zeros ... ?
                set pitet [position $t_ll_rc $x $lm]
                #$p insert end "cette colonne \n\
                #$pitet\n\
                #est-elle de dimension acceptable par rapport au choix initial et contient-elle ce choix ?"
                #$p insert end "rapport des tailles [llength $pitet]  / [llength $colonne]"
                set nb_zeros [expr {([lindex $pitet end] - [lindex $pitet 0]) - [llength $pitet] + 1}]
                if {[llength $colonnes] > 0} {
                    
                    if {$nb_zeros < [expr {0.25*[llength $pitet]}]} {
                        set rt [expr { [llength $pitet] * 1.0 / [llength $colonnes]}]
                        set compteur 0
                        foreach item $colonnes {
                            foreach truc $pitet {
                                if {$item == $truc} {incr compteur}
                            }
                        }
                        set ldl [expr {$compteur * 1.0 /  [llength $colonnes]}]
                        if {$ldl > 0.65} {
                            append liste_col "$x "
                            lappend pref_lig "$x $ldl"
                        }
                    }
                }
            }
        }
        
        #$p insert end "\nRaisonnement sur la colonne choisie dans la ligne de choix Sur cette colonne, ou sont les (autres) gdes lignes ? rep/ 2, 3"
        foreach y $colonnes {
            if {$y != $lig} {
                set pitet [position $ll_rc $y $lm]
                #$p insert end "cette ligne est-elle de dimension acceptable par rapport au choix initial et contient-elle ce choix ?"
                #$p insert end "rapport des tailles [expr { [llength $pitet] * 1.0 / [llength $ligne]}]"
                set nb_zeros [expr {([lindex $pitet end] - [lindex $pitet 0]) - [llength $pitet] + 1}]
                if {[llength $lignes] > 0} {
                    if {$nb_zeros < [expr {0.25*[llength $pitet]}]} {
                        set rt [expr { [llength $pitet] * 1.0 / [llength $lignes]}]
                        set compteur 0
                        foreach item $lignes {
                            foreach truc $pitet {
                                if {$item == $truc} {incr compteur}
                            }
                        }
                        set ldl [expr {$compteur * 1.0 /  [llength $lignes]}]
                        if {$ldl > 0.65} {
                            append liste_lig "$y "
                            lappend pref_col "$y $ldl"
                        }
                        
                    }
                }
            }
        }
        
        
        if {$pref_lig == ""} {set pref_lig $col}
        if {$pref_col == ""} {set pref_col $lig}
        
        set lig_pref [lindex [lsort -real -decreasing -index 1 $pref_lig] 0]
        set col_pref [lindex [lsort -real -decreasing -index 1 $pref_col] 0]
        
        set newx  [lindex $col_pref 0]
        set newy  [lindex $lig_pref 0]
        #$p insert end "\nliste lig $liste_lig \ntarget = $newx"
        #$p insert end "\nliste col $liste_col \ntarget = $newy"
        $p insert end "\nProcedure TARGET\tnew lig = $newx\tnew col = $newy"
        
        return [list $newx $newy]
    }
    #
    # Procedure TARGET - fin
    
    
    
    
    
    # Procedure RECADRAGE
    #
    proc recadrage {image min max lm} {
        
        
        global p cote_rect landmark rgb_max wi he
        
        set th [image height $image]
        set tw [image width $image]
        set rec [image create photo -height $th -width $tw]
        set ba [expr {$max - $min}]
        
        # ATTENTION ba peut etre nul => division pr zero !! A FAIRE A FAIRE A FAIRE
        if {$ba > 0} {
            
            for {set tx 0} {$tx < $tw} {incr tx} {
                set pix ""
                for {set ty 0} {$ty < $th} {incr ty} {
                    set x [$image get $tx $ty]
                    set rgb [lindex $x 0]
                    set xa [expr {$rgb - $min}]
                    set xnew [expr {int(255 * ($xa * 1.0 / $ba))}]
                    #debug
                    #$p insert end "b $max \ta $min \tb-a $ba\tx-a $xa\t$x -> $xnew ^^^ \n"
                    # R = G = B = moyenne des pixels
                    lappend pix [
                    format #%02x%02x%02x $xnew $xnew $xnew
                    ]
                    # On rentre les valeurs RGB de chaque pixel de l'image
                    $rec put $pix -to $tx 0
                }
            }
            # On montre l'image contrastee
            img_show $rec $lm
            
            return $rec
        } else  {
            return $image
        }
        
        
    }
    #
    # Procedure RECADRAGE - fin
    
    # Procedure MORMAT MORPHOLOGIE MATHEMATIQUE - debut
    #
    proc mormat {vector_table lm} {
        
        # -----------------------eliminer les '1' isoles et coudre --------------- debut -----------
        #
        global p cote_rect tucol tulig landmark rgb_max
        
        set x [expr {[lindex $cote_rect($lm) 0] * 2}]
        set y [expr {[lindex $cote_rect($lm) 1] * 2}]
        
        # HUME matrix -  liste des listes ('vector_table') transformee en matrice de Hume ('hume_table')
        set hume_table "2 $x $y "
        foreach item $vector_table {if {$item != "\n"} {append hume_table "$item "}}
    }
    #
    # Procedure MORMAT MORPHOLOGIE MATHEMATIQUE - fin
    
    # Procedure BLUEPOINT - debut
    #
    proc bluepoint {vector_table seuillist lm} {
        
        # ----------------------- Methode BLEUE --------------- debut -----------
        #
        global p cote_rect tucol tulig landmark rgb_max rgb_min
        
        set x [expr {[lindex $cote_rect($lm) 0] * 2}]
        set y [expr {[lindex $cote_rect($lm) 1] * 2}]
        
        # HUME matrix -  liste des listes ('vector_table') transformee en matrice de Hume ('hume_table')
        set hume_table "2 $x $y "
        foreach item $vector_table {if {$item != "\n"} {append hume_table "$item "}}
        #
        # Les points bleus se calculent sur la liste des listes transformee en matrice de Hume
        mnorms_br hume_table means_col sigma_col
        
        set hume_table_t [transpose $hume_table]
        mnorms_br hume_table_t means_lig sigma_lig
        
        
        #debug
        #$p insert end "\nMethode BLEUE"
        #$p insert end "\nMEANS_COL\n[show $means_col %8.2f]"
        #$p insert end "\nMEANS_LIG\n[show $means_lig %8.2f]"
        
        
        # CACULER LA VALEUR MOYENNE ET LA VAR DES COLONNES
        #
        set means_col_r [lreplace $means_col 2 2 1]
        
        mnorms_br means_col_r moy_col var_col
        set moy_col [vtrim $moy_col]
        set var_col [vtrim $var_col]
        #debug
        #$p insert end "\nmoyenne de toutes les colonnes\t$moy_col\variance de toutes les colonnes\t$var_col"
        #
        # CALCULER LA VALEUR MOYENNE ET LA VAR DES LIGNES
        #
        set means_lig_r [lreplace $means_lig 2 2 1]
        
        mnorms_br means_lig_r moy_lig var_lig
        set moy_lig [vtrim $moy_lig]
        set var_lig [vtrim $var_lig]
        #debug
        #$p insert end "\nmoyenne de toutes les lignes\t$moy_lig\variance de toutes les lignes\t$var_lig"
        
        #
        # CONSTRUIRE UNE LISTE {VAR MOY}, LIGNES (varmoy_lig) et COLONNES (varmoy_col)
        #
        # LIGNES
        #
        set v_lig [vtrim $sigma_lig]
        set m_lig [vtrim $means_lig]
        set compteur 0
        set varmoy_lig ""
        foreach item $v_lig {
            lappend varmoy_lig "$item [lindex $m_lig $compteur]"
            incr compteur
        }
        #
        # COLONNES
        #
        set v_col [vtrim $sigma_col]
        set m_col [vtrim $means_col]
        set compteur 0
        set varmoy_col ""
        foreach item $v_col {
            lappend varmoy_col "$item [lindex $m_col $compteur]"
            incr compteur
        }
        
        #
        # # Sans eviter les bords, trouver la position de la moyenne la plus proche du seuil
        #
        set seuil_1 [lindex $seuillist 0]
        set seuil_2 [lindex $seuillist 1]
        if {$seuil_2 != ""} {set seuil $seuil_2} {set seuil $seuil_1}
        set seuil [expr {abs($rgb_min - $moy_col)*1.0/2}]
        #
        # COLONNES
        #
        set compteur 0
        set diff_col ""
        foreach item $varmoy_col {
            incr compteur
            set m [lindex $item 1]
            set diff [expr {abs($m - $seuil)}]
            lappend diff_col "$diff $compteur"
        }
        #debug
        #$p insert end "diff_col (moy - seuil) *** \n$diff_col"
        set diff_col_ordre [lsort -index 0 -real $diff_col]
        #$p insert end "\nMethode BLEUE, suite: diff_col_ordre (moy - seuil)\n$diff_col_ordre\n"
        foreach item $diff_col_ordre {
            #debug
            #$p insert end "\n$item <--  [lindex $item 1]"
            set hors_bord [lindex $item 1]
            set tucol $hors_bord
            break
        }
        if {[info exists tucol] < 1} {tk_messageBox  -message "Meth. bleue: des embrouilles dans les col" -type ok}
        
        #
        # LIGNES
        #
        set compteur 0
        set diff_lig ""
        foreach item $varmoy_lig {
            incr compteur
            set m [lindex $item 1]
            set diff [expr {abs($m - $seuil)}]
            lappend diff_lig "$diff $compteur"
        }
        #debug
        #$p insert end "\ndiff_lig (moy - seuil) ### \n$diff_lig"
        set diff_lig_ordre [lsort -index 0 -real $diff_lig]
        #$p insert end "\nMethode BLEUE, suite: diff_lig_ordre (moy - seuil)\n$diff_lig_ordre\n"
        foreach item $diff_lig_ordre {
            set hors_bord [lindex $item 1]
            set tulig $hors_bord
            break
        }
        if {[info exists tulig] < 1} {tk_messageBox  -message "Meth. bleue: des embrouilles dans les lig" -type ok}
        
        #$p insert end "\nMethode BLEUE, suite et fin : x = $tulig et y = $tucol\n"
        # Choisir la moy la moins eloignee de seuil_2 et eviter les bords
        #
        #
        #
        #debug
        #$p insert end "\nMATRIX hume_table\n"
        #$p insert end "Colonnes hume_table\n[show $means_col %8.1f]\t$moy_col\n[show $sigma_col %8.1f]\n"
        #$p insert end "Lignes   hume_table\n[show $means_lig %8.1f]\t$moy_lig\n[show $sigma_lig %8.1f]\n"
        
        #------------------ Methode BLEUE --------------- fin -----------
        #
        return [list $tulig $tucol]
    }
    #
    # procedure BLUEPOINT
    
    # Procedure DECOUSU - start
    #
    # examine l'etat de la couture: combien de zeros inseres dans les blocs de '1' ?
    #
    proc decousu  {vector lm} {
        
        global p
        
        #$p insert end "\nControle technique sur :\n$vector"
        
        set sum_uns_lig 0
        set sum_zeros_lig 0
        set boulier 0
        foreach item [split $vector "\n"] {
            if {[llength $item] > 0} {
                set pitet [position $vector $boulier $lm]
                #puts "$boulier: $item --> $pitet"
                if {[llength $pitet] > 0} {
                    # On ne prend en compte que les pseudo-blocs n'excedant pas la moitie d'un cote <- CONDITION brievement RETIREE LE 13 JAN 2007
                    if {[expr {[lindex $pitet end] - [lindex $pitet 0]}] < [expr {[llength $item] / 2}]} {
                        set nb_zeros [expr {([lindex $pitet end] - [lindex $pitet 0]) - [llength $pitet] + 1}]
                        # On ne prend en compte que les pseudo-blocs dont le nb zeros n'excede pas le nb de '1'
                        if {$nb_zeros <= [lsum $item]} {
                            set sum_uns_lig [expr {$sum_uns_lig + [lsum $item]}]
                            set sum_zeros_lig [expr {$sum_zeros_lig + $nb_zeros}]
                        }
                    }
                    
                }
                incr boulier
            }
        }
        
        if {$sum_uns_lig > 0} {
            set rapport_lig [expr {$sum_zeros_lig * 1.0 / $sum_uns_lig}]
            return $rapport_lig
        } else  {
            set rapport_lig ""
        }
        
    }
    #
    # Procedure DECOUSU - end
    
    
    
    # Procedure COUTURE_101 -start
    #
    # La 'couture' consiste a reunir des blocs positifs (d'une ligne ou d'une colonne) separes par seulement un pixel nul
    # ... donc '1 0 1' devient '1 1 1'
    #
    # Utilite supposee: identifier des blocs mal reveles par le threshold choisi
    #
    proc couture_101 {vector_table lm} {
        #
        # Appelee par la procedure 'pix-bin'
        #
        global p cote_rect
        
        foreach line [split $vector_table "\n"] {
            append vector_table_new "[lindex $line 0] "
            set colonnes [llength $line]
            for {set k 1} {$k < $colonnes} {incr k} {
                set prec [expr {$k-1}]
                set suiv [expr {$k+1}]
                #
                # couture : '1 0 1' devient '1 1 1'
                #
                if {[lindex $line $k] == 0 && [lindex $line $prec] > 0 && [lindex $line $suiv] > 0} {
                    append vector_table_new "1 "
                    #$p insert end "\n\tposition $i: [lindex $ligne $i] = $compteur_uns"
                } else  {
                    append vector_table_new "[lindex $line $k] "
                }
            }
            append vector_table_new "\n"
        }
        
        return $vector_table_new
    }
    #
    # Procedure COUTURE_101 - end
    
    
    
    # Procedure COUTURE_1001 - start
    #
    # La 'couture_1001' consiste a reunir des blocs positifs (d'une ligne ou d'une colonne) separes par deux pixels nuls
    # ... donc '1 0 0 1' devient '1 1 1 1'
    #
    # Utilite supposee: identifier des blocs mal reveles par le threshold choisi
    #
    proc couture_1001 {vector_table lm} {
        #
        # Appelee par la procedure CRUCE
        #
        global p cote_rect
        
        foreach line [split $vector_table "\n"] {
            
            set colonnes [llength $line]
            for {set k 0} {$k < $colonnes} {incr k} {
                set prec [expr {$k-1}]
                set suiv [expr {$k+1}]
                set apres [expr {$k+2}]
                #
                # reperage des "0 0"
                #
                if {[lindex $line $k] == 0 && [lindex $line $suiv] == 0 &&  [lindex $line $apres] > 0 &&  [lindex $line $prec] > 0} {
                    append vector_table_new "1 1 "
                    incr k
                } else  {
                    append vector_table_new "[lindex $line $k] "
                }
            }
            append vector_table_new "\n"
        }
        return $vector_table_new
    }
    #
    # Procedure COUTURE_1001 - end
    
    
    
    
    # Procedure BLOC -start
    #
    # que fait cette procedure ?
    # Pour chaque liste (en horiz et en vertic) de pixels r=g=b, 'BLOC' compte les blocs de "1", et compte les "1" dans ces blocs
    #
    #
    # Rem: un '1' isole n'est pas pris en compte pour le denombrement des blocs
    #
    proc bloc {ligne colonnes} {
        #
        #la procedure BLOC est appelee par la procedure UN_ZERO
        #
        # La variable 'ligne' est une liste qui contient une suite de r=g=b
        # La variable 'colonnes' correspond au nombre de pixels de cette ligne
        global p
        set total_uns 0
        set total_null 0
        set compteur_uns 0
        set compteur_null 0
        set liste_uns ""
        set local_uns ""
        
        
        # On parcourt la ligne indiquee par la procedure 'un_zero' (la procedure qui appelle 'bloc')
        #
        for {set i 1} {$i < $colonnes} {incr i} {
            set j [expr {$i - 1}];# position precedente
            #debug puts
            #puts "ON EXAMINE: [lindex $ligne $j] et [lindex $ligne $i] qui se suivent dans la ligne $ligne"
            # si " 1 1 " se suivent, le bloc se construit
            if {[lindex $ligne $i] > 0 && [lindex $ligne $j] > 0} {
                incr total_uns
                #debug puts
                #puts "incr total_uns --> $total_uns"
                incr compteur_uns
                #debug puts
                #puts "incr compteur_uns --> $compteur_uns"
                set compteur_null 0
                append liste_uns "$compteur_uns "
                #debug puts
                #puts "append liste_uns 'compteur_uns' ---> $liste_uns"
                if {$i == 1} {
                    #incr total_uns
                    #incr compteur_uns
                    #set compteur_null 0
                    #append liste_uns "$compteur_uns "
                    append local_uns "$j $i "
                    #debug puts
                    #puts "if i == 1 append local_uns 'j i' ---> $local_uns"
                }
                if {$i > 1} {
                    if {[lindex $ligne [expr {$j-1}]] == 0} {append local_uns "$j $i "} else {append local_uns "$i "}
                }
                if {$i == [expr {$colonnes-1}]} {
                    incr total_uns
                    incr compteur_uns
                    append liste_uns "$compteur_uns "
                }
                #debug puts
                #puts "\n\
                #\t1 1 aux positions $j et $i : compteur_uns: $compteur_uns et total: $total_uns\
                #        \tLa liste des '1' devient $liste_uns et elle est localisee selon $local_uns"
            }
            ## si " 1 0 " une fin de bloc est supposee (si i > 1), le compteur des uns est remis a zero,
            if {[lindex $ligne $i] == 0 && [lindex $ligne $j] == 1} {
                if {$i > 1 || $i == [expr {$colonnes-1}]} {
                    if {[lindex $ligne [expr {$j-1}]] == 1} {
                        incr total_uns
                        incr compteur_uns
                        append liste_uns "$compteur_uns "
                        set compteur_uns 0
                    }
                }
            }
            ## si " 0 0 " se suivent, un autre bloc est suppose, le compteur des uns est remis a zero,
            if {[lindex $ligne $i] == 0 && [lindex $ligne $j] == 0} {
                
                incr compteur_null
                set compteur_uns 0
                append liste_null "$compteur_null "
                #puts "\n\tposition $i: [lindex $ligne $i] = $compteur_null"
            }
        }
        #debug puts
        #if {$total_uns > 0} {
        #    puts "BLOC : liste des '1' $liste_uns Total des '1' $total_uns \tLocalisation des '1' $local_uns\n"
        #}
        
        return [list $liste_uns $total_uns $local_uns]
    }
    #
    # Procedure BLOC - stop
    
    
    
    # Procedure UN_ZERO - debut
    #
    #
    # qui appelle cette procedure ?
    # Reponse: la procedure CLASSEMENT
    #
    proc un_zero {vector_table} {
        
        global p
        
        set ligne -1
        set blocs_doc ""
        #
        # On va parcourir chaque liste du vector_table
        #
        foreach line [split $vector_table "\n"] {
            incr ligne
            #$p insert end "\n---------------------------------------------\nline $line\n---------------------------------------------\n"
            set colonnes [llength $line]
            set compteur_ligne 1
            set depart $compteur_ligne
            
            # Appel a la procedure BLOC ... (ci-dessus)
            # Cette derniere classe chaque rectangle en nb de blocs et qte de 1 (horiz et vertic)
            #
            set examen [bloc $line $colonnes]
            #
            # La liste retournee ('examen') contient 3 listes :
            # _ les '1'
            # _ le nbre total de '1' de la ligne
            # _ et les positions correspondantes aux '1' des blocs (localisation du loc)
            
            if {[lindex $examen 1] > 0} {
                #debug
                #$p insert end "Procedure un_zero\tLigne $ligne: 'examen' \t$examen"
                #debug important
                #$p insert end "\n------------------\nLigne $ligne"
            };# {$examen (blocs de 1 et NOMBRE de uns)}
            
            # Reccueil des 3 listes dans 3 variables differentes
            #
            set blocs_liste [lindex $examen 0]
            set total_uns [lindex $examen 1]
            set blocs_local  [lindex $examen 2]
            #
            
            # --------------------------
            # On compte les blocs de "1" a partir de 'bloc_liste'
            #
            set bloc 0
            for {set i 0} {$i < $total_uns} {incr i} {
                #debug
                #$p insert end "\ni = $i: Element $i de $blocs_liste soit : [lindex $blocs_liste $i]"
                if {[lindex $blocs_liste $i] == 1} {
                    incr bloc
                }
            }
            #
            # fin du comptage des blocs
            # ---------------------------
            
            if {[lindex $examen 1] > 0} {}
            
            # -----------------------------------------
            # On compte les elements "1" de chaque bloc
            #
            set segment 0
            set compteur 0
            set bloc_accu ""
            set lignes_doc ""
            #
            for {set i 1} {$i < [expr {$total_uns+1}]} {incr i} {
                if {[lindex $blocs_liste $i] == 1} {
                    incr compteur
                    set suite($compteur) [expr {$i-$segment}]
                    set segment [expr {$segment + $suite($compteur)}]
                    #debug important
                    #$p insert end "\nbloc $compteur $suite($compteur)"
                    append bloc_accu "$suite($compteur) "
                }
                if {$i == [expr {$total_uns}]} {
                    incr compteur
                    set suite($compteur) [expr {$i-$segment}]
                    set segment [expr {$segment + $suite($compteur)}]
                    #debug important
                    #$p insert end "\nbloc-fin $compteur $suite($compteur)"
                    append bloc_accu "$suite($compteur) "
                }
                
            };# fin du compte des elements de chaque bloc
            # --------------------------------------------
            
            #debug
            #$p insert end "\nnb total de pixels + = $segment"
            
            
            # 'lines_doc' contient (pos 0) num ligne (pos 1) nb blocs (pos suiv) nb elem / bloc (pos end) nb total elem
            append lignes_doc "$ligne $compteur $bloc_accu $segment "
            if {[lindex $lignes_doc 1] > 0} {
                #debug
                #$p insert end "\n--- $lignes_doc"
                lappend blocs_doc [list $lignes_doc [list $ligne $blocs_local]]
            }
            
        };# fin des lignes du vecteur (line vector)
        return $blocs_doc
    }
    #
    # Procedure UN_ZERO - fin
    
    
    
    # Procedure CROIX - debut
    #
    proc croix {local_mono_dabord local_mono_dapres lm} {
        #
        # appelee 2 FOIS par la procedure CRUCE (laquelle appelle d'abord d'autres procedures: to_z_o, couture, couture_1001 et mono)
        #
        global p cote_rect  long_bloc_dabord long_bloc_dapres
        
        set monoblocs ""
        #debug
        # local_mono_dabord et dapres contiennent toutes les lignes et en deuxieme item les positions des elements du bloc sur la ligne
        #$p insert end "\nProcedure croix: local_mono_dabord \n $local_mono_dabord \nProcedure croix: local_mono_dapres \n $local_mono_dapres"
        # 'dabord' et 'dapres' en fonction de l'ordre des arguments de l'appel a procedure: 'lig col' ou 'col lig'
        foreach item $local_mono_dabord {
            set local_dabord [lindex $item 0];# devrait simplement contenir le numero de la ligne
            set ldabord [lindex $item 1];;# devrait contenir les positions y des elements du bloc sur la ligne examinee
            #$p insert end "\nitem $item \tlocal_dabord $local_dabord"
            foreach truc $local_mono_dapres {
                # Attention, il y a une symetrie inversee des noms de variable
                set local_dapres [lindex $truc 1];# devrait contenir les positions x des elements du bloc sur la colonne examinee
                set ldapres [lindex $truc 0];#devrait contenir le numero de la colonne
                #$p insert end "\ntruc $truc \tlocal_col $local_col"
                foreach pos $local_dapres {
                    #$p insert end "\npos = local : $pos = $local ?"
                    if {$pos == $local_dabord} {
                        # A ce stade, le couple n'est que candidat, il faut encore verifier si ligne et colonne se croisent effectivement
                        foreach y $ldabord {
                            if {$y == $ldapres} {
                                set choix_dabord $local_dabord
                                set choix_dapres [lindex $truc 0]
                                set long_bloc_dapres [llength $local_dapres]
                                set long_bloc_dabord [llength [lindex $item 1]]
                                $p insert end "\nlongueur des blocs $long_bloc_dabord et $long_bloc_dapres"
                                #au lieu de break, faudra dire append, puis choisir judicieusement
                                break
                            }
                        }
                    }
                    if {[info exists choix_dabord ]} {break}
                }
                if {[info exists choix_dabord ]} {break}
            }
            if {[info exists choix_dabord ]} {break}
        }
        
        if {[info exists choix_dabord]} {
            return [list $choix_dabord $choix_dapres]
        } else  {
            set choix_dabord "none"
            set choix_dapres "none"
            return [list $choix_dabord $choix_dapres]
        }
    }
    #
    # procedure CROIX - fin
    
    
    # Procedure MIROIR - start
    #
    # que fait cette procedure ?
    # Elle retourne une image en miroir d'une table presentee sous forme de 'vector_table' (une liste de listes)
    #
    proc miroir {vector_table 1} {
        
        proc dagobert {ligne} {
            set ligne_new ""
            foreach item [split $ligne] {
                if {$item != ""} {set ligne_new [linsert $ligne_new 0 $item]}
            }
            return $ligne_new
        }
        #
        #la procedure MIROIR contient la procedure DAGOBERT et est appelee par la procedure ...
        #
        global p
        set vector_table_new ""
        # On parcourt la ligne indiquee par la procedure 'un_zero' (la procedure qui appelle 'bloc')
        #
        foreach item [split $vector_table "\n"] {
            if {$item != ""} {
                set ligne_inverse [dagobert $item]
                append vector_table_new "$ligne_inverse  \n"
            }
        }
        return $vector_table_new
    }
    #
    # Procedure MIROIR - fin
    
    
    # Procedure DIAGONALE - start
    #
    # que fait cette procedure ?
    # Elle retourne le triangle superieur gauche qui partage en deux une table carree presentee sous forme de 'vector_table' (une liste de listes)
    #
    proc diagonale {vector_table 1} {
        
        #puts "vector_table \n$vector_table"
        foreach item [split $vector_table "\n"] {
            if {$item != ""} {lappend vector_table_new $item}
        }
        #puts "vector_table_new \n$vector_table_new"
        set colonnes [llength [lindex $vector_table_new 0]]
        #puts "colonnes $colonnes"
        
        for {set i 0} {$i < [expr {$colonnes+1}]} {incr i} {
            append piste "$i "
        }
        #puts "piste $piste"
        
        set piste_inverse [dagobert $piste]
        
        for {set i 0} {$i < $colonnes} {incr i} {
            #puts "i = $i"
            set a [lindex $vector_table_new $i]
            
            set boutdepiste [lrange $piste 0 $i]
            set boutdepiste [dagobert $boutdepiste]
            for {set j 0} {$j < [lindex $piste [expr {$i+1}]]} {incr j} {
                #puts "j = $j"
                set hrz [lindex $vector_table_new $j]
                #puts "y = [lindex $boutdepiste $j]"
                set y [lindex $boutdepiste $j]
                set vert [lindex [lindex $vector_table_new $j] $y]
                #puts "extraction de $vert"
                append triangle "$vert "
            }
            append triangle "\n"
        }
        #puts "triangle\n$triangle"
        return $triangle
    }
    #
    # Procedure DIAGONALE - fin
    
    
    # procedure TRANSPOSITION D'UNE TABLE EN FORME DE LISTE DE LISTES - debut
    #
    proc vector_transp {vector} {
        #
        # Je transpose 'vector' (format rc) dans une nouvelle 'vector_table' pour utiliser sur les colonnes les memes procedures que sur les lignes
        #
        global p
        #$p insert end "\nProcedure vector_transp: vector RECU \n$vector"
        set colonnes_compte 0
        set compteur 0
        foreach line [split $vector "\n"] {
            if {$compteur < 1} {
                foreach item $line {incr colonnes_compte}
            }
            incr compteur
        }
        set lignes_compte 0
        foreach line [split $vector "\n"] {
            if {[llength $line] == $colonnes_compte} {incr lignes_compte}
        }
        
        set lignes 0
        set colonnes 0
        foreach line [split $vector "\n"] {
            if {$line != ""} {set colonnes 0}
            foreach item $line {
                set table($lignes,$colonnes) $item
                incr colonnes
            }
            #if {$line != ""} {incr lignes} else {incr lignes -1}
            if {$line != ""} {incr lignes}
        }
        
        #$p insert end "colonnes $colonnes_compte \nlignes $lignes_compte"
        for {set k 0} {$k < $colonnes_compte} {incr k} {
            for {set j 0} {$j < $lignes_compte} {incr j} {
                append vector_table_transp "$table($j,$k) "
            }
            append vector_table_transp "\n"
        }
        #$p insert end "\nProcedure vector_transp: vector CONCU \n$vector_table_transp"
        return $vector_table_transp
    }
    #
    # procedure TRANSPOSITION D'UNE TABLE EN FORME DE LISTE DE LISTES - fin
    
    
    # ------------------------------------------------------------------------
    proc lsum {liste} {
        set sum 0
        foreach item $liste {
            set sum [expr {$sum + $item}]
        }
        return $sum
    }
    # ------------------------------------------------------------------------
    proc tranches {vector lm} {
        global cote_rect
        set tot [lindex $cote_rect($lm) 0]
        set longueur [expr {$tot * 2}]
        for {set i 0} {$i < $longueur} {incr i} {
            append remplacement "0 "
        }
        set vector [rc_cb $vector]
        for {set i 1} {$i < $longueur} {incr i} {
            puts "$i:\t[lindex $vector $i] \tsomme des '1' [lsum [lindex $vector $i]]"
            if {[lsum [lindex $vector $i]] == 0} {append liste_elem_zeros "$i "}
        }
        if {[info exists liste_elem_zeros] < 1} {return $vector}
        set nb_elem_zeros [llength $liste_elem_zeros]
        set diff [expr {[lindex $liste_elem_zeros end] - [lindex $liste_elem_zeros 0] + 1}]
        
        # On essaie de 'voir' si c'est un bloc de lignes, et d'en connaitre la taille
        if {$nb_elem_zeros == $diff} {
            set monobloc_zeros 1
        } else {
            if {$diff < [expr {$nb_elem_zeros * 2}] && $diff < $tot} {
                set monobloc_zeros 1
            } else  {
                set monobloc_zeros 0
            }
        }
        
        
        #puts "liste_elem_zeros $liste_elem_zeros\tnb_elem_zeros = $nb_elem_zeros\tmonobloc_zeros $monobloc_zeros"
        # S'il y a un seul bloc de lignes nulles
        if {$monobloc_zeros == 1} {
            # On remplace par en haut (ou a gauche si la fenetre est transposee) si la prem ligne nulle est dans la prem moitie de la fenetre
            if {[lindex $liste_elem_zeros 0] < $tot} {
                for {set i 0} {$i < [lindex $liste_elem_zeros 0]} {incr i} {
                    set vector [lreplace $vector $i $i $remplacement]
                }
            } else  {
                for {set i [lindex $liste_elem_zeros end]} {$i < $longueur} {incr i} {
                    set vector [lreplace $vector $i $i $remplacement]
                }
            }
        } else  {
            # S'il y a plusieurs de ces blocs ou des lignes nulles dispersees, on elimine le haut, puis on elimine la bas
            for {set i 0} {$i < [lindex $liste_elem_zeros 0]} {incr i} {
                set vector [lreplace $vector $i $i $remplacement]
            }
            for {set i [lindex $liste_elem_zeros end]} {$i < $longueur} {incr i} {
                set vector [lreplace $vector $i $i $remplacement]
            }
        }
        return $vector
    }
    
    
    # ------------------------------------------------------------------------
    
    # Proc MONO - fin
    #
    proc mono {vector lm} {
        
        #
        # appele par la procedure ..., la procedure MONO va faire appel 2 FOIS a la procedure CLASSEMENT
        #
        
        global p cote_rect nobibloc onretaille
        
        set monoblocs ""
        set pluribloc ""
        set onretaille 0
        
        set gch [lindex $cote_rect($lm) 0]
        set dt  [lindex $cote_rect($lm) 0]
        set ht  [lindex $cote_rect($lm) 1]
        set bas  [lindex $cote_rect($lm) 1]
        set col_max [expr {$gch+$dt}]
        set lig_max [expr {$bas+$ht}]
        
        
        # Classement des lignes - debut - PROCEDURE classement
        #
        set choix [classement $vector $lig_max]
        #
        set lignes_mono_lig [lindex $choix 0]
        set local_mono_lig [lindex $choix 1]
        
        lappend monoblocs $local_mono_lig
        append pluribloc "$nobibloc "
        
        #debug
        #$p insert end "\nProcedure MONO local_mono_lig\n$local_mono_lig\n"
        # Classement des lignes - fin
        
        
        # Transposition - debut
        #
        # On transpose la table pour proceder aux memes operations, mais sur les colonnes
        set vector_table_transp [vector_transp $vector]
        #
        # Transposition - fin
        
        # Classement des colonnes - debut - PROCEDURE classement
        #
        set choix [classement $vector_table_transp $col_max]
        #
        set lignes_mono_col [lindex $choix 0]
        set local_mono_col [lindex $choix 1]
        
        lappend monoblocs $local_mono_col
        
        append pluribloc "$nobibloc "
        
        if {[lindex $pluribloc 0] == 1 && [lindex $pluribloc 1] == 1} {
            set nobibloc 1
            # et dans ce cas il vaut mieux retailler la fenetre que de changer de seuil
        } else {
            set nobibloc 0
        }
        #
        # Classement des colonnes - fin
        
        return $monoblocs
    }
    #
    # Proc MONO - fin
    
    
    
    
    # Proc CLASSEMENT - debut
    #
    proc classement  {vector_table max} {
        #
        # appelee deux fois par la procedure MONO (une fois pour les lignes, une fois pour les colonnes)
        #
        global p nobibloc onretaille
        
        set onretaille 0
        # Connaitre la definition des lignes en termes de blocs
        set pro_doc [un_zero $vector_table]
        #debug
        #$p insert end "\n\n==============="
        #$p insert end "\npro DOC == $pro_doc"
        
        set doc ""
        set fit ""
        foreach element $pro_doc {
            lappend doc [lindex $element 0];# ligne - nbbloc - elbloc - tot
            lappend fit [lindex $element 1];# ligne - localisation du bloc
            #debug
            #$p insert end "\n$element\t<- $doc \t<-> $fit"
        }
        # Statistiques sur ces blocs et leur contenu
        set doc_ordre_nbbloc [lsort -real -index 1 $doc]
        set doc_ordre_nbelem [lsort -real -decreasing -index end $doc]
        #debug
        #$p insert end "\nLignes ordonnees par nbre de blocs\nProcedure classement: doc_ordre_nbbloc = $doc_ordre_nbbloc\n"
        #$p insert end "\n"
        #$p insert end "\nLignes ordonnees par nbre TOTAL d'elem (ordre decroissant) \n$doc_ordre_nbelem"
        #$p insert end "\n==============="
        
        set nobibloc 0
        set lignes_mono ""
        set lignes_pluri ""
        foreach item $doc_ordre_nbbloc {
            if {[lindex $item 1] == 1} {
                lappend lignes_mono $item
            } else  {
                lappend lignes_pluri $item
            }
        }
        set lignes_mono [lsort -real -decreasing -index 2 $lignes_mono]
        #debug
        #$p insert end "\nLignes mono bloc ordonnees par nombre decroissant d'elements / bloc\n$lignes_mono"
        
        set local_mono ""
        foreach item $lignes_mono {
            foreach truc $fit {
                if {[lindex $truc 0] == [lindex $item 0]} {
                    lappend local_mono $truc
                }
            }
        }
        # Si une seule veine passe dans le rectangle, ne pas accepter la solution
        
        if {$lignes_pluri == ""} {
            set nobibloc 1
            # et dans ce cas, il vaut mieux retailler la fenetre que de changer de seuil
        }
        $p insert end "\nPLURI blocs \n$lignes_pluri , sur un total de [llength $doc_ordre_nbbloc] listes \nnobibloc = $nobibloc\n"
        
        return [list $lignes_mono $local_mono]
    }
    #
    # Procedure CLASSEMENT - fin
    
    
    
    # Procedure PIX_BIN -debut
    #
    #
    #
    proc pix_bin {vector_cousu seuil lm} {
        
        #
        # appelee par la procedure SAMPLING, PIX_BIN fera appel a la procedure COUTURE_1001
        #
        global p candidats long_bloc_dabord long_bloc_dapres onretaille cote_rect
        set onretaille 0
        #
        #1 0 0 1 -> 1 1 1 1 - PROCEDURE couture_1001
        set vector_recousu [couture_1001 $vector_cousu $lm]
        #debug
        $p insert end "\nLM $lm Apres couture_1001 sur les lignes SIGNAL / BRUIT = [sb $vector_recousu $lm]\n$vector_recousu"
        #
        
        set controle_technique [decousu [vector_transp $vector_recousu] $lm]
        $p insert end "\nLM $lm Apres couture_1001 sur les lignes, on passe a couture_1001 COLONNES \
                si le controle technique sur les colonnes > 0.02 (ici, c_t = $controle_technique)"
        
        if {$controle_technique > 0.02} {
            
            if {[sb $vector_recousu $lm] <= 0.35} {
                $p insert end "\nOn passe a 1001 COLONNES, d'accord, mais a condition que le rapport signal_fond < 0.35: \
                        rapport signal_fond [sb $vector_recousu $lm]"
                
                # On transpose la table pour proceder aux memes operations, mais sur les colonnes - PROCEDURE vector_transp
                set vector_recousu_transp [vector_transp $vector_recousu]
                # 1 0 0 1 -> 1 1 1 1  sur les colonnes a present
                set vector_recousu2fois [couture_1001 $vector_recousu_transp $lm]
                # On transpose a nouveau la table pour restituer son orientation initiale
                set vector_cousu [vector_transp $vector_recousu2fois]
                #debug
                $p insert end "\nLM $lm Apres couture_1001 sur les colonnes SIGNAL/fond = [sb $vector_cousu $lm]\n$vector_cousu"
            }
        }
        
        return $vector_cousu
    }
    
    
    # Procedure BLOC_STAT
    #
    proc bloc_stat {vector_cousu lm} {
        
        # Appel a la procedure MONO
        #
        # Les lignes et colonnes qui ne contiennent qu'un seul bloc sont identifiees, et classees par importance du bloc
        #
        set monoblocs [mono $vector_cousu $lm]
        #
        set local_mono_lig [lindex $monoblocs 0]
        
        set local_mono_col [lindex $monoblocs 1]
        
        return [list $local_mono_lig $local_mono_col]
    }
    #
    # Procedure BLOC_STAT
    
    proc cibles {local_mono_lig local_mono_col lm} {
        global p
        # Croisements de monoblocs, en deux questions complementaires
        #     - le plus grand monobloc en ligne croise avec quel plus grand monobloc en colonne ?
        #     - le plus grand monobloc en colonne croise avec quel plus grand monobloc en ligne ?
        set choix_plusieurs ""
        #
        # On commence l'examen lignes d'abord - appel a la procedure CROIX
        #
        set croisement [croix $local_mono_lig $local_mono_col $lm]
        #$p insert end "\nCROISEMENT en commencant la lecture par les lignes $croisement"
        set prem_choix_lig [lindex $croisement 0]
        lappend choix_plusieurs $prem_choix_lig
        set prem_choix_col [lindex $croisement 1]
        lappend choix_plusieurs $prem_choix_col
        #$p insert end "\nprem_choix_lig $prem_choix_lig et prem_choix_col $prem_choix_col et enfin choix_plusieurs $choix_plusieurs"
        #
        # Les colonnes ensuite - deuxieme appel a la procedure CROIX
        #
        set croisement [croix $local_mono_col $local_mono_lig $lm]
        #$p insert end "\nCROISEMENT en commencant la lecture par les colonnes $croisement"
        set deux_choix_lig [lindex $croisement 1]
        lappend choix_plusieurs $deux_choix_lig
        set deux_choix_col [lindex $croisement 0]
        lappend choix_plusieurs $deux_choix_col
        #
        #$p insert end "\ncibles\t$choix_plusieurs\n"
        #set choix_plusieurs [list $prem_choix_lig $prem_choix_col $deux_choix_lig $deux_choix_col]
        
        return $choix_plusieurs
    }
    #
    # Procedure CIBLE
    
    
    # Procedure CONFIG_RECTAILLE - debut
    #
    proc config_rectaille {} {
        
        set typeList {
            {{"Text files"} {.seq .txt .nts .TXT .NTS .csv .CSV .tps .TPS} }
        }
        global infile_rectaille dossier_rectaille texte references
        
        # Chercher la route dans un fichier de configuration
        # 1. verifier que le fichier de configuration existe
        set route "[pwd]/dossier_rectaille.txt"
        #puts "route $route"
        # 2. s'il n'existe pas, voir ci-dessus
        if {[file exists $route] == 0} {
            set candidat_dossier_rectaille [pwd]
        } else  {
            #puts "La conf existe"
            # 3. s'il existe reccueillir la derniere route parcourue
            # open le fichier pour lire
            catch [
            set fid [open $route r]
            set candidat_dossier_rectaille [read -nonewline $fid]
            close $fid
            ] ici
            #puts "candidat_dossier $candidat_dossier"
            set candidat_dossier_rectaille [lindex [split $candidat_dossier_rectaille \n] 1]
            # debug
            #puts "dossier_rectaille = $candidat_dossier_rectaille"
        }
        
        # On va chercher les donnees
        #catch [
        set infile_rectaille [tk_getOpenFile -initialdir $candidat_dossier_rectaille -filetypes $typeList]
        set fid [open $infile_rectaille]
        set texte [read -nonewline $fid]
        close $fid
        #] ici
        
        #infile contient la route et le nom du fichier des donnees
        set dossier_rectaille "[file dirname $infile_rectaille]/";#repertoire des donnees
        # 4 si ce n'est pas la route indiquee par candidat_dossier_rectaille, il faut reecrire le fichier de configuration
        if {$dossier_rectaille != $candidat_dossier_rectaille} {
            # open le fichier de conf pour ecrire
            catch [
            set fid [open $route w+]
            puts -nonewline $fid "Dernier dossier de donnees utilisees par RECTAILLE:\n"
            puts -nonewline $fid $dossier_rectaille
            close $fid
            ] ici
        }
        #debug
        #puts "route = $dirinfile"
        set tailfile [file tail $infile_rectaille];#fichier nu, avec l'extension
        #debug
        #puts "\ntail = $tailfile"
        set rootfile [file rootname $tailfile];#fichier nu, sans l'extension
        #debug
        #puts "root = $rootfile"
        
        return $texte
    }
    #
    # procedure CONFIG_RECTAILLE - fin
    
    
    # Procedure DISCUSSION
    #
    proc discussion {prem_choix_lig prem_choix_col deux_choix_lig deux_choix_col nocontrast nobibloc onretaille lm} {
        
        global p cote_rect candidats vector_bckup solutions
        
        set solutions 0
        
        # ATTENTION _ c qui suit ne doit pas necessairement se trouver ici
        set gch [lindex $cote_rect($lm) 0]
        set dt  [lindex $cote_rect($lm) 0]
        set ht  [lindex $cote_rect($lm) 1]
        set bas  [lindex $cote_rect($lm) 1]
        set col_max [expr {$gch+$dt}]
        set lig_max [expr {$bas+$ht}]
        # ATTENTION
        
        if {$prem_choix_lig == "none" || $prem_choix_col == "none" || $deux_choix_lig == "none" || $deux_choix_col == "none"\
                    || $nocontrast == 1 || $nobibloc == 1 || $onretaille == 1} {
            $p insert end "\nLM lm : echec de la methode verte au premier choix :\
                    prem_choix_lig = $prem_choix_lig etc. nocontrast = $nocontrast nobibloc = $nobibloc onretaille = $onretaille"
            return "none none"
        } else  {
            # Les deux choix doivent etre identiques <- NON, ils doivent etre dans la meme zone
            #debug
            $p insert end "\nPremier seuil choisi: les deux choix doivent etre les memes sans quoi on annule ces choix \
                    \t$prem_choix_lig ~= $deux_choix_lig  E T  $prem_choix_col ~= $deux_choix_col ??\n"
            set diff_x [expr {abs($prem_choix_lig - $deux_choix_lig)}]
            set diff_y [expr {abs($prem_choix_col - $deux_choix_col)}]
            if {$diff_x < 5 && $diff_y < 5} {
                set solutions 1
                return [list $prem_choix_lig $prem_choix_col $deux_choix_lig $deux_choix_col]
            } else  {
                $p insert end "\nLM lm : Methode verte, PROBLEME : 2 solutions differentes: $prem_choix_lig $prem_choix_col $deux_choix_lig $deux_choix_col"
                set solutions 2
                return "none none"
            }
        }
        #debug - fin
        
        # Un croisement putatif a peut-etre ete trouve, mais ...
        # si la quantite de signal vaut moins que deux cotes, il vaut mieux elargir un peu la fenetre - C'EST VRAI CELA ?
        # ou alors si le bloc n'est pas bibloc, cela entraine automatiqueent onretaille = 1 ce qui annule les choix
        #
        if {$onretaille == 1} {
            $p insert end "\nonretaille ... == $onretaille, donc ELARGIR LE RECTANGLE D'ECHANTILLONNAGE\n"
            set prem_choix_lig "none"
            set prem_choix_col "none"
            set deux_choix_lig "none"
            set deux_choix_col "none"
            return [list $prem_choix_lig $prem_choix_col $deux_choix_lig $deux_choix_col]
        }
        #
        
        # ------------------------------------------------------------------------------------------------------------
        #
        #                          d i s c u s s i o n                              f i n
        #
        # ------------------------------------------------------------------------------------------------------------
        set retour [list $prem_choix_lig $prem_choix_col $deux_choix_lig $deux_choix_col]
        return $retour
    }
    #
    # Procedure DISCUSSION - end
    
    
    
    # Procedure SAMPLING - debut
    #
    # La procedure SAMPLING est appelee par la boucle 'D'un LM a l'autre' du script principal,
    #  - la procedure est munie d'une taille de fenetre (cfr. argument 'taillerect')
    #  - mais pas d'un seuil signal / fond, qui doit etre calcule ici (cfr. liste 'candidats')
    #
    # La procedure SAMPLING fera appel successivement aux procedures suivantes: DECOUPAGE, BIN2GRAY, RECADRAGE, ONCALCVEC, THRESHOLD, BLUEPOINT, PIX_BIN.
    #
    proc sampling {taillerect lm} {
        
        global p wi he ho ha landmark plaquex plaquey rgb_max
        global lmixe lmigr color taill cote_rect
        global candidats
        global tucol tulig defect
        global nocontrast nobibloc long_bloc_dabord long_bloc_dapres onretaille
        global vector_bckup vector_transit
        global solutions
        #
        set i $lm
        set nocontrast 0
        set nobibloc 0
        set onretaille 0
        
        # Les valeurs (approximatives si predites) du LM sont rentrees dans x et y
        set x $lmixe($i)
        set y $lmigr($i)
        
        # On DECOUPE un rectangle autour du landmark (approximatif, eventuellement) - Procedure DECOUPAGE
        #
        # 17 dec 2006: ce rectangle pourrait etre DIFFERENT pour chaque LM, DESSINE par l'UTILISATEUR !! cfr procedure USERECT config_rectaille
        #
        set dec($i) [decoupage $i $x $y $taillerect]
        
        # TRANSFORMATION EN GRIS de chaque rectangle - Procedure BIN2GRAY
        #
        set res [bin2gray $dec($i) 1 $i]
        $dec($i) copy $res
        
        # Recherche des valeurs extremes de pixels - debut
        set th [image height $dec($i)]
        set tw [image width $dec($i)]
        for {set tx 0} {$tx < $tw} {incr tx} {
            for {set ty 0} {$ty < $th} {incr ty} {
                append pix_list "[$dec($i) get $tx $ty] "
            }
        }
        #debug
        #$p insert end "\npix_list\n$pix_list\n"
        set pix_min [lindex [lsort -real $pix_list] 0]
        set pix_max [lindex [lsort -real $pix_list] end]
        # Recherche des valeurs extremes de pixels - fin
        
        # Augmenter le contraste - Procedure RECADRAGE - n'est sans doute pas vraiment necessaire
        set rec [recadrage $dec($i) $pix_min $pix_max $i]
        $dec($i) copy $rec
        
        # Recuperation de la valeur unique R=G=B dans vector - Procedure ONCALCVEC
        set wi [image width $dec($i)]
        set he [image height $dec($i)]
        set vector($i) [oncalcvec $dec($i) $wi $he]
        
        # Recherche du SEUIL, ou threshold - Procedure THRESHOLD, qui, prudente, renvoie une liste de seuils ... !
        $p insert end "\nLM $i - - - \n$vector($i)"
        set seuil($i) [treshold $vector($i) $i];# ici, seuil($i) contient seuil_1 et seuil_2, la liste se trouve dans la variable globale 'candidats'
        $p insert end "\nLM $i : seuil $seuil($i) , et candidats = $candidats\timage width $wi\n"
        
        # Methode bleue sur les pixels non binarises - on la garde ou pas ? Elle produit tulig et tucol (variables globales)
        set methode_bleue [bluepoint $vector($i) $seuil($i) $i]
        set vector_transit [to_z_o $vector($i) $seuil($i)]
        
        set automatic "1 5 10 15 20 25 30 35 40 45 50 55"
        foreach item $automatic {
            set candidats [linsert $candidats "$item " 0]
        }
        
        
        if {$candidats != ""} {
            
            foreach umbral $candidats {
                
                set seuil($i) $umbral
                #debug
                $p insert end "\nUMBRAL UMBRAL UMBRAL UMBRAL\t LM $i : umbral ou seuil $umbral\n"
                
                # Depart du classement base sur la premiere proposition de seuil
                set vector_bckup $vector($i)
                # Methode bleue sur les pixels non binarises - on la garde ou pas ? Elle produit tulig et tucol (variables globales)
                set methode_bleue [bluepoint $vector($i) $seuil($i) $i]
                # Methode anciennement appelee 'methode verte', plusieurs procedures:  TO_Z_O PIX_BIN, BLOC_STAT, CIBLES
                #
                # Utilisation du seuil pour separer le bruit du signal - Procedure TO_Z_O
                set vector_table [to_z_o $vector($i) $seuil($i)]
                #debug ---
                $p insert end "\nLM $i vector_table avant nettoyage\tseuil $umbral\tfenetre $cote_rect($lm)\
                        \n$vector_table"
                
                # Appel inconditionnel a la procedure TRANCHES: effacement de possibles taches ...
                # (On devrait pouvoir aussi au moins detecter des taches obliques: si +, onretaille = 1 ?)
                # - # Effacement en haut ou en bas
                set vertical [tranches $vector_table $i]
                set vector_table_propre($i) [cb_rc $vertical]
                # - # Effacement a gauche ou a droite
                set toubet $vector_table_propre($i)
                set horizontal [vector_transp $toubet]
                set bascule [tranches $horizontal $i]
                set ttoubet [cb_rc $bascule]
                set toubet [vector_transp $ttoubet]
                set vector_table_propre($i) $toubet
                # Fin du nettoyage de la fenetre
                
                set signal_fond [sb $vector_table_propre($i) $lm]
                #debug ---
                $p insert end "\nLM $i premieres infos apres nettoyage\tseuil $umbral\tfenetre $cote_rect($lm)\tSIGNAL/BRUIT $signal_fond\
                        \n$vector_table_propre($i)"
                
                # Appel inconditionnel a la couture des lignes : 1 0 1 - > 1 1 1
                #
                set vector_cousu [couture_101 $vector_table_propre($i)  $i]
                $p insert end "\nLM $i couture 101 des lignes inconditionnelle\n$vector_cousu"
                
                set controle_technique [decousu [vector_transp $vector_cousu] $i]
                $p insert end "\ncontrole_technique $controle_technique; on passe a couture_101 COLONNES si le controle sur les colonnes donne un pourcentage > 0.09"
                
                if {$controle_technique > 0.09} {
                    # Appel sous condition a la couture plus serree des colonnes
                    set signal_fond [sb $vector_cousu $lm]
                    $p insert end "\nOn passe a 101 COLONNES, d'accord, mais a condiion que le rapport signal_fond < 0.35: rapport signal_fond $signal_fond"
                    if {$signal_fond < 0.35} {
                        set vector_cousu_transp [vector_transp $vector_cousu]
                        set vector_cousu2fois [couture_101 $vector_cousu_transp $i]
                        # On transpose a nouveau la table pour restituer son orientation initiale
                        set vector_table_propre_cousu($i) [vector_transp $vector_cousu2fois]
                        
                        set signal_fond [sb $vector_table_propre_cousu($i) $lm]
                        $p insert end "\nLM $i signal/fonds apres couture_101 des colonnes = $signal_fond\n$vector_table_propre_cousu($i)"
                        
                        set vector_cousu $vector_table_propre_cousu($i)
                        if {$signal_fond < 0.35} {
                            # Appel a la procedure PIX_BIN, qui renvoie un vector 'cousu' 1001 -> 1111 si autorise)
                            #
                            set vector_cousu [pix_bin $vector_cousu $seuil($i) $i]
                        }
                    }
                }
                
                set vector_transit $vector_cousu
                
                # Tentative de choix ---------------------------------
                #
                #
                set yesorno [oui_ou_non $vector_cousu $umbral $signal_fond  $lm]
                #debug
                $p insert end "\nyesorno = $yesorno sur vector_cousu, LM $i : seuil $umbral, fenetre $cote_rect($i)\n"
                #
                #
                #
                # Examen de la tentative ------------------------------
                #
                if {$yesorno != "none none"} {
                    set retour $yesorno
                    break
                } else {
                    set retour $yesorno
                };# fin de la trois verif yesorno
                #set retour "none none"
            };# fin de foreach umbral
        } else  {
            set retour "none none"
        }
        return $retour
    };# fin de la procedure sampling
    
    
    # ---------------------------------------------------------------------------------------
    #
    # SCRIPT SOURCE PRINCIPAL de la superprocedure MICROGRAAL
    #
    # ---------------------------------------------------------------------------------------
    
    global wi he;# largeur et hauteur de la grande image
    global ho ha;# 'ho' et 'ha' le chgt de coord si la gde image est bougee ?
    global landmark
    global plaquex plaquey;# contiennent listes des tags (points colores) correspondant aux LM
    global plaque_aal
    global nocontrast nobibloc
    global rgb_max
    global candidats;# contient la liste des seuils possibles
    global cote_rect
    global tucol tulig
    global defect
    global lmixe lmigr
    global color
    global taill
    global vector_transit; #permet la procedure TARGET
    
    # Les valeurs des pixels r=g=b sont non modifiees (0 a 255)
    if {[info exists rgb_max] < 1} {set rgb_max 3.1875}
    
    set nocontrast 0;# to record and count uncontrasted images
    set nobibloc 0;# to record image without crossing veins
    
    # On efface ce qui a deja ete ecrit sur la grande image
    #$p insert end "\nPLAQUEX \n$plaquex \nPLAQUEY \n$plaquey"
    #foreach tagitem $plaquex {
    #    .can delete $tagitem
    #}
    #foreach tagitem $plaquey {
    #    .can delete $tagitem
    #}
    
    foreach item $coord_obs {
        append coord_predites "[expr {int($item)}] "
    }
    #$p insert end "\n**** coord_obs ****\n$coord_obs\n**** coord_predites ****\n$coord_predites\n"
    
    # Nombre total de LM
    set lm_nb [expr {[llength $coord_predites] * 1/2}]
    
    # les valeurs predites (ou observees si micrograal au lieu de mini) sont placees dans des variables indicees: lmixe et lmigr
    set compteur 0
    for {set i 1} {$i < [expr {$lm_nb + 1}]} {incr i} {
        set a [lindex $coord_predites $compteur]
        set b [lindex $coord_predites [expr {$compteur+1}]]
        set lmixe($i) $a
        set lmigr($i) $b
        # Dessin du landmark sur la grande image - n'est pas necessaire a l'usage final
        #set ixe $a
        #set igr $b
        #.can create oval [expr {$ixe-3}] [expr {$igr-3}] [expr {$ixe+3}] [expr {$igr+3}] -fill yellow
        # Dessin du landmark sur la grande image -fin
        incr compteur 2
        
    }
    
    # (on est toujours dans le SCRIPT PRINCIPAL de la superprocedure)
    
    # couleur et taille du futur LM calcule
    set color "green"
    set taill "3"
    
    # ATTENTION _ ce qui precede ne doit pas necessairement se trouver ici
    # On se prepare
    set coord_aal ""
    set plaque_aal ""
    # ------------------------------
    #
    # SAMPLING d'un LM a l'autre -- start --- (on est toujours dans le SCRIPT PRINCIPAL de la superprocedure MINIGRAAL)
    #
    # ------------------------------
    for {set i 1} {$i < [expr {$lm_nb + 1}]} {incr i 1} {
        set landmark $i
        # Les valeurs approximatives du LM sont rentrees dans x et y - (on est dans ' d'un LM a l'autre ')
        set x $lmixe($i)
        set y $lmigr($i)
        
        # La taille de la fenetre d'echantillonnage se decide ici; le seuil se definit et, eventuellement se change,\
        # a l'interieur d'une taille de fenetre definie ici (cfr sampling 'foreach item candidats etc.')
        set liste_rect "12 24 32 40"
        foreach echantillon $liste_rect {
            set cote_rect($i) "$echantillon $echantillon"
            $p insert end "\n\n\n\n\n\n\n\n\n\nLM $i \nFenetre actuelle $cote_rect($i)\n----------------------------\n"
            set choix_aal [sampling $cote_rect($i) $i]
            if {[string first "none" $choix_aal] < 0} {
                $p insert end "\nCHOIX AAL = $choix_aal, la fenetre : $cote_rect($i)\n"
                break
            }
            $p insert end "\nComme CHOIX AAL = $choix_aal, on se RETAILLE un rectangle\n"
        }
        
        # Si choix_aal est vide, choisir tucol tulig (methode bleue)
        # On passe aussi a un autre nom de variable
        if {[string first "none" $choix_aal] > -1} {
            set choix_aal "$tulig $tucol"
            #il faut ensuite affiner le tir, se mettre si possible dans le centre geometrique du croisement de deux nervures
            # c'est le travil de la procedure TARGET
            #set choix_aal [target $vector_transit $choix_aal $i]
            set choix_lig($i) [lindex $choix_aal 0]
            set choix_col($i) [lindex $choix_aal 1]
        } else  {
            #il faut ensuite affiner le tir, se mettre si possible dans le centre geometrique du croisement de deux nervures
            # c'est le travil de la procedure TARGET
            #set choix_aal [target $vector_transit $choix_aal $i]
            set choix_lig($i) [lindex $choix_aal 0]
            set choix_col($i) [lindex $choix_aal 1]
        }
        
        #Si la distance est trop gde avec le choix de l'utilisateur ...
        set gch [lindex $cote_rect($i) 0]
        set dt  [lindex $cote_rect($i) 0]
        set ht  [lindex $cote_rect($i) 1]
        set bas  [lindex $cote_rect($i) 1]
        set col_max [expr {$gch+$dt}]
        set lig_max [expr {$bas+$ht}]
        #debug
        $p insert end "\nset ixe expr {lmixe($i) - (lmixe($i) - gch + choix_col($i) + 1)}\
                = set ixe expr '$lmixe($i) - ($lmixe($i) - $gch + $choix_col($i) + 1)'"
        set ixe [expr {$lmixe($i) - ($lmixe($i) - $gch + $choix_col($i) + 1)}]
        set igr [expr {$lmigr($i) - ($lmigr($i) - $ht + $choix_lig($i))}]
        set distanciation($i) [expr {hypot($ixe,$igr)}]
        if {$distanciation($i) > 13} {
            set color red
        } else  {
            set color green
        }
        # Nouveau
        set tagger "lm_aal_"
        set xaal $tagger$i
        set yaal $tagger$i
        #
        # Dessin du landmark sur la grande image
        set ixe [expr {$lmixe($i) - $gch + $choix_col($i) + 1}]
        set igr [expr {$lmigr($i) - $ht + $choix_lig($i)}]
        .can create oval [expr {$ixe - $taill}] [expr {$igr - $taill}] [expr {$ixe + $taill}] [expr {$igr + $taill}] -fill $color -tag $tagger$i
        .can create text [expr {$ixe-3}] [expr {$igr-8}] -fill $color -text "$i";# -tag $yaal
        # Dessin des rectangles sur la grande image
        set cote [lindex $cote_rect($i) 0]
        .can create rectangle [expr {$lmixe($i)-$cote}] [expr {$lmigr($i)-$cote}] [expr {$lmixe($i)+$cote}] [expr {$lmigr($i)+$cote}] -outline yellow -tag rect
        # Pour l'enregistrement des coordonnees produites par micrograal
        append plaque_aal "$tagger$i "
        #$p insert end "PLAQUE EN COURS $plaque_aal"
        append coord_aal "$ixe\t$igr\n"
    }
    # # ------------------------------
    #
    # D'un LM a l'autre -- end ---
    #
    # # ------------------------------
    
    # (on est toujours dans le SCRIPT PRINCIPAL de la superprocedure MICROGRAAL)
    #
    # AAL prend le risque ou se retire
    #
    
    #$p insert end "\n\n------------------------------------------"
    #$p insert end "\n-   P  E  R  F  O  R  M  A  N  C  E  S   -"
    #$p insert end "\n------------------------------------------"
    set somme 0
    for {set i 1} {$i <  [expr {$lm_nb + 1}]} {incr i} {
        set somme [expr {$somme + $distanciation($i)}]
        #$p insert end "\nLM $i: $distanciation($i)"
    }
    set acceptabilite [expr {$somme / $i}]
    if {$acceptabilite > 7} {
        tk_messageBox -message "Please make manual corrections" -icon warning -type ok
    }
    if {$nocontrast >  [expr {$lm_nb / 4}]} {
        tk_messageBox -message "The image lacks of contrast and should not be retained" -icon warning -type ok
    }
    set defect $acceptabilite
    $p insert end "\n------------------------------------------"
    $p insert end "\nLM aal avant lm_move"
    $p insert end "\n$coord_aal"
    $p insert end "\n------------------------------------------"
    
    # PREFERENCES
    
    # Pour habiliter les deux fonctions on/off du bouton lm_move (frout.rb5)
    set S(lm_move) 1
    
    
    # A FAIRE A FAIRE A FAIRE
    #Comment puis-je, à partir des noms des tags, connaitre leurs coordonnees sur le canvas ?
    $p insert end "\nPlaque_aal: $plaque_aal\n"
    foreach item $plaque_aal {
        $p insert end "\n$item : [.can coords $item]"
    }
    
    
    return $coord_aal
}

