
# Procedure PROCRUSTES
#
# Calcul de la distance de Procrustes entre deux jeux de coordonnées BRUTES (procr_mdonnees)
#
proc procrustes {procr_mdonnees} {
    #
    # procr_mdonnees contient une matrice HUME de deux lignes (individus) et 'lm' colonnes
    #
    global p colcount tablind reference rot rot2 reference reference2
    global nombre lm nombre ts_inconnu_file rotUnknown rotInconnu
    #
    #------------------ script de la procedure PROCRUSTES -------------- # ---- commence après les sub procedures
    #
    # 5 sub procedures: _translation _scaling _rotation_ distance _composantes
    
    # Sous-Procedure Procrustes_translation_1_1
    #
    proc Procrustes_translation_1_1 {lm} {
        global  p grandp imoinsp rawindividus effectifs_mog tablind colcount translatedx numero nombre
        set lignes 2
        # Matrices I, P, I-P
        set identite [mident $lm]
        set unsurp [expr {1.0/$lm}]
        set grandp "2 $lm $lm"
        for {set i 0} {$i < [expr {$lm*$lm}]} {incr i} {
            append grandp " $unsurp"
        }
        set imoinsp [msub $identite $grandp]
        # Translation ou pre-multiplication des matrices par I-P
        global translatedx
        for  {set i 1} {$i < [expr {$lignes+1}]} {incr i} {
            set x "2 $lm 2 $tablind($i)"
            set translatedx($i) [mmult $imoinsp $x]
        }
    }
    #
    # Sous-Procedure Procrustes_translation_1_1
    
    
    # Sous-Procedure Procrustes_scaling_1_1
    #
    proc Procrustes_scaling_1_1 {lm} {
        global p imoinsp rawindividus translatedx transcaledindiv centroide
        global numero nombre preindividus effectifs_mog centroid ts_inconnu ts_inconnu_file
        set consensus 0
        set centroide ""
        set lignes 2
        set suit_cons 0
        # UNIT CTR Scaling to unit centroid size
        for  {set i 1} {$i < [expr {$lignes+1}]} {incr i} {
            set ttranslatedx($i) [transpose $translatedx($i)]
            set prectr [mmult $translatedx($i) $ttranslatedx($i)]
            set proctr [mmult $prectr $imoinsp]
            set presquectr [transpose $proctr]
            # La taille centroide elevee au carre est la trace de presquectr
            set prectr2 [vdiag $presquectr]
            set proctr2 [vtrim $prectr2]
            set ctr2 0
            foreach item $proctr2 {set ctr2 [expr {$ctr2+$item}]}
            set ctr [expr {sqrt($ctr2)}]
            set centroid($i) $ctr
            set scalefactor [expr {1.00000/$ctr}]
            set transcaledindiv($i) [mscale $ttranslatedx($i) $scalefactor]
            set transcaledindiv($i) [transpose $transcaledindiv($i)]
            append centroide "$ctr\n"
        }
    }
    #
    # Sous-Procedure Procrustes_scaling_1_1
    
    
    # Sous-Procedure rotation_1_1
    #
    proc Procrustes_rotation_1_1 {} {
        global p transcaledindiv colonnes lm effectifs_mog consensus rotInconnu
        global infile reference reference2 rot rot2 p inconnu_nb
        set lignes 2
        set consensus 0
        set V2 [composantes $transcaledindiv(2)]
        set prex2t [mmult $transcaledindiv(2) $V2 ]
        set reference2 $prex2t
        set x1t [transpose $prex2t]
        # et donc x1t deviendrait la transposee non du consensus des deux individus
        # mais de l'individu 2 - qui est le consensus du groupe, dans ce programme ...
        # La distance est calculée directement entre les deux configurations
        # et non entre chaque configuration et le consensus des deux
        for  {set i 1} {$i < [expr {$lignes+1}]} {incr i} {
            set ttrantran($i) [mmult $x1t $transcaledindiv($i)]
            # singular value decomposition
            set U $ttrantran($i)
            msvd_br U S V
            set S "2 2 2 1 0 0 1"
            set H [mmult [mmult $V $S] $U]
            set rot($i) [mmult $transcaledindiv($i) $H]
        }
        update
    }
    #
    # Sous-Procedure Procrustes_rotation_1_1
    
    
    # Sous-Procedure composantes
    #
    proc composantes {pop} {
        global p suit_rot pxpu
        set lignes [lindex $pop 1]
        set colonnes [lindex $pop 2]
        set obsminusone [expr {$lignes - 1}]
        mnorms_br pop popmeans popsigmas
        set popones "2 1 $colonnes 1"
        set x 1
        set y $colonnes
        while {$x < $y} {lappend popones 1; set x [expr {$x + 1}]}
        set Z [mnormalize $pop $popmeans $popones]
        set Z [mnormalize $pop $popmeans $popones]
        set ZZ [mmult [transpose $Z] $Z]
        set nless1 [expr {1./$obsminusone}]
        set vcm [mscale $ZZ $nless1]
        set V $vcm
        mevsvd_br V evals
        return $V
    }
    #
    # Sous-Procedure composantes
    
    # Script de la procédure principale --------------- debut --------------- partie 1
    # 
    set procr_matinit [lrange $procr_mdonnees 0 2]
    set colonnes [lindex $procr_matinit 2]
    set lignes [lindex $procr_matinit 1]
    set lm [expr {$colonnes/2}]
    #
    # calcul du consensus (configuration moyenne)
    set consensus [mnorms $procr_mdonnees]
    set consensus [lrange $consensus 3 [expr {3+$colonnes-1}]]
    set donnees [lrange $procr_mdonnees 3 end]
    set data ""
    for  {set i 0} {$i < [expr {$lignes*$colonnes}]} {incr i $colonnes} {
        set tranche [lrange $donnees $i [expr {$i+$colonnes-1}]]
        append data "$tranche\n"
    }
    #
    # On classe les coordonnees successives par individu dans la table tablind()
    set i -1
    set compteur 0
    set tablind(0) $consensus
    foreach item [split $data "\n"] {
        if {$item != ""} {
            # On saute la premiere ligne
            incr compteur
            if {$compteur > 0} {
                #verification du nombre de lm de chaque individu
                if  {$colonnes != [llength $item]} {
                    exit
                }
                incr i
                set tablind($compteur) $item
            }
        }
    }
    #
    set lm [expr {$colcount * 1/2}]
    Procrustes_translation_1_1 $lm
    Procrustes_scaling_1_1 $lm
    Procrustes_rotation_1_1
    update
    # A ce stade, il faut calculer les distances entre coordonnees des deux individus
    # ou de chaque individu avec leur consensus (voir les questions posees dans procedure rotation)
    # une procedure pour cela sera appelee Procrustes_distances
    #
    # Script de la procédure principale --------------- fin de la partie 1 -----------------------
    
    # Sous-Procedure Procrustes_distances
    #
    proc Procrustes_distances {} {
        global rot rot2 lm fingerprint nombre p
        set sum 0
        set compteur 0
        for  {set i 3} {$i < [expr {$lm*2+3}]} {incr i 2} {
            global fingerprint
            set a1 [lindex $rot(1) $i]
            set a2 [lindex $rot(2) $i]
            set b1 [lindex $rot(1) [expr {$i+1}]]
            set b2 [lindex $rot(2) [expr {$i+1}]]
            set a1moinsa2 [expr {$a1-$a2}]
            set b1moinsb2 [expr {$b1-$b2}]
            #debug
            $p insert end "\nlandmark [expr {$compteur+1}]\t [format %1.4f $a2] - [format %1.4f $b2] \
                    et [format %1.4f $a1] - [format %1.4f $b1]\n[format %1.4f $a1moinsa2]\
                    et [format %1.4f $b1moinsb2]"
            set d [ expr { pow($a1moinsa2,2) + pow($b1moinsb2,2) } ]
            set sum [expr {$sum + $d}]
            set fingerprint($compteur) $d
            incr compteur
        }
        # On renvoie la distance de Procruste
        return [expr {sqrt($sum)}]
    }
    #
    # Sous-Procedure Procrustes_distances
    
    # Script de la procédure principale ---------- partie 2 ----- la fin
    #
    set distancia [Procrustes_distances]
    # On renvoie la distance de Procruste
    return $distancia
}
#
# Procedure PROCRUSTES

