package require Tk


# Procedure PROCRUSTES
#
# Calcul de la distance de Procrustes entre deux jeux de coordonnées (procr_mdonnees)
#
proc procrustes {procr_mdonnees} {
    #
    global p colcount tablind reference rot rot2 reference reference2
    global nombre lm nombre ts_inconnu_file rotUnknown rotInconnu
    
    #------------------ script de la procedure PROCRUSTES -------------- #

    # Sous-Procedure Procrustes_translation_1_1
    #
    proc Procrustes_translation_1_1 {lm} {
        global  p grandp imoinsp rawindividus effectifs_mog tablind colcount translatedx numero nombre
        set lignes 2
        # Matrices I, P, I-P
        set identite [mident $lm]
        #debug
        $p insert end "\nMatrice identite \n[show $identite]"
        set unsurp [expr {1.0/$lm}]
        set grandp "2 $lm $lm"
        for {set i 0} {$i < [expr {$lm*$lm}]} {incr i} {
            append grandp " $unsurp"
        }
        set imoinsp [msub $identite $grandp]
        # Translation ou pre-multiplication des matrices par I-P
        global translatedx
        for  {set i 1} {$i < [expr {$lignes+1}]} {incr i} {
            set x "2 $lm 2 $tablind($i)"
            set translatedx($i) [mmult $imoinsp $x]
            #debug
            $p insert end "\nProcrustes_translation_1_1\
                    \tTranslated matrix object $i\n [show $translatedx($i) %1.4f]\n"
        }
    }
    #
    # Sous-Procedure Procrustes_translation_1_1
    
    
    # Sous-Procedure Procrustes_scaling_1_1
    #
    proc Procrustes_scaling_1_1 {lm} {
        global p imoinsp rawindividus translatedx transcaledindiv centroide
        global numero nombre preindividus effectifs_mog centroid ts_inconnu ts_inconnu_file
        set consensus 0
        set centroide ""
        set lignes 2
        set suit_cons 0
        # UNIT CTR Scaling to unit centroid size
        for  {set i 1} {$i < [expr {$lignes+1}]} {incr i} {
            set ttranslatedx($i) [transpose $translatedx($i)]
            set prectr [mmult $translatedx($i) $ttranslatedx($i)]
            set proctr [mmult $prectr $imoinsp]
            set presquectr [transpose $proctr]
            # La taille centroide elevee au carre est la trace de presquectr
            set prectr2 [vdiag $presquectr]
            set proctr2 [vtrim $prectr2]
            set ctr2 0
            foreach item $proctr2 {set ctr2 [expr {$ctr2+$item}]}
            set ctr [expr {sqrt($ctr2)}]
            set centroid($i) $ctr
            set scalefactor [expr {1.00000/$ctr}]
            set transcaledindiv($i) [mscale $ttranslatedx($i) $scalefactor]
            set transcaledindiv($i) [transpose $transcaledindiv($i)]
            append centroide "$ctr\n"
        }
        #debug
        $p insert end "\n\ncentroid sizes\n$centroide"
    }
    #
    # Sous-Procedure Procrustes_scaling_1_1
    
    
    # Sous-Procedure rotation_1_1
    #
    proc Procrustes_rotation_1_1 {} {
        global p transcaledindiv colonnes lm effectifs_mog consensus rotInconnu
        global infile reference reference2 rot rot2 p inconnu_nb
        set lignes 2
        set consensus 0
        set V2 [composantes $transcaledindiv(2)]
        set prex2t [mmult $transcaledindiv(2) $V2 ]
        set reference2 $prex2t
        set x1t [transpose $prex2t]
        # et donc x1t deviendrait la transposee non du consensus des deux individus
        # mais de l'individu 2 - qui est le consensus du groupe, dans ce programme ...
        # La distance est calculée directement entre les deux configurations
        # et non entre chaque configuration et le consensus des deux
        for  {set i 1} {$i < [expr {$lignes+1}]} {incr i} {
            set ttrantran($i) [mmult $x1t $transcaledindiv($i)]
            # singular value decomposition
            set U $ttrantran($i)
            msvd_br U S V
            set S "2 2 2 1 0 0 1"
            set H [mmult [mmult $V $S] $U]
            set rot($i) [mmult $transcaledindiv($i) $H]
            #debug
            $p insert end "\n******** rot($i) ******** \n[show $rot($i)]\n"
        }
        update
    }
    #
    # Sous-Procedure Procrustes_rotation_1_1
    
    
    # Sous-Procedure composantes
    #
    proc composantes {pop} {
        global p suit_rot pxpu
        set lignes [lindex $pop 1]
        set colonnes [lindex $pop 2]
        set obsminusone [expr {$lignes - 1}]
        mnorms_br pop popmeans popsigmas
        set popones "2 1 $colonnes 1"
        set x 1
        set y $colonnes
        while {$x < $y} {lappend popones 1; set x [expr {$x + 1}]}
        set Z [mnormalize $pop $popmeans $popones]
        set Z [mnormalize $pop $popmeans $popones]
        set ZZ [mmult [transpose $Z] $Z]
        set nless1 [expr {1./$obsminusone}]
        set vcm [mscale $ZZ $nless1]
        set V $vcm
        mevsvd_br V evals
        return $V
    }
    #
    # Sous-Procedure composantes
    
    # debug
    $p insert end "\nDONNEES INITIALES\n[show $procr_mdonnees %1.3f]"
    set procr_matinit [lrange $procr_mdonnees 0 2]
    set colonnes [lindex $procr_matinit 2]
    set lignes [lindex $procr_matinit 1]
    set lm [expr {$colonnes/2}]
    # debug
    $p insert end "\n\n$lignes\tconfiguraciones\n$lm\tlandmarks\n"
    
    # calcul du consensus (configuration moyenne)
    set consensus [mnorms $procr_mdonnees]
    set consensus [lrange $consensus 3 [expr {3+$colonnes-1}]]
    set donnees [lrange $procr_mdonnees 3 end]
    set data ""
    for  {set i 0} {$i < [expr {$lignes*$colonnes}]} {incr i $colonnes} {
        set tranche [lrange $donnees $i [expr {$i+$colonnes-1}]]
        append data "$tranche\n"
    }
    
    # On classe les coordonnees successives par individu dans la table tablind()
    set i -1
    set compteur 0
    set tablind(0) $consensus
    foreach item [split $data "\n"] {
        if {$item != ""} {
            # On saute la premiere ligne
            incr compteur
            if {$compteur > 0} {
                #verification du nombre de lm de chaque individu
                if  {$colonnes != [llength $item]} {
                    exit
                }
                incr i
                set tablind($compteur) $item
            }
        }
    }
    
    set lm [expr {$colcount * 1/2}]
    Procrustes_translation_1_1 $lm
    Procrustes_scaling_1_1 $lm
    Procrustes_rotation_1_1
    
    update
    
    # A ce stade, il faut calculer les distances entre coordonnees des deux individus
    # ou de chaque individu avec leur consensus (voir les questions posees dans procedure rotation)
    # une procedure pour cela sera appelee Procrustes_distances
    
    # Sous-Procedure Procrustes_distances
    #
    proc Procrustes_distances {} {
        global rot rot2 lm fingerprint nombre p
        set sum 0
        set compteur 0
        for  {set i 3} {$i < [expr {$lm*2+3}]} {incr i 2} {
            global fingerprint
            set a1 [lindex $rot(1) $i]
            set a2 [lindex $rot(2) $i]
            set b1 [lindex $rot(1) [expr {$i+1}]]
            set b2 [lindex $rot(2) [expr {$i+1}]]
            set a1moinsa2 [expr {$a1-$a2}]
            set b1moinsb2 [expr {$b1-$b2}]
            #debug
            $p insert end "\nlandmark [expr {$compteur+1}]\t [format %1.4f $a2] - [format %1.4f $b2] \
                    et [format %1.4f $a1] - [format %1.4f $b1]\n[format %1.4f $a1moinsa2]\
                    et [format %1.4f $b1moinsb2]"
            set d [ expr { pow($a1moinsa2,2) + pow($b1moinsb2,2) } ]
            set sum [expr {$sum + $d}]
            set fingerprint($compteur) $d
            incr compteur
        }
        ##debug
        $p insert end "\n==================\nDistance de Procrustes entre \
                les deux configurations \n[expr {sqrt($sum)}]\n=================="
        return [expr {sqrt($sum)}]
    }
    #
    # Sous-Procedure Procrustes_distances
    
    set distancia [Procrustes_distances]
    return $distancia
}
#
# Procedure PROCRUSTES

# ----------------------------------------------------------------------------------------------------------
# Script
#

# Pour la version DEVELOPPEMENT
if {[file tail [info script]] != "idd"} {
    cd "/home/jepi/mytcl/Developpement/COVlin"
}


set tcl_precision 14

source math/statistics.tcl
source math/math.tcl
source math/optimize.tcl
source math/linalg.tcl
source 101La.tcl
source jpdraw_11_3A.tcl
source ballons.tcl
#
global colcount p
# Envoyer deux jeux de coordonnées à la mesure de la distance de Procrustes
#
#
# Une fenêtre de rapport
set scb [scrollbar .scrollA -command ".debut yview"]
pack $scb -side left -fill y
set p [text .debut -background "light yellow" -font "courier 12" \
        -wrap word -yscrollcommand ".scrollA set"]
pack $p -side right -fill both -expand yes
$p insert end "\nrapport"
#
# Deux jeux de coordonnées
set eluno "2 1 6 1.0 2.0 3.0 4.0 5.0 6.0"
set elotro "2 1 6 1.2 2.9 3.1 4.5 5.3 6.6"
#
set colcount [mcols $eluno]
#
set procr_mdonnees [join_rows $eluno $elotro]
# ----------------------------------------------------------------------------------------------------------

procrustes $procr_mdonnees