#

# Utiliser S(centroide) et fascores pour un graphe de correlation
#
###############################################################################
# Select
# Copyright ©  2003 Dujardin J.-P. dujardin@mpl.ird.fr
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##############################################################################

# La procedure de l'histogramme est incluse

# Note:
# Au debut du programme, mais à la fin du code source, j'envoie l'interpreteur a la procedure 
# 'PRINCIPAL'
# Cette procedure 'PRINCIPAL' fait appel a toutes les autres procedures du script


package require Tk

cd [file dirname [info script]]
#
global S
set S(dirscript) [pwd];#il n'y a pas de "/" a la fin du nom du dossier du script (utiliser "file join etc")
#
cd ..
set S(dirclic) [pwd]
cd $S(dirscript)

set tcl_precision 14

set version "[file tail [info script]] Version 52"

# 30 plus de graphiques
# le MS side est teste sur MS sj (interaction) et non pas sur le MS residual, cfr. Palmer & Strobecq, 1986 p. 406
# 31 short report
# 40 ANOVA for asymmetry corrected for size
# 41 short report shows corrected analysis only if there is correlation with size
# 42 multiple variables (two separate entries)
# 43 fixed bug in the histogram building (cfr vtrim hume)
# 44 le MS indiv est teste sur MS sj  (interaction) et non pas sur le MS residual
# 45 R scripts
# 46 ADAPTED TO ACCEPT OTHER COMBINED SIZE-SHAPE FILES
# 47 loading file was affected by an empty final row; fixed
# 48-50 Single concatenated file of raw coordinates, computing shape FA scores, symm and asymm comp, regressing shape FA scores on size and on size FA scores
# 51 A different set of input files
# 52 Bug in plotting FA scores shape versus size: fixed (CLIC 96-1)

# Source: Sokal & Rohlf, Biometry, p.324
source chull.tcl
source ballons.tcl
source [file join $S(dirclic) "+/jpdraw_15.tcl"]
source [file join $S(dirclic) "+/101La.tcl"]
source [file join $S(dirclic) "+/math/statistics.tcl"]
#source mogdraw.tcl
#tk_messageBox -message "tclplatform(os) $tcl_platform(os)" -type ok
switch $tcl_platform(platform) {
    "unix" {
        global S
        set S(liwiclic) "linclic"
        #set S(hasmac) [tk_messageBox -message "Are you using MacOS (Apple) ?" -type yesno]
        if {$tcl_platform(os) == "Darwin"} {
            set voisin [file join $S(dirclic) "+/L/Neighbor.app" ]
            set S(appel) "open $voisin"
            #
            set njplotte [file join $S(dirclic) "+/L/Njplot.app" ]
            set S(appelnjplotte) "open $njplotte"
	    set S(hasmac) "yes"
        } else {
            set fin [file join $S(dirclic) "+/L"]
            #
            set suf "/neighbor"
            set S(appel) "\[open \"| xterm $fin$suf\"\]"
            #
            set suf "/njplot.linux"
            set S(appelnjplotte) "\[open \"| $fin$suf\"\]"
	    set S(hasmac) "no"
        }
        set S(filename1) [file join $S(dirclic) "+/L/translat.txt"]
        set S(filename2) [file join $S(dirclic) "+/L/treefile"]
        set S(dossierdumod) [file join $S(dirclic) "+" "L" "infile" ]
    }
    "windows" {
        global S
	set S(hasmac) 0
        set S(liwiclic) "winclic"
        set fin "/+/W"
        set suf "/neighbor.exe"
        set S(appel) "exec cmd.exe /c neighbor.exe"
        set suf "/njplot.exe"
        set S(appelnjplotte) "exec cmd.exe /c njplot.exe"
        set S(filename1) [file join $S(dirclic) "+/L/translat.txt"]
        set S(filename2) [file join $S(dirclic) "+/L/outtree"]
        set S(dossierdumod) [file join $S(dirclic) "+" "W" "infile" ]
    }
    "macintosh" {
	debug "Mac"
	set S(hasmac) 1
    }
}

#
# NEWPW - debut de l'espace de nommage
#
# --------------------------------------------------------------
#
# --------------------------------------------------------------
namespace eval newpw {
    
    # Jean-Pierre Dujardin
    # ------------------------------------------------------------
    # Dec 2014 - nouveaux calculs (600 lignes) de PW - from MOG
    #
    
    # Procedure LFORMAT # debut
    #
    proc lformat {ainsi liste} {
	foreach item $liste {
	    set item [format $ainsi $item]
	    lappend fliste $item
	}
	return $fliste
    }
    #
    # Procedure LFORMAT # fin
    
    # Procedure TRANSLATION # debut - NAMESPACE NEWPW
    #
    proc translation {lm} {
	global  p grandp imoinsp rawindividus tablind polygone
	global suit_tran suit_cons nosi suit_rot susnesnoc references
	set susnesnoc 0
	set nosi 0
	set mmm [llength $rawindividus]
	set lignes [expr {$mmm/$lm}]
	if {[info exists objet_cons]} {unset objet_cons}
	set suit_cons 0
	# Matrices I, P, I-P
	set identite [mident $lm]
	set unsurp [expr {1.0/$lm}]
	set grandp "2 $lm $lm"
	for {set i 0} {$i < [expr {$lm*$lm}]} {incr i} {
	    append grandp " $unsurp"
	}
	set imoinsp [msub $identite $grandp]
	# Translation ou pre-multiplication des matrices par I-P
	global translatedx
	#
	for  {set i 0} {$i < [expr {$lignes+1}]} {incr i} {
	    set x "2 $lm 2 $tablind($i)"
	    set translatedx($i) [mmult $imoinsp $x]
	}
    }
    #
    # Procedure TRANSLATION- fin
    
    
    # Procedure SCALING - debut  # debut NAMESPACE NEWPW
    #
    proc scaling {lm} {
	global S p tailles_base references
	global imoinsp rawindividus translatedx transcaledtotal transcaledindiv preindividus
	global suit_scal polygone nosi suit_rot neweff consensus susnesnoc centroid
	set consensus 0
	set susnesnoc 0
	set nosi 0
	#set suit_rot 0
	set mmm [llength $rawindividus]
	set lignes [expr {$mmm/$lm}]
	set suit_cons 0
	# UNIT CTR Scaling to unit centroid size
	for  {set i 0} {$i < [expr {$lignes+1}]} {incr i} {
	    set ttranslatedx($i) [transpose $translatedx($i)]
	    set prectr [mmult $translatedx($i) $ttranslatedx($i)]
	    set proctr [mmult $prectr $imoinsp]
	    set presquectr [transpose $proctr]
	    # La taille centroide elevee au carre est la trace de presquectr
	    set prectr2 [vdiag $presquectr]
	    set proctr2 [vtrim $prectr2]
	    set ctr2 0
	    foreach item $proctr2 {set ctr2 [expr {$ctr2+$item}]}
	    set ctr [expr {sqrt($ctr2)}]
	    set centroid($i) $ctr
	    set scalefactor [expr {1.00000/$ctr}]
	    set transcaledindiv($i) [mscale $ttranslatedx($i) $scalefactor]
	    set transcaledindiv($i) [transpose $transcaledindiv($i)]
	    if {$S(suit_scale) == 0} {
		append transcaledtotal "[lrange $transcaledindiv($i) 3 end]\n"
		append centroide "$ctr\n"
		append S(tailles_base) "$ctr "
	    }
	}
	return $S(tailles_base);# a qui est destine ce return ?
    }
    #
    # Procedure SCALING - fin
    
    
    # Procedure ROTATION - debut  # debut NAMESPACE NEWPW
    #
    proc rotation {} {
	global p transcaledindiv colonnes lm suit_rot suit_cons consensus objet_cons
	global infile reference rot indivmoy polygone rawindividus nosi susnesnoc
	global references
	set mmm [llength $rawindividus]
	set lignes [expr {$mmm/$lm}]
	set consensus 0
	set susnesnoc 1
	set suit_cons 1
	set nosi 0
	set indivmoy 0;#pour ne pas repeter les rapports du bouton jpdraw/moyens
	set V [composantes $transcaledindiv(0)]
	set prex1t [mmult $transcaledindiv(0) $V ]
	# formation des points (individus) du consensus prex1t
	set landmark 0
	for  {set i 3} {$i<[expr {$lm*2+2}]} {incr i 2} {
	    incr landmark
	    set item [lrange $prex1t $i [expr {$i+1}]]
	    if {$item != ""} {lappend objet_cons "2 1 2 $item"}
	    if {[info exists suit_rot] == 0} {append aligned "$item "}
	}
	if {[info exists suit_rot] == 0} {append aligned "\n"}
	# Le consensus dans la variable reference (ici le mot reference est au singulier)
	#debug $suit_rot
	if {[info exists suit_rot] == 0} {set reference $aligned}
	# references au pluriel = la premiere ligne du fichier de donnees
	
	# Rotations proprement dites
	set x1t [transpose $prex1t]
	for  {set i 1} {$i < [expr {$lignes+1}]} {incr i} {
	    set ttrantran($i) [mmult $x1t $transcaledindiv($i)]
	    # singular value decomposition
	    set U $ttrantran($i)
	    msvd_br U S V
	    set S "2 2 2 1 0 0 1"
	    set H [mmult [mmult $V $S] $U]
	    set rot($i) [mmult $transcaledindiv($i) $H]
	    append rotind " [lrange $rot($i) 3 end]";#pour creer une liste des bascules
	}
	return $rotind
    }
    #
    # Procedure ROTATION - fin
    
    
    # Procedure COMPOSANTES - start- NAMESPACE NEWPW
    #
    proc composantes {pop} {
	global p suit_rot pxpu
	set lignes [lindex $pop 1]
	set colonnes [lindex $pop 2]
	set obsminusone [expr {$lignes - 1}]
	# CALCULER MOYENNES ET DEVIATIONS STANDARD DES VARIABLES, et ...
	mnorms_br pop popmeans popsigmas
	# ... CREER UN VECTEUR AVEC AUTANT DE 1 QUE DE VARIABLES ...
	# IL SERVIRA POUR CREER UNE MATRICE CENTREE au lieu de NORMALISEE
	set popones "2 1 $colonnes 1"
	set x 1
	set y $colonnes
	while {$x < $y} {lappend popones 1; set x [expr {$x + 1}]}
	# CENTRER LES DONNEES PLUTOT QUE LES NORMALISER
	# ------------------------------------------------
	set Z [mnormalize $pop $popmeans $popones]
	# SINGULAR VALUE DECOMPOSITION using VARCOV for the eigensolution
	# ---------------------------------------------------------------
	set Z [mnormalize $pop $popmeans $popones]
	set ZZ [mmult [transpose $Z] $Z]
	set nless1 [expr {1.0/$obsminusone}]
	set vcm [mscale $ZZ $nless1]
	# SVD selon linalg, Maximum de 7 DECIMALES, pas plus!!
	#set Z $ttrantran($i)
	set vcm "[lrange $vcm 0 2] [show $vcm %8.7f]";# pas > 7 decimales
	set Ztcllib [::math::linearalgebra::from_LA $vcm]
	set svd_tcllib [::math::linearalgebra::determineSVD $Ztcllib]
	set U [::math::linearalgebra::to_LA [lindex $svd_tcllib 0]]
	set S [::math::linearalgebra::to_LA [lindex $svd_tcllib 1]]
	set V [::math::linearalgebra::to_LA [lindex $svd_tcllib 2]]
	return $V
    }
    #
    # Procedure COMPOSANTES - fin
    
    
    # Procedure PARTIALITE ## Debut ### - NAMESPACE NEWPW
    #
    proc partialite {reference} { 
	global partindiv;# ajout Aug_2009
	global p suit_part rawindividus rot lm
	global infile l1 l2 raizl1l2 ecnerefer prinw
	global tailles_base
	set consensus 0
	set mmm [llength $rawindividus]
	set lignes [expr {$mmm/$lm}]
	# APPEL a PRINW
	# Calcul des 'principal' warps a l'aide de la procedure PRINW (eigenvectors of the inverse of Lp)
	set prinw [newpw::prinwarps $reference]
	# APPEL a PARTIALW
	# Calcul des 'partial' warps 'scores' a l'aide de la procedure PARTIALW
	for  {set i 1} {$i < [expr {$lignes+1}]} {incr i} {
	    set objet $rot($i)
	    set matrefer [concat "2 $lm 2" $reference]
	    set partialwarps($i) [partialw $i $objet $matrefer $prinw]
	}
	if {[info exists suit_part] == 0} {
	    set partindiv "";# ajoute Aug2009
	    set compteur 0;#il n'existe pas un individu consensus rote
	    while {$compteur < $lignes} {
		incr compteur
		if {[info exists suit_part] == 0} {
		    append partindiv "[lrange $partialwarps($compteur) 3 end]\n"
		    append cs_pw_base "[lindex $tailles_base $compteur] [lrange $partialwarps($compteur) 3 end]\n"
		}
	    }
	}
	if {[info exists suit_part] == 0} {
	    return $partindiv
	}
	set suit_part "1"
	return $partindiv
    }
    #
    # Procedure PARTIALITE - fin
    
    
    # Procedure PRINWARPS = principal warps ## Debut - NAMESPACE NEWPW
    #
    proc prinwarps {reference} {
	global p lm
	# D'abord construire la L matrix, puis l'inverser, puis prendre le bloc pxp
	#
	# L matrix Etape 1.
	# ----------------------
	# Le bloc 'pxp' en haut a gauche est la matrice 'U'
	for {set i 0} {$i < [expr {$lm*2}]} {incr i 2} {
	    set x1 [lindex $reference $i]
	    set y1 [lindex $reference [expr {$i+1}]]
	    #$p insert end "\n\n---- x$i = $x1\ty$i = $y1\n\n"
	    for {set j 0} {$j < [expr {$lm*2}]} {incr j 2} {
		set x2 [lindex $reference $j]
		set y2 [lindex $reference [expr {$j+1}]]
		set r2 [expr {pow(($x1-$x2),2) + pow(($y1-$y2),2)} ]
		if {$r2 != 0} {set lnr2 [expr {log($r2)}]}
		if {$r2 != 0} {set r2lnr2 [expr {$r2*$lnr2}]}
		if {$r2 != 0} {append preU "$r2lnr2 "} else {append preU "0.000000 "}
	    }
	}
	set U "2 $lm $lm $preU"
	
	# L matrix Etape 2.
	# ---------------------
	# A cette matrice 'U' de distances je joins 3 colonnes,
	# l'une de 1 et les deux autres contenant la reference (l'objet consensus)
	set popones "2 $lm 1 1 "
	set x 1
	set y $lm
	while {$x < $y} {lappend popones 1; set x [expr {$x + 1}]}
	set consens "2 $lm 2 $reference"
	# $p insert end "\n On va tenter de fusionner\n[show $popones]\net\n[show $consens %1.4f]"
	set ajoutdroit [join_cols $popones $consens]
	# debug "\n On va  la lui ajouter a droite \n[show $ajoutdroit %1.4f]"
	set U_droit [join_cols $U $ajoutdroit]
	# debug "\nSoit:\n[show $U_droit %1.4f]"
	#
	# L matrix Etape 3.
	# ----------------------
	# A cette nouvelle matrice de lm+3 colonnes j'ajoute 3 lignes bas,
	# l'une de 1 et les deux autres contenant la reference (l'objet consensus)
	# puis je termine la matrice par des zeros (dans le coin inferieur droit)
	set preajoutbas [transpose $ajoutdroit]
	set leszeros "2 3 3 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0"
	set ajoutbas [join_cols $preajoutbas $leszeros]
	# debug "\n On va lui ajouter en bas \n[show $ajoutbas %1.4f]"
	set U_droit_bas [join_rows $U_droit $ajoutbas]
	# debug "\n Soit alors \n[show $U_droit_bas %1.4f]"
	
	# Inverse de la matrice L
	set Lmatrix $U_droit_bas
	
	set identite [mident [expr {$lm+3}]]
	set invLmatrix [msolve $Lmatrix $identite]
	
	# Upper left 'p x p' bloc of this matrix as the bending energy matrix
	set bendingenergy [mrange $invLmatrix 0 $lm 0 $lm]
	
	# APPEL a  COMPOSANTES
	# Principal warps (appel a la procedure 'composantes')
	global pxpu
	set pxpu 1
	set V [composantes $bendingenergy]
	set prinW [mrange $V 0 [expr {$lm-4}] 0 [expr {$lm-1}]]
	return $prinW
    }
    #
    # Procedure PRINCIPAL warps
    
    
    # Procedure PARTIALW - (partial warps scores) - debut - NAMESPACE NEWPW
    #
    # 1. Calcul des pw scores, composante uniforme
    #
    # 2. Calcul des partial warps scores selon '1/sqrt(n) . Vi . E. (alpha = 0, donc scale = 1)'
    #
    ### 'Partial warps scores are the inner product of each row of Vi = Xi - Xc
    ### 'with the p-k-1 columns of E (Rohlf, 1996).'
    #
    proc partialw {i objet matrefer prinw} {
	
	global p lm
	
	# L'objet et la reference
	set grandxi [transpose $objet]
	set reference [transpose $matrefer]
	# Vi = difference entre objet et reference
	set grandvi [msub $grandxi $reference]
	
	# Zeldich et al ------------------ start -----------------------------
	set ajout "2 3 2 0 0 0 0 0 0"
	set tgrandvi [transpose $grandvi]        
	set grandve [join_rows $tgrandvi $ajout]
	# Zeldich et al ------------------  fin  -----------------------------
	
	# ---
	#
	# Composante AFFINE, ou UNIFORME, de la CONFORMATION
	#
	# ---
	
	# ---
	# Toujours le calcul des U1 et U2 commence dans la procedure rotation
	# ---
	# Preparation du calcul de U1 et U2
	set grandxi2 [mprod $grandxi $grandxi]
	set grandxi2 [transpose $grandxi2]
	set prelambda [msum $grandxi2]
	set l1 [lindex $prelambda 3]
	set l2 [lindex $prelambda 4]
	set raizl1l2 [expr {sqrt($l1 * $l2)}]
	
	set col2 [mrange $grandxi 0 [expr {$lm-1}] 0 0]
	set col1 [mrange $grandxi 0 [expr {$lm-1}] 1 1]
	set ixdnarg [join_rows $col1 $col2]
	
	# ---
	# Le calul de U2
	# ---
	#
	set proU2 [mprod $grandxi $grandvi]
	
	set proU2 [transpose $proU2]
	set preU2 [lindex [msum $proU2] 4]
	set U2($i) [expr {$preU2/$raizl1l2}]
	#
	# ---
	# Le calul de U1
	# ---
	#
	# Le calcul de U1 est plus alambique !
	set presydeltax [mprod $ixdnarg $grandvi]
	set presydeltax [transpose $presydeltax]
	set sydeltax [lindex [msum $presydeltax] 3]
	set sxdeltay [lindex [msum $presydeltax] 4]
	set sumoins [expr {$sxdeltay - $sydeltax}]
	set l2moins [expr {$l2 * $sumoins}]
	set U1($i) [expr {($sydeltax + $l2moins)/$raizl1l2}]
	set uniform "2 1 2 $U1($i) $U2($i)"
	#
	# --------------------------------------------------------------------
	#
	# Composante NON AFFINE ou NON UNIFORME de la CONFORMATION
	#
	# --------------------------------------------------------------------
	# #     ATTENTION TEST 4 JUILLET 2006 RETRAIT DU SCALE FACTOR
	set scalefactor 1
	## 1/sqrt(n)
	#set scalefactor [expr {1.0*(1.0/sqrt(2))}]
	## 1/sqrt(n) . Vi
	set propw [mscale $grandvi $scalefactor]
	# 1/sqrt(n) . Vi . E. (alpha = 0, donc scale = 1)
	set prepw [mmult $propw $prinw]
	set partialwarps($i) [transpose $prepw]
	#
	# --------------------------------------------------------------------
	#
	# Les deux composantes AFFINE, et NON AFFINE, dans une meme matrice
	#
	# --------------------------------------------------------------------
	#
	set partialwarps($i) [join_rows $partialwarps($i) $uniform]
	return $partialwarps($i)
    }
    #
    # procedure PARTIAL warps
    
    
    
    # Procedure SUBDIVIDED_MATRIX_PW - debut - NAMESPACE NEWPW
    #
    proc subdivided_matrix_pw {mdonnees} {
	# 'mdonnees' est une HUME matrice
	global S lignes colonnes lm rawindividus tablind reference
	global proindividus;# Aug_2009
	#debug "\nDONNEES transmises par newpw::pw a subdivided_matrix_pw\n[show $mdonnees %1.3f]\n"
	set matinit [lrange $mdonnees 0 2]
	set colonnes [lindex $matinit 2]
	set lignes [lindex $matinit 1]
	set lm [expr {$colonnes/2}]
	# calcul du consensus (configuration moyenne)
	set consensus [mnorms $mdonnees];# a ce stade, 'consensus' est une matrice de Hume
	set consensus [lrange $consensus 3 [expr {3+$colonnes-1}]];# a present, 'consensus' est une liste
	set consensus [lformat %1.3f $consensus]
	set donnees [lrange $mdonnees 3 end];# les donnees sous forme de liste, egalement
	set data "";# data deviendra une liste de listes ayant l'aspect d'un tableau
	for  {set i 0} {$i < [expr {$lignes*$colonnes}]} {incr i $colonnes} {
	    set tranche [lrange $donnees $i [expr {$i+$colonnes-1}]]
	    append data "$tranche\n"
	}
	# On classe les coordonnees successives par individu dans la table tablind()
	if {[info exists tablind]} {unset tablind}
	set i -1
	set compteur 0
	set tablind(0) $consensus
	foreach item [split $data "\n"] {
	    if {$item != ""} {
		# On saute la premiere ligne
		incr compteur
		if {$compteur > 0} {
		    if  {$colonnes != [llength $item]} {
			puts "$lm <> [llength $item]"
			exit
		    }
		    incr i
		    set tablind($compteur) $item
		}
	    }
	}
	
	# AJOUTE EN SEPTEMBRE 2008
	# On cree une LISTE (un individu) de MATRICES  (une matrice est un couple de coordonnees, elle a 1 ligne et 2 colonnes)
	# (chaque 'landmark' est une 'matrice')
	if {[info exists rawindividus]} {unset rawindividus} else {set rawindividus ""}
	set n [llength $data]
	for  {set i 0} {$i < $n} {incr i 2} {
	    set item [lrange $data $i [expr {$i+1}]]
	    lappend rawindividus "2 1 2 $item"
	}
	# calculs PW proprement dits
	set preindividus [newpw::translation $lm]
	set proindividus [newpw::scaling $lm]
	#
	set rotindividus [newpw::rotation]
	# ATTENTION
        #
	# ATTENTION
	#
	set hume "2 [expr {$colonnes / 2}] 2 "
	set montref $hume$reference;# 'reference' apportee comme variable globale
	#
	set part [partialite $reference]
	set hume "2 $lignes [expr {$colonnes - 4}] "
	set montpart $hume$part
	return $rotindividus;# CE SONT LES INDIVIDUS ALIGNES
    }
    #
    # Procedure SUBDIVIDED_MATRIX_PW - fin
    
    
    #
    # PROCEDURES d'ITERATION - NAMESPACE NEWPW
    #
    proc iteration_aligned {x_mat_aligned hume} {
	# Alignement des individus (ce que renvoie la procedure 'subdivided_matrix_pw' )
	set x_mat_aligned [subdivided_matrix_pw $x_mat_aligned]
	set x_input "$hume $x_mat_aligned"
	return $x_input;# les alignes dans une nouvelle variable pour iteration
    }
    
    #
    # PROCEDURES d'ITERATION - NAMESPACE NEWPW
    #
    proc iteration_reference {x_input} {
	# Calcul du nouveau consensus, et donc des PW correspondant
	set reftrac [mnorms_br x_input reftruc std]
	set reference [lrange $reftruc 3 end]
	return [partialite $reference]
    }
    # Procedure ITERATION - end
    
    # PROCEDURE PW - start - NAMESPACE NEWPW
    #
    proc pw {x_total} {
	global S p lm lignes colonnes individus partindiv reference proindividus
	set x_mat $x_total
	set hume [lrange $x_total 0 2];# pour liste -> matrice
	# Pour le premier tour, les PW sont calcules par  'subdivided_matrix_pw', mais on s'en fout,
	# ce qui importe, c'est que la procedure  'subdivided_matrix_pw' renvoie les individus alignes
	#
	set S(premier_tour) 0;# a retirer plus tard
	#debug "\nnwpw::pw S(premier_tour) = $S(premier_tour)"
	set S(suit_scale) 0
	set S(tailles_base) ""
	#
	set aligned [newpw::subdivided_matrix_pw $x_mat]
	set aligned "$hume $aligned"
	set S(ctr) $proindividus
	# La moyenne de ces aligned deviendra la nouvelle reference (le nouveau consensus)
	# ... qu'on enverra dans la procedure 'partialite'
	#
	# La procedure subdivided_matrix_pw calcule aussi les PW, places dans la variable 'partindiv'
	set PW_hume "2 $lignes [expr {$colonnes - 4}] "
	set PW_iteration(0) "$PW_hume $partindiv"
	# La premiere reference est  montree ici pour presentation
	#debug "\nPrem ref \n$reference\n"
	#
	set sum 0.001
	set i 1
	while {[expr {abs([show $sum])}]  > 0.0001} {
	    global partindiv
	    #for {set i 1} {$i < 3} {incr i} {}
	    update
	    #.buttons.partiel configure -bg "light yellow" -fg red -text "PW $ RW\nITERATION\n$i"
	    #
	    set S(premier_tour) $i;# a retirer plus tard
	    set S(suit_scale) 1
	    #
	    set referit [mnorms_br aligned referite refestd]
	    if {[info exists S(themall)] != 1}  {
		$p insert end "\n\nITERATION $i : consensus:\n[show $referite]\n"
	    }
	    $p see end
	    #
	    set new_aligned [iteration_aligned $aligned $hume]
	    set aligned $new_aligned
	    set PW_iteration(0) "$PW_hume $partindiv"
	    set new_ref [iteration_reference $new_aligned]
	    #
	    set PW_iteration($i) "$PW_hume $partindiv"
	    #
	    set a [msub $PW_iteration($i) $PW_iteration([expr $i - 1]) ]
	    set sum [msum [msum $a]]
	    if { [expr {abs([show $sum])}]  < 0.0001} {
		update
		global S
		if {[info exists S(themall)]} {
		    if {[info exists S(refinc)]} {
		    } else {
			set S(lbl) "Now you can perform a Mahalanobis classification of external data"
		    }
		} else  {
		    set S(lbl) "Now you can enter the unknown individuals (new right hand button)."
		}
		
		#.buttons.lbl configure -text $S(lbl);
		#
		return  $PW_iteration($i)
	    } else {
		update
		global S
		#
		if {[info exists S(button_DAPW)]} {
		    if  {$S(button_DAPW) == 1} {
			set S(lbl) "Each classification performed on re-computed PW (reference + unknown)"
		    }
		} else  {
		    set S(lbl) "PW, iterative computation of consensus: ITERATION [expr {$i+1}]"
		}
		#.buttons.lbl configure -text $S(lbl)
		#
		#.buttons.partiel configure -bg red
	    }
	    incr i
	}
    }
    # Procedure PW - end
    #
    # Aug_2009 - nouveaux calculs de PW ------------  fin ----------------
}
#
# NEWPW - fin de l'espace de nommage ------------------------------------
#
# ------------------------------------------------------------------------
#
# ------------------------------------------------------------------------












# Procedure TYPE - start
#
proc type {message} {
    tk_messageBox -message $message -type ok 
}
#
# Procedure TYPE - end

# procedure TYPE_YN - debut
#
proc type_yn {message} {
    set reponse [tk_messageBox -message $message -type yesno]
    return $reponse
}
#

# devrait se trouver dans ::math::statistics
#
proc pnorm {x} {
    #
    # cumulative distribution function (cdf)
    # for the standard normal distribution like in the statistical software 'R'
    # (mean=0 and sd=1)
    #
    # x -> value for which the cdf should be calculated
    #
    set sum [expr {double($x)}]
    set oldSum 0.0
    set i 1
    set denom 1.0
    while {$sum != $oldSum} {
        set oldSum $sum
        incr i 2
        set denom [expr {$denom*$i}]
        #puts "$i - $denom"
        set sum [expr {$oldSum + pow($x,$i)/$denom}]
    }
    return [expr {0.5 + $sum * exp(-0.5 * $x*$x - 0.91893853320467274178)}]
}

proc arranco {} {
    cd [file dirname [info script]]
    #
    global S
    set S(dirscript) [pwd];#attention, il n'y a pas de "/" a la fin du nom du dossier du script (utiliser "file join etc")
    #
    cd ..
    set S(dirclic) [pwd]
    cd $S(dirscript)
    #
    #set S(md_num) 0
    #set S(md_list) ""
    #set S(perm) 0
    set S(save) "_INFO_ASI.txt"
    #
    set tcl_precision 14
    #
}


# Procedure UNOT - start
#
proc unot {} {
    
    global version
    set version $version
    
    
    
    global p S infile references sentier inputval inputval1 inputval2 mes version lignes colonnes sigdif nbindiv r \
	mediciones n lado c partialw nbvar procrustes_ss_model procrustes_ss_indiv \
	procrustes_ss_side procrustes_ss_interact procrustes_ss_residual
    
    set systeme " p S infile references sentier inputval inputval1 inputval2 mes version lignes colonnes sigdif nbindiv r \
            mediciones n lado c partialw nbvar procrustes_ss_model procrustes_ss_indiv \
            procrustes_ss_side procrustes_ss_interact procrustes_ss_residual"
    
    foreach item $systeme {
        upvar #0 $item x
        if {[info exists x]} {
            unset x
        }
    }
    global version
    set version "[file tail [info script]] 23"
    #arranco
    principal
}
#
# Procedure UNOT - end

#
# ################### bleu ##### Procedure ## Debut ##
proc bleu {mesg args} {
    global p
    $p insert end $mesg "bleu"
    update
    flush stdout
}

################### bleu ####### PROCEDURE ## FIN ###

################### rouge ##### Procedure ## Debut ##
proc rouge {mesg args} {
    global p
    $p insert end $mesg "rouge"
    update
    flush stdout
}

################### rouge ####### PROCEDURE ## FIN ###

################### vert ##### Procedure ## Debut ##
proc vert {mesg args} {
    global p
    $p insert end $mesg "vert"
    update
    flush stdout
}

################### vert ####### PROCEDURE ## FIN ###


# Procedure AFFICHE - debut
#
# Permet de faire apparaitre un avis
#
#
proc affiche {x y avis} {
    # x est la position sur l'axe horizontal de l'ecran
    # y est la position sur l'axe vertical de l'ecran
    # missive contient le message qu'on veut afficher
    #puts "help"
    destroy .avis
    toplevel .avis
    wm geometry .avis +$x+$y
    message .avis.t -text $avis
    pack .avis.t
}
#
# Procedure AFFICHE - fin


# procedure DEBUG
#
proc debug {output} {
    # fichier pour debugging
    global flag memo bugatti
    append bugatti "\n$output\n"
    set flag 1
    # text widget
    if {[info exists .debug]} {
        $memo insert end $bugatti
    } else  {
        destroy .debug .non
        toplevel .debug
        wm geometry .debug 710x600+205+66
        set memo [text .debug.t -wrap word -height 22 -font "courier 13" \
		      -bg grey88 \
		      -fg black\
		      -tabs "3c left"\
		      -yscrollcommand ".debug.v_scroll set" \
		      -xscrollcommand ".debug.h_scroll set"]
        
        scrollbar .debug.v_scroll -command ".debug.t yview"
        scrollbar .debug.h_scroll -command ".debug.t xview" -orient horizontal
        pack .debug.v_scroll -side right -fill y
        pack .debug.h_scroll -side bottom -fill x
        pack .debug.t -side bottom -fill both -expand 1
        
        button .debug.exit -fg red -text "Exit" -command {
            global find_last_indx pointf ici
            destroy .debug .head
        }
        button .debug.wrap -fg red -text "Wrap" -command {
            global flag
            if {$flag == 1} {
                .debug.t configure -wrap none
                set flag 0
            } else  {
                .debug.t configure -wrap word
                set flag 1
            }
        }
        #
        $memo insert end $bugatti
        pack .debug.exit .debug.wrap -side left -padx 5 -fill x
    }
}
#
# Procedure DEBUG fin




# Procedures associees a l'AIDE R ------------------------- start
#
# r_rouge, r_bleu, r_brun, r_vert, faar, fiar, afficheinfo, erela
# Il faut y ajouter en debut de source: "font create onlist -family courier -size 13 -weight normal -slant roman" et S(liwiclic)
#
# Procedures associees a l'AIDE R ------------------------- start
#
# Ces procedures sont appelees a partir du menu R
#
################### r_bleu ##### Procedure ## Debut ##
proc r_bleu {mesg args} {
    global S
    $S(ere) insert end $mesg "bleu"
    $S(ere) tag configure bleu -foreground blue
}
################### r_bleu ####### PROCEDURE ## FIN ###
#
################### r_brun ##### Procedure ## Debut ##
proc r_brun {mesg args} {
    global S
    $S(ere) insert end $mesg "brun"
    $S(ere) tag configure brun -foreground brown
}
################### r_brun ####### PROCEDURE ## FIN ###
#
################### r_vert ##### Procedure ## Debut ##
proc r_vert {mesg args} {
    global S
    $S(ere) insert end $mesg "vert"
    $S(ere) tag configure vert -foreground "dark green"
}
################### r_vert ####### PROCEDURE ## FIN ###
#
################### r_rouge ##### Procedure ## Debut ##
proc r_rouge {mesg args} {
    global S
    $S(ere) insert end $mesg "rouge"
    $S(ere) tag configure rouge -foreground red
}
################### r_rouge ####### PROCEDURE ## FIN ###
#
# Procedure EDIT_COPY
#
# cfr Eric Foster-Johnson, Second Edition, p.295
#
proc edit_copy {textwidget} {
    #tk_messageBox -message "J'ouvre le clipboard" -type ok
    set owner [selection own]
    if {$owner == $textwidget} {
        clipboard clear
        catch {
            clipboard append [selection get]
        }
    }
}
#
# Procedure EDIT_COPY

# Procedure EDIT_PASTE - start
#
# cfr Eric Foster-Johnson, Second Edition, p.297
#
proc edit_paste {textwidget} {
    catch {
        set clip [selection get -selection CLIPBOARD]
    }
    set idx [$textwidget index insert]
    set owner [selection own];# check if any text is selected in textwidget
    if {$owner == $textwidget} {
        catch {
            set idx [$owner index sel.first]
            $textwidget delete sel.first sel.last
        }
    }
    catch {
        $textwidget insert $idx $clip
    }
}
#
# Procedure EDIT_PASTE - fin


# Procedure FAAR - Fenetre d'aide au R - start
#
proc faar {} {
    global S infile
    #
    wm iconify .
    #
    destroy .rwidget
    toplevel .rwidget
    wm title .rwidget "R help"
    wm geometry .rwidget 750x600+450+50
    global S
    set S(ere) [text .rwidget.t -wrap word -height 22 -font courier \
		    -bg ivory \
		    -fg black\
		    -tabs "3c left"\
		    -yscrollcommand ".rwidget.v_scroll set" \
		    -xscrollcommand ".rwidget.h_scroll set"]
    scrollbar .rwidget.v_scroll -command ".rwidget.t yview"
    scrollbar .rwidget.h_scroll -command ".rwidget.t xview" -orient horizontal
    pack .rwidget.v_scroll -side right -fill y
    pack .rwidget.h_scroll -side bottom -fill x
    pack .rwidget.t -side bottom -fill both -expand 1
    #
    set S(rhelp) Rhelp
    #
    bind .rwidget.t <Control-Key-c> "edit_copy .rwidget.t; break"
}
# Procedure FAAR - Fenetre d'aide au R - end


# Procedure FAAR+ - Fenetre d'aide au R AVEC BOUTON clic+- start
#
proc faar+ {} {
    global S infile
    #
    #
    wm iconify .
    #
    #
    destroy .rwidget
    toplevel .rwidget
    wm title .rwidget "R help"
    wm geometry .rwidget 850x600+550+50
    global S
    set S(ere) [text .rwidget.t -wrap word -height 22 -font courier \
		    -bg ivory \
		    -fg black\
		    -tabs "3c left"\
		    -yscrollcommand ".rwidget.v_scroll set" \
		    -xscrollcommand ".rwidget.h_scroll set"]
    scrollbar .rwidget.v_scroll -command ".rwidget.t yview"
    scrollbar .rwidget.h_scroll -command ".rwidget.t xview" -orient horizontal
    pack .rwidget.v_scroll -side right -fill y
    pack .rwidget.h_scroll -side bottom -fill x
    pack .rwidget.t -side bottom -fill both -expand 1
    #
    set S(rhelp) Rhelp
    #
    bind .rwidget.t <Control-Key-c> "edit_copy .rwidget.t; break"
    #
    label .rwidget.l -text "The complete set of R instructions (including functions and package) in a single clic? Hit clic+"
    button .rwidget.b -text "CLIC+" -command {}
    pack .rwidget.b .rwidget.l -side left
}
# Procedure FAAR+ - Fenetre d'aide au R avec bouton clic+ - end

# Procedure FIAR - Fichier d'aide au R - start
#
proc fiar {analyse} {
    global S
    set fileraide  [file join $S(dirclic) $S(rhelp) $analyse]
    #type "$fileraide"
    set fid [open $fileraide r]
    set raide [read -nonewline $fid]
    close $fid
    #
    #$S(ere) insert end $raide
    r_bleu $raide
    #
}
#
# Procedure FIAR - Fichier d'aide au R - end


# Procedure AFFICHEINFO - debut - Appelee par la procedure ERELA
#
proc afficheinfo {id} {
    global attente
    if {[eof $id]} {
        set attente 1
        close $id
        return
    }
    gets $id ligne
}
#
# Procedure AFFICHEINFO - fin

# Procedure ERELA -- DEBUT
#
proc erela {} {
    global S
    update
    flush stdout
    
    if { [catch {
	#
	if {$S(liwiclic) == "linclic"} {
	    global attente numli
	    if {$S(hasmac)} {
		set fid [open "| open /Applications/R.app"]
		fconfigure $fid -buffering line
		catch [fileevent $fid readable "afficheinfo $fid"] ici
	    } else {
		set fid [open "| xterm R"]
		fconfigure $fid -buffering line
		catch [fileevent $fid readable "afficheinfo $fid"] ici
	    }
	} else  {
	    type "Windows users.\nR might open and not work properly.\
                  \nIn that case, please open it using your Windows explorer"
	    if {[file exists $S(dirclic)/+/W/cfgW/pathR.txt]} {
		#
		# open le fichier pour lire
		#
		set fid [open $S(dirclic)/+/W/cfgW/pathR.txt r]
		set S(endroit) [read -nonewline $fid]
		close $fid
		#
		set fid [open "| \"$S(endroit)\""]
		fconfigure $fid -buffering line
		catch [fileevent $fid readable "afficheinfo $fid"] ici
	    } else {
		# fichier absent
		type "CLIC did not find the path of Rgui\
\nIn case of persistant problem, just go to your Windows explorer to open Rgui, and put it close to the CLIC widget "
		set onvachercher [type_yn "Do you want to find it for CLIC?"]
		switch -- $onvachercher {
		    yes {
			set S(endroit) [tk_getOpenFile]
			set fid [open $S(dirclic)/+/W/cfgW/pathR.txt w]
			puts -nonewline $fid $S(endroit)
			close $fid
		    }
		    no {}
		}
		#fichier present
	    }
	}
	#
	update
	flush stdout
    } result] } {
        # There is an error
        #
        if {$S(liwiclic) == "linclic"} {
            type "CLIC did not find the 'xterm' terminal, or did not find R\
                    \nhttp://cran.r-project.org/"
            set S(erela) 1
            return
        } else  {
	    set endroit $S(endroit)
            type "CLIC did not find Rgui here: \n$endroit\
                    \n\nhttp://cran.r-project.org/ \
                    \nIn case of persistant problem, just go to your Windows explorer to open Rgui, and put it close to the CLIC widget"
	    set onvachercher [type_yn "Do you want to find it for CLIC?"]
	    switch -- $onvachercher {
		yes {
		    set S(endroit) [tk_getOpenFile]
		    set fid [open $S(dirclic)/+/W/cfgW/pathR.txt w]
		    puts -nonewline $fid $S(endroit)
		    close $fid
		}
		no {}
	    }
	}
	# Version future: Proposer d'ecrire le chemin qui conduit au programme
    }
    set S(erela) 1
    return
}
#
# Procedure ERELA -- fin
#
#
# Procedures associees a l'AIDE R --------------------------- end
# r_rouge, r_bleu, r_brun, r_vert, fiar, faar, afficheinfo, erela
# Procedures associees a l'AIDE R --------------------------- end

# Procedure HISTOGRABE_ASI
#
proc  histograbe_asi {data1 valclass c args} {
    global infile references
    #
    set sizetransf $args
    #
    # La variable c contient le nom du CANVAS des graphiques, ce nom est donne dans SIGNEDIFF
    #
    # Recherche des limites + valeur maximale des donnees
    set data_tot $data1
    set data_tot_ordre [lsort -real -increasing $data_tot]
    
    # valeurs entieres
    #
    # translation des valeurs au-dessus de 0
    set min [lindex $data_tot_ordre 0]
    set translation [expr {0 - $min}]
    set data_tot_ordre_new ""
    foreach item $data_tot_ordre {
        append data_tot_ordre_new "[expr {$item + $translation}] "
    }
    # On prend les valeurs multipliees par 1000
    set data_tot_ordre_newnew ""
    #
    foreach item $data_tot_ordre_new {
        append data_tot_ordre_newnew "[expr {int(1000000 * $item)}] "
    }
    #
    set data_ordre $data_tot_ordre_newnew
    #
    set max [expr {int([lindex $data_ordre end])}]
    set min [expr {int([lindex $data_ordre 0])}]
    set envergure [expr {$max - $min}]
    if {$valclass > $envergure} {set valclass [expr {$envergure - 1}]}
    set epaisseur [expr {int(round($envergure / $valclass))}]
    #
    for {set i [expr {$min + $epaisseur}]} {$i < [expr {$max + $epaisseur}]} {incr i $epaisseur} {
        lappend limits $i
    }
    #
    # Calcul des hauteurs des rectangles pour les limites choisies
    set counts_1 [::math::statistics::histogram $limits $data_ordre]
    #
    set i 0
    set sum1 0
    set sum_share 0
    #
    foreach item $limits {
        set a [lindex $counts_1 $i]
        set sum1 [expr {$sum1 + $a}]
        incr i
    }
    #
    set counts_tot $counts_1
    set counts_tot_ordre [lsort -real -increasing $counts_tot]
    set counts_max [lindex $counts_tot_ordre end]
    
    # On colle le canvas
    pack $c -fill both -expand yes
    
    set add [expr {int(0.25 * (400 / $valclass))}]
    
    set ht 290
    set htt [expr {$ht - 10}]
    
    # Trace de l'histogramme
    set x [expr {65}]
    foreach v $counts_1 {
        set y [expr {$ht - ($htt * $v * (1.0 / $counts_max))}]; # la hauteur de v corrigee en fonction de la valeur maximale
        set x2 [expr {$x+$add}];# de x a x2 la largeur d'un rectangle
        $c create rectangle $x $y $x2 $ht -outline black -fill black
        set x [expr $x2+$add+$add]; # le x du prochain rectangle est deplace vers la droite
    }
    # dessiner l'echelle verticale sur la gauche de l'histogramme
    $c create line 40 0 40 [expr {$ht + 10}] -fill black
    # Dessiner la ligne de base de l'histogramme
    $c create line 40 $ht 480 $ht -fill black -width 3
    # Dessiner l'echelle des classes
    for {set i [expr {65+ $add + $add}]} {$i < 440} {incr i [expr {$add*2}]} {
        $c create line $i $ht $i [expr {$ht + 10}] -fill black
    }
    set addy 20
    # Dessiner l'echelle de l'ordonnee
    for {set i -10} {$i < $htt} {incr i [expr {$addy}]} {
        $c create line 40 [expr $i + $addy] 50 [expr $i + $addy] -fill black
    }
    # Comentaires
    $c create text 40 [expr {$ht + 35}] -text "$infile : n = $sum1" -anchor nw -justify left
    $c create text 40  [expr {$ht + 55}] -text "Number of intervals \t$valclass + ' > ' " -anchor nw -justify left
    $c create text 40  [expr {$ht + 75}] -text $sizetransf -anchor nw -justify left
}
#
# Procedure HISTOGRABE_ASI


# Procedure CHEMIN -start
#
proc chemin {} {
    
    global p S dossier
    
    if {[info exists dirinfile] == 0} {
	set dirinfile [pwd]
    }
    
    # ----------------------------------------------------
    # Nom du FICHIER et son CHEMIN (path) CORRESPONDANT
    # ---------------------------------------------------

    # Chercher sa route dans un fichier de configuration: il y en a un dans le dossier ASI (chemin), et un dans le dossier CLIC (route)
    # dans le dossier ASI
    set chemin "[pwd]/dossier_asi.txt"
    # dans le dossier CLIC
    set route $S(dirclic)/dossier_CLIC.txt
    #
    # On lit d'abord le fichier dossier_CLIC, s'il existe
    # sinon
    # on lit le fichier du script, s'il existe
    #
    # 1. verifier que les fichiers de routage existent
    if {[file exists $route] == 0} {
	# dans le cas ou il n'y a pas de fichier de routage dans le dossier CLIC
	set candidat_dossier_CLIC [pwd]
	catch [
	       set fid [open $route w+]
	       puts -nonewline $fid "Dernier dossier de donnees:\n"
	       puts -nonewline $fid [pwd]
	       close $fid
	      ] ici
    } else  {
	# 3. si le fichier de routage existe dans le dossier CLIC, reccueillir alors le dernier chemin parcouru
	# open le fichier pour lire
	catch [
	       set fid [open $route r]
	       set candidat_dossier_CLIC [read -nonewline $fid]
	       close $fid
	      ] ici
	set candidat_dossier_CLIC [lindex [split $candidat_dossier_CLIC \n] 1]
	#debug "\ndossier selon CLIC = $candidat_dossier_CLIC"
    }
    
    #debug "chemin $chemin"
    # 2. s'il n'existe pas, voir ci-dessus
    if {[file exists $chemin] == 0} {
	set candidat_dossier [pwd]
    } else  {
	# 3. si le fichier de routage du MODULE existe reccueillir le dernier chemin parcouru
	# open le fichier pour lire
	catch [
	       set fid [open $chemin r]
	       set candidat_dossier [read -nonewline $fid]
	       close $fid
	      ] ici
	set candidat_dossier [lindex [split $candidat_dossier \n] 1]
	#debug "\ndossier selon FOG: $candidat_dossier ... dossier selon CLIC = $candidat_dossier_CLIC"
    }
    
    #debug "si le dossier_CLIC n'est pas le dossier en cours (CLIC_XX): $candidat_dossier_CLIC"
    # A present, on a le choix entre candidat_dossier_CLIC et candidat_dossier
    if {$candidat_dossier_CLIC != [pwd]} {
	#debug "si le dossier_CLIC n'est pas le dossier en cours (CLIC_XX)"
	set candidat_dossier $candidat_dossier_CLIC
    } else  {
	# sinon, on adopte le candidat_dossier
    }
    
    # On va chercher le fichier
    set typeList {
    	{{"Text files"} {.txt .nts .TXT .NTS} }
    }
    set infile [tk_getOpenFile -filetypes $typeList -initialdir $candidat_dossier]
    
    #infile contient le chemin et le nom du fichier des donnees
    set dossier "[file dirname $infile]/";#repertoire des donnees
    
    # 4 si ce n'est pas le chemin indique par candidat_dossier, il faut reecrire le fichier de configuration
    if {$dossier != $candidat_dossier} {
    	# open le fichier de conf pour ecrire
    	catch [
    	       set fid [open $chemin w+]
    	       puts -nonewline $fid "Dernier dossier de donnees utilisees par COV:\n"
    	       puts -nonewline $fid $dossier
    	       close $fid
    	      ] ici
    	# open le fichier de routage pour ecrire
    	catch [
    	       set fid [open $route w+]
    	       puts -nonewline $fid "Dernier dossier de donnees :\n"
    	       puts -nonewline $fid $dossier
    	       close $fid
    	      ] ici
    }
    
    set dirinfile "[file dirname $infile]/";#repertoire des donnees
    set tailfile [file tail $infile];#fichier nu, avec l'extension
    set rootfile [file rootname $tailfile];#fichier nu, sans l'extension
    
    # 
    return $infile
}
#
# Procedure CHEMIN - end


# ###loadfile# PROCEDURE ## DEBUT###
proc loadfile {pathname} {
    if {$pathname == ""} {
        return
    } else  {
        set fid [open $pathname]
        set text [read -nonewline $fid]
        close $fid
        set rowcount 0
        set colcount 0
        global references
        set references "";#vu l'append en-dessous...
        #
        set compteur 0
        set lignes 0
        set data ""
	set text [string trim $text]
        foreach line [split $text \n] {
            incr compteur
            if {$compteur > 1} {
                if  {$line != ""} {
                    set colcount [llength $line]
		    foreach item [split $line] { 
			if {$item != ""} {
			    incr lignes
			    lappend data $item
			}
		    }
		}
	    } else {
		set references $line
	    }
        }
        set rowcount [expr {$lignes / $colcount}]
        set result [concat 2 $rowcount $colcount $data]
        #debug "Commentaire brut :\n$references"
        # -- Formater le commentaire
        # -- 1. Enlever d'eventuelles tabulations
        foreach mot $references {
            if {$mot != ""} {
                append comentario_sin_tab "$mot "
            }
        }
        set references $comentario_sin_tab
        #debug "commentaire sans tabulations;\n$references"
        # -- 2. Introduire des passages a la ligne (\n)
        set compteur 0
        set nbcaract [string length $references]
        #debug "Nombre de caracteres $nbcaract"
        if {$nbcaract > 80} {
            while {$compteur < $nbcaract} {
                incr compteur
                if {[expr $compteur % 80] != 0} {
                    #debug "string index references $compteur -> [string index $references $compteur]"
                    append newref [string index $references [expr {$compteur-1}] ]
                } else {
                    append newref "[string index $references [expr {$compteur-1}] ] - \n"
                }
            }
        } else  {
            set newref $comentario_sin_tab
            #debug "ELSE newref $newref"
        }
        set references $newref
        #debug "Commentaire formate\n$references"
        
        # -- FIN du formatage de la premiere ligne
        return $result
    }
}
########################loadfile# PROCEDURE ## FIN ###


###################################################################
proc  savelisting {} {
    # ***************************************************************
    # Tentative d'ecriture d'un fichier DE RAPPORT
    # ***************************************************************
    global p
    global infile
    
    set megainfo "\nFichier : $infile\n"
    update
    $p configure -bg grey66
    set megatexte [$p get 1.0 end]
    # OPEN THE FILE
    set filename "[file rootname $infile]_ASI_INFO.txt"
    set fileid [open $filename w]
    puts -nonewline $fileid $megainfo
    puts -nonewline $fileid "$megatexte"
    # CLOSE THE FILE
    close $fileid
}
###################################################################


# SUPERPOSITION de PROCRUSTES
# 
# TROIS PROCEDURES, + la procedure composante
#

# Procedure composantes
#
proc composantes {pop} {
    global p suit_rot pxpu
    set lignes [lindex $pop 1]
    set colonnes [lindex $pop 2]
    set obsminusone [expr {$lignes - 1}]
    mnorms_br pop popmeans popsigmas
    set popones "2 1 $colonnes 1"
    set x 1
    set y $colonnes
    while {$x < $y} {lappend popones 1; set x [expr {$x + 1}]}
    # CENTRER LES DONNEES PLUTOT QUE LES NORMALISER
    set Z [mnormalize $pop $popmeans $popones]
    # SVD using VARCOV for the eigensolution
    # --------------------------------------
    set Z [mnormalize $pop $popmeans $popones]
    set ZZ [mmult [transpose $Z] $Z]
    set nless1 [expr {1.0/$obsminusone}]
    set vcm [mscale $ZZ $nless1];# varcov
    # SINGULAR VALUE DECOMPOSITION selon linalg, Maximum de 7 DECIMALES, pas plus!!
    set vcm "[lrange $vcm 0 2] [show $vcm %8.7f]";# varcov, pas > 7 decimales
    #
    set Ztcllib [::math::linearalgebra::from_LA $vcm]
    set svd_tcllib [::math::linearalgebra::determineSVD $Ztcllib]
    set U [::math::linearalgebra::to_LA [lindex $svd_tcllib 0]]
    set S [::math::linearalgebra::to_LA [lindex $svd_tcllib 1]]
    set V [::math::linearalgebra::to_LA [lindex $svd_tcllib 2]]
    return $V
}
#
# Procedure composantes
    
proc translation {lm} {
    global  p grandp imoinsp rawindividus effectifs_mog tablind polygone
    global suit_tran suit_cons nosi suit_rot susnesnoc references
    set susnesnoc 0
    set nosi 0
    #
    set mmm [llength $rawindividus]
    set lignes [expr {$mmm/$lm}]
    if {[info exists suit_tran] == 0} {$p insert end "\n\nTRANSLACION de los OBJETOS sobre UN mismo CENTROIDE\n"}
    if {[info exists objet_cons]} {unset objet_cons}
    set suit_cons 0
    # Matrices I, P, I-P
    set identite [mident $lm];# a pxp identity matrix
    set unsurp [expr {1.0/$lm}]
    set grandp "2 $lm $lm"
    for {set i 0} {$i < [expr {$lm*$lm}]} {incr i} {
        append grandp " $unsurp";# a pxp matrix whith all elements equal to 1/p
    }
    set imoinsp [msub $identite $grandp];# I - P
    
    # Translation ou pre-multiplication des matrices par I-P
    global translatedx
    
    for  {set i 0} {$i < [expr {$lignes+1}]} {incr i} {
        set x "2 $lm 2 $tablind($i)"
        set translatedx($i) [mmult $imoinsp $x];# (I - P)X
        if {[info exists suit_tran] == 0} {$p insert end "\nTranslated matrix object $i\n[show $translatedx($i)]\n"}
    }
    # Formation des individus pour les graphiques
    set compteur -1
    while {$compteur < $lignes} {
        incr compteur
        set landmark 0
        #$p insert end "\n\nFormation des individus:\n$translatedx($compteur)\n"
        for  {set i 3} {$i<[expr {$lm*2+2}]} {incr i 2} {
            incr landmark
            set item [lrange $translatedx($compteur) $i [expr {$i+1}]]
            #debug "\ntranslated LM$landmark = \t$item"
            if {$item != ""} {lappend pourtranslatedindividus "2 1 2 $item"}
        }
    }
    set preindividus [lrange $pourtranslatedindividus $lm end]
    set polygone 0
    return $pourtranslatedindividus
}
# Proc TRANSLATION  - fin

### Procedure SCALING ### debut
#
proc scaling {lm} {
    # comments from Rohlf & Slice, 1990. Extensions of the Procrustes Method for the optimal superimposition of landmarks. Syst. Zool., 39(1): 40-59
    global tailles_base references
    # Mars 2005 j'ai ajoute transcaledtotal dans la liste globale ci-dessous, sans quoi il n'est pas possible de faire plusieurs aller-retour sur les boutons verts et rouges (raw, translation, scaling, rotation)
    global p imoinsp rawindividus translatedx transcaledtotal transcaledindiv preindividus
    global effectifs_mog
    global suit_scal polygone nosi suit_rot neweff consensus susnesnoc centroid
    global S;# pour S(centroide)
    set consensus 0
    set susnesnoc 0
    set nosi 0
    #set suit_rot 0
    set mmm [llength $rawindividus]
    set lignes [expr {$mmm/$lm}]
    # informe
    if {[info exists suit_scal] == 0} {$p insert end "\n\n\nSCALING to UNIT Centroid Size\n"}
    
    set suit_cons 0
    # UNIT CTR Scaling to unit centroid size
    for  {set i 0} {$i < [expr {$lignes+1}]} {incr i} {
        set ttranslatedx($i) [transpose $translatedx($i)]
        set prectr [mmult $translatedx($i) $ttranslatedx($i)]
        set proctr [mmult $prectr $imoinsp]
        set presquectr [transpose $proctr]
        # La taille centroide elevee au carre est la trace de presquectr
        set prectr2 [vdiag $presquectr]
        set proctr2 [vtrim $prectr2]
        set ctr2 0
        foreach item $proctr2 {set ctr2 [expr {$ctr2+$item}]}
        set ctr [expr {sqrt($ctr2)}]
        set centroid($i) $ctr
        #set ctr [format %1.1f $ctr]
        set scalefactor [expr {1.00000/$ctr}]
        set transcaledindiv($i) [mscale $ttranslatedx($i) $scalefactor]
        set transcaledindiv($i) [transpose $transcaledindiv($i)]
        # informe
        if {[info exists suit_scal] == 0} {
            $p insert end "\nScaled individual $i\n$ctr\n[show $transcaledindiv($i) %1.3f]"
            append transcaledtotal "[lrange $transcaledindiv($i) 3 end]\n"
            append centroide "$ctr\n"
            append tailles_base "$ctr "
        }
    }
    
    # # On sauve les tailles centroides dans filename_CTR
    # if {[info exists suit_scal] == 0} {
    #     savecoord $transcaledtotal "translated and scaled (first one is consensus)"
    #     savecoord $centroide "centroid sizes (first one is consensus)"
    # }
    set S(centroide) "2 $lignes 1 [lrange $tailles_base 1 end]";# enlever la taille du consensus: 'lrange ... 1 end'
    #debug "Tailles centroides\n$S(centroide)\n[show $S(centroide)]"
    
    set compteur -1
    while {$compteur < $lignes} {
        incr compteur
        #Formation des individus:\n$translatedx($compteur)\n"
        set landmark 0
        for  {set i 3} {$i<[expr {$lm*2+2}]} {incr i 2} {
            incr landmark
            set item [lrange $transcaledindiv($compteur) $i [expr {$i+1}]]
            if {$item != ""} {lappend pourtranscaledindividus "2 1 2 $item"}
        }
    }
    set proindividus [lrange $pourtranscaledindividus $lm end]
    return $pourtranscaledindividus
}
#
### Procedure SCALING ## FIN ###


### Procedure ROTATION ## debut ###
#
proc rotation {} {
    global p transcaledindiv colonnes lm effectifs_mog suit_rot suit_cons consensus objet_cons
    global infile reference rot indivmoy polygone rawindividus nosi susnesnoc
    global references
    global global
    set mmm [llength $rawindividus]
    set lignes [expr {$mmm/$lm}]
    set consensus 0
    set susnesnoc 1
    set suit_cons 1
    set nosi 0
    set indivmoy 0;#pour ne pas repeter les rapports du bouton jpdraw/moyens
    #
    if {[info exists suit_rot] == 0} {
	$p insert end "\n\nROTATION (aligned specimens)";#affichage
    }
    set V [composantes $transcaledindiv(0)];# il s'agit du consensus
    set prex1t [mmult $transcaledindiv(0) $V ];# Rohlf, 1990
    #
    if {[info exists suit_rot] == 0} {
	$p insert end  "\n\nConsensus aligned to its \
                principal component axes\n\n[show $prex1t %1.4f]\n"
    }
    # formation des points (individus) du consensus prex1t
    set landmark 0
    for  {set i 3} {$i<[expr {$lm*2+2}]} {incr i 2} {
        incr landmark
        set item [lrange $prex1t $i [expr {$i+1}]]
        if {$item != ""} {lappend objet_cons "2 1 2 $item"}
        if {[info exists suit_rot] == 0} {append aligned "$item "}
    }
    if {[info exists suit_rot] == 0} {append aligned "\n"}
    #$p insert end "\n$objet_cons"
    
    # Le consensus dans la variable reference (ici au singulier)
    if {[info exists suit_rot] == 0} {set reference $aligned}
    # 'references' au pluriel c'est la premiere ligne du fichier de donnees
    #
    # A present j'enleve du fichier Aligned le consensus qui gene sur la premiere position (Salaya, June 2007)
    set aligned ""
    # -------------------------------------------------------------------------
    
    # Rotations proprement dites
    set x1t [transpose $prex1t]
    for  {set i 1} {$i < [expr {$lignes+1}]} {incr i} {
        set ttrantran($i) [mmult $x1t $transcaledindiv($i)]
        # singular value decomposition
        set U $ttrantran($i)
        msvd_br U S V
        set S "2 2 2 1 0 0 1"
        set H [mmult [mmult $V $S] $U];# Rohlf & Slice, 1990, p.42 (Gower, 1971)
        set rot($i) [mmult $transcaledindiv($i) $H]
    }
    
    # Formation des points ('individus') apres rotation, pour le graphique
    set compteur 0;#il n'existe pas un individu consensus rote
    while {$compteur < $lignes} {
        incr compteur
        if {[info exists suit_rot] == 0} {$p insert end "\n\nrotated object $compteur \
                    \n[show $rot($compteur) %1.4f]"}
        set landmark 0
        for  {set i 3} {$i<[expr {$lm*2+2}]} {incr i 2} {
            incr landmark
            set item [lrange $rot($compteur) $i [expr {$i+1}]]
            if {$item != ""} {lappend pourotindividus "2 1 2 $item"}
            if {[info exists suit_rot] == 0} {append aligned "$item "}
        }
        if {[info exists suit_rot] == 0} {append aligned "\n"}
    }
    #
    set rotindividus [lrange $pourotindividus 0 end];#encore une fois, pas de consensus rote
    # individus alignes - coordonnees residuelles (objet par objet) --- debut
    set aligned_hume ""
    set hume  [lrange $prex1t 0 2]
    foreach item [split $aligned  "\n"] {
        if {$item != ""} {
            lappend aligned_hume "$hume $item"
        }
    }
    set global(aligned) $aligned_hume
    #debug "ALIGNED"
    #debug $aligned_hume;# rendre cette variable 'globale' comparer avec procrustesresidues
    #debug "PROCRUSTES RESIDUES"
    #set S(aligned) $aligned_hume
    # aligned_hume = liste de matrices à 2 colonnes
    #
    foreach item $aligned_hume {
        append procrustes_residues "[lrange [msub $prex1t $item] 3 end]\n";#calcul naif
    }
    #debug $procrustes_residues
    # if {[info exists suit_rot] == 0} {
    #     savecoord $procrustes_residues "Procrustes residues"
    # }
    # CONSTRUCTION DES RESIDUS DE PROCRUSTES ------------------------- fin
    
    # ---------------ORTHOGONAL PROJECTION---------calcul correct ---------
    #
    # Projection des résidus sur un plan tangent à l'espace de la forme - debut
    # According to Rohlf (1999)
    # "Orthogonal projection minimizes larges differences while stereographic projection accentuates them" (J. Claude, 1998)
    #
    set k 2;#2D analysis
    set kp [expr {$k*$lm}]
    #
    # Matrices grandX,Ikp, xm, xprimm
    #
    # grandX
    set hume  "2 $lignes $kp "
    set grandX $hume$aligned;#debug "\nAligned preshapes to the reference \n[show $grandX]"
    # Ikp
    set Ikp [mident $kp];#debug "\n$kp X $kp identity matrix, Ikp \n[show $Ikp]"
    # xm
    set hume "2 1 $kp "
    set xm $hume$reference;#debug "\nMean shape \n[show $xm]";#vectorized form of the mean
    # xprimm
    set xprimm [transpose $xm];#debug "transposee de xm \n[show $xprimm]"
    #
    set xprimmxm [mmult $xprimm $xm];#debug "xprimm X xm \n$xprimmxm";#"\n[show xprimmxm]"
    set difference [msub $Ikp $xprimmxm];#debug "difference \n[show $difference]"
    set orp [mmult $grandX $difference];#debug "Projections orthogonales dans le plan tangent \n[show $orp]"
    # Enregistrement (OP = orthogonal projection)"
    #
    set global(orp) $orp
    #savecoord $orp "Procrustes residues OP"
    #debug "Orthogonal projections\n$orp"
    # Projection des résidus sur un plan tangent à l'espace de la forme - fin
    #
    # ----------------------------------------------------------------------

    set suit_rot "1"
    #.menubar.filemenu6 configure -bg "light green"
    # Return deplace ci-dessous en fev. 2005 (ci-dessus unreachable !)
    return $pourotindividus
}
#
### Procedure rotation ## FIN



# SUPERPOSITION de PROCRUSTES


# Procedure GET_LM ---------------- debut
#
# normalement, il y a un 'mediciones' comme argument
# on entre un fichier '..._format.txt' (raw coordinates)
# (contenant au moins deux digitalisations successives, etc.)
#
proc get_lm {mediciones} {
    global p S infile
    global pop_lm pop_pwcs sentier references inputval 
    global mes;# mes() tableau indexe des matrices de donnees (une par mesure)
    global diff_gd;#hume matrice (variable indicée) des différ. gauche-droite
    #
    # -----------------------------------------------
    # Recherche du dossier - debut
    # -----------------------------------------------
    #
    # Nom du FICHIER et son CHEMIN (path)
    # -----------------------------------
    # Chercher sa route dans un fichier de configuration:
    # ... dans le dossier ASI (chemin), ... et dans le dossier CLIC (route)
    # dans le dossier ASI
    set chemin "[pwd]/dossier_asi.txt"
    #
    # dans le dossier CLIC
    set route $S(dirclic)/dossier_CLIC.txt
    #
    # On lit d'abord le fichier dossier_CLIC, s'il existe
    # sinon
    # on lit le fichier du script, s'il existe
    #
    # 1. verifier que les fichiers de routage existent
    if {[file exists $route] == 0} {
        # dans le cas ou il n'y a pas de fichier de routage dans le dossier CLIC
        set candidat_dossier_CLIC [pwd]
        catch [
	       set fid [open $route w+]
	       puts -nonewline $fid "Dernier dossier de donnees:\n"
	       puts -nonewline $fid [pwd]
	       close $fid
	      ] ici
    } else  {
        # 3. si le fichier de routage existe dans le dossier CLIC, reccueillir alors le dernier chemin parcouru
        # open le fichier pour lire
        catch [
	       set fid [open $route r]
	       set candidat_dossier_CLIC [read -nonewline $fid]
	       close $fid
	      ] ici
        set candidat_dossier_CLIC [lindex [split $candidat_dossier_CLIC \n] 1]
    }
    # 2. s'il n'existe pas, voir ci-dessus
    if {[file exists $chemin] == 0} {
        set candidat_dossier [pwd]
    } else  {
        # 3. si le fichier de routage du MODULE existe reccueillir le dernier chemin parcouru
        # open le fichier pour lire
        catch [
	       set fid [open $chemin r]
	       set candidat_dossier [read -nonewline $fid]
	       close $fid
	      ] ici
        set candidat_dossier [lindex [split $candidat_dossier \n] 1]
    }
    
    # A present, on a le choix entre candidat_dossier_CLIC et candidat_dossier
    if {$candidat_dossier_CLIC != "./"} {
        # si le dossier_CLIC n'est pas le dossier en cours (CLIC_XX)
        set candidat_dossier $candidat_dossier_CLIC
        #
        catch [
	       set fid [open $chemin w+]
	       puts -nonewline $fid "Dernier dossier de donnees utilise par ASI:\n"
	       puts -nonewline $fid $candidat_dossier
	       close $fid
	      ] ici
    } else  {
        # sinon, on adopte le candidat_dossier
        set candidat_dossier_CLIC $candidat_dossier
        catch [
	       set fid [open $route w+]
	       puts -nonewline $fid "Dernier dossier de donnees:\n"
	       puts -nonewline $fid $candidat_dossier_CLIC
	       close $fid
	      ] ici
    }
    # -----------------------------------------------
    # Recherche du dossier - fin
    # -----------------------------------------------
    
    # -----------------------------------------------
    # On va chercher les donnees - debut
    # -----------------------------------------------
    #
    set typeList {
        {{"Text files"} {.txt .nts .TXT .NTS} }
    }
    set infile [tk_getOpenFile -filetypes $typeList -initialdir $candidat_dossier]
    #infile contient le chemin et le nom du fichier des donnees
    set dossier "[file dirname $infile]/";#repertoire des donnees
    # 4 si ce n'est pas le chemin indique par candidat_dossier, il faut reecrire le fichier de configuration
    if {$dossier != $candidat_dossier} {
        # open le fichier de conf pour ecrire
        catch [
	       set fid [open $chemin w+]
	       puts -nonewline $fid "Dernier dossier de donnees utilisees par ASI:\n"
	       puts -nonewline $fid $dossier
	       close $fid
	      ] ici
        # open le fichier de routage pour ecrire
        catch [
	       set fid [open $route w+]
	       puts -nonewline $fid "Dernier dossier de donnees :\n"
	       puts -nonewline $fid $dossier
	       close $fid
	      ] ici
    }
    #
    set sentier [file dirname $infile]
    # --------------------------------------------------
    # USAGE DE LA PROCEDURE loadfile --> MATRICE pop_lm
    # --------------------------------------------------
    global version; # le numero de cette version ASI
    set queueinfile [file tail $infile]
    set dirinfile [file dirname $infile];# meme contenu que sentier
    set longdirinfile [llength $dirinfile]
    #
    if {$longdirinfile > 2} {
        set dirinfile "[lrange $dirinfile 0 2]/.../..."
    } else  {
        set dirinfile "$dirinfile/..."
    }
    wm title . "$version  -  $dirinfile$queueinfile"
    #
    # usage de loadfile pour chercher les donnees
    if {$infile != ""} {
        set pop_lm [loadfile "$infile"]
	# pop_lm une matrice de Hume
    } else  {
        return
    }
    #
    # -----------------------------------------------
    # On va chercher les donnees - pop_lm - fin
    # -----------------------------------------------
    #

    bleu "\n\nArchivo"
    rouge "\t$infile"
    rouge "\n$references\n\n"
    #debug [show $pop_lm]
    
    # --------------------------------------------------------
    # obtenir le pop_pwcs à partir du pop_lm...
    # --------------------------------------------------------
    #
    # # -------------------------------------------------
    # 1. SUPERPOSITION DE PROC
    # 2. CALCUL DES pw
    # 3. constitution du fichier '_PW_CS' (pop_pwcs)
    
    #####
    # 1. SUPERPOSITION DE PROCRUSTES ---------------------
    #####
    global S
    global lm 
    global tablind 
    global reference
    global rawindividus
    global proindividus
    # lm <- nombre de lm
    set lm [expr {[mcols $pop_lm]/2}]
    # Une liste de coordonnees ('data')
    set donnees [vtrim $pop_lm]
    set data "";# data deviendra une liste de listes ayant l'aspect d'un tableau
    for  {set i 0} {$i < [expr {[mrows $pop_lm]*[mcols $pop_lm]}]} {incr i [mcols $pop_lm]} {
	set tranche [lrange $donnees $i [expr {$i+[mcols $pop_lm]-1}]]
	append data "$tranche\n"
    }
    #
    # CONSENSUS (configuration moyenne)
    #    
    set consensus [mnorms $pop_lm]
    set consensus [lrange $consensus 3 [expr {3+[mcols $pop_lm]-1}]]
    # On classe les coordonnees successives par individu dans la table tablind()
    set i -1
    set compteur 0
    set tablind(0) $consensus
    foreach item [split $data "\n"] {
        #$p insert end "\n\nPour tablind:\n$item\n"
        if {$item != ""} {
            # On saute la premiere ligne
            incr compteur
            #$p insert end "\nPour tablind (compteur = $compteur):\n$item\n"
            if {$compteur > 0} {
                #verification du nombre de lm de chaque individu
                if  {[mcols $pop_lm] != [llength $item]} {
                    puts "$lm <> [llength $item]"
                    exit
                }
                incr i
                set tablind($compteur) $item
            }
        }
    }
   
    # A toutes fin utile (graphes), on cree une LISTE (un individu) de MATRICES 
    # (une matrice est un couple de coordonnees, elle a 1 ligne et 2 colonnes)
    # (chaque 'landmark' est une 'matrice')
    if {[info exists rawindividus]} {unset rawindividus} else {set rawindividus ""}
    set n [llength $data]
    #debug "\nCREATION DES INDIVIDUS ($data)"
    for  {set i 0} {$i < $n} {incr i 2} {
	set item [lrange $data $i [expr {$i+1}]]
	lappend rawindividus "2 1 2 $item"
    }
    #
    ## TRANSLATION - appel à la procédure homonyme AVEC argument (lm)
    #
    set preindividus [translation $lm];# une liste de matrices à une ligne deux colonnes
    #debug "TRANSLATED INDIVIDUALS\n $preindividus"
    #
    ## SCALING - appel à la procédure homonyme AVEC argument (lm)
    #    
    if {[info exists preindividus] == 1} {
     	global x1 translatedx transcaledindividus suit_scal
         set proindividus [scaling $lm];# une liste de matrices à une ligne deux colonnes
         #set compromis_transcaled [lrange $proindividus 0 [expr {$lm-1}]]
         set transcaledindividus [lrange $proindividus $lm end]
     } else {
     	type "Please TRANSLATE  first"
    }
    #debug "scaled INDIVIDUALS\n $proindividus"
    #    
    # ROTATION - appel à la procédure homonyme SANS argument
    #    
    global transcaledindiv suit_rot suit_cons polygone
    if {[info exists transcaledindiv] == 1} {
     	set polygone 0
     	set pourotindividus [rotation]
     	set suit_rot 1
    } else {
     	type "Please TRANSLATE  and SCALE first"
    }
    set suit_cons 0
    #debug "ROT\n$pourotindividus"

    # 
    ######
    # 2. CALCUL DES pw -----------------------------------
    ######
    set pop_pw [newpw::pw $pop_lm];#debug "PW = \n$pop_pw"
    #

    ######
    # 3. Constitution d'un fichier '_PW_CS' --------------
    ######
    set pop_pwcs [join_cols $S(centroide) $pop_pw]
    # debug "_PW_CS\n[show $pop_pwcs]\nS(centroide) \n[show $S(centroide)]"
    # type "[mrange $S(centroide) 0 0 0 5]"

    # -------------------------------------------------
    # QUELQUES STATISTIQUES SUR LA MATRICE pop UTILISEE
    # -------------------------------------------------
    # Nombre de lignes des fichiers separes (un fichier par mesure)
    set lig_pwcs [mrows $pop_pwcs]
    set col_pwcs [mcols $pop_pwcs]
    # Nombre de lignes des fichiers separes (un fichier par mesure)
    set lig_mes_pwcs [expr {$lig_pwcs / $mediciones}]
    
    # Il faudrait ensuite separer le fichier en autant de mesures, p. ex. en 2
    # et placer cela dans mes(1) et mes(2)
    # pour ensuite passer a STATIS
    
    # Separation du fichier pop_pwcs en autant de fichiers que de mesures
    set k 0
    for {set i 0} {$i < $mediciones} {incr i} {
        set mes($i) "2 $lig_mes_pwcs $col_pwcs "
        for {set j [expr {$k + 0}]} {$j < [expr {$k + $lig_mes_pwcs}]} {incr j} {
            append mes($i) "[lrange [mrange $pop_pwcs 0 end $j $j] 3 end] "
        }
        set k [expr {$k + $lig_mes_pwcs}]
    }

    for {set i 0} {$i < $mediciones} {incr i} {
        # On va extirper de mes($i) une matrice 'lr' des valeurs a gauche 'left' et une matrice des valeurs a droite 'right'
        # ... a ce stade, la gauche et la droite sont des lignes successives
        set gauche ""
        set droite ""
        for {set z 0} {$z < [mrows $mes($i)]} {incr z 2} {
            append gauche "[vtrim [mrange $mes($i) 1 end $z $z]] "
            #debug "ligne gauche [expr {$z+1}]:\n$gauche\n[llength $gauche]"
            set dr [expr {$z+1}]
            append droite "[vtrim [mrange $mes($i) 1 end $dr $dr]] "
            #debug "ligne droite [expr {$dr+1}]:\n$droite\n[llength $droite]"
        }
        #debug "pre-matrice gauche :\n$gauche\n[llength $gauche]"
        #debug "pre-matrice droite :\n$droite\n[llength $droite]"
        set hume "2 [expr {$lig_mes_pwcs * 1/2}] [expr {$col_pwcs - 1}] "
        set gauche "$hume $gauche"
        set droite "$hume $droite"
        #debug "\n&&&&&\nGAUCHE\n[show $gauche]\nDROITE\n[show $droite]"
        #set gd [msub $gauche $droite]
	set diff_gd($i) [msub $gauche $droite]
        set gd [join_rows $gauche $droite]
        #debug "\n&&&&&\n[show $diff_gd($i)]"
        #
    }
    # fin de 'On va extirper de mes($i) une matrice 'lr ... etc.'
    
    
    # - ASI_ACP des differences D G des PW, sans distinction des sessions de mesures...
    if {$gd != ""} {
        #debug [show $gd]
	#debug "\npop_pw_cs\n[show $pop_pwcs]\n"
        global p individus fectifs
        #acp $gd
        asi_acp [mrange $pop_pwcs 1 end 0 end];# 'asi_'acp car 'silent'
        set fectifs ""
        for {set e 0 }  {$e < [expr {[mrows $pop_pwcs] * 1/2}]} {incr e} {
            append fectifs "2 "
        }
        #debug "Nombre de lignes... pw_cs\t[mrows $pop_pwcs]\n effectifs: [llength $fectifs]\nfectifs\nINDIVIDUS\n$individus"
        global lignes colonnes asi_individus
set asi_individus $individus
        if {[info exists individus] == 1} {global effectifs; set effectifs $fectifs;numind $individus}
    }
    # ------
    
    # Nombre d'individus (non de lignes)
    set nbindiv [expr {($lig_mes_pwcs)/2}];# '2' parce que gauche et droite
    set inputval "$nbindiv $mediciones"
}
#
# Procedure GET_LM ---------------- fin

# Procedure GET_pw ---------------- debut
#
# appel par le bouton 'Shape' (et retour à ce bouton, vers la procedure STATIS)
#
proc get_pw {mediciones} {
    
    global pop_pw p S infile sentier references inputval 
    global mes;# mes() est le tableau indexe des matrices de donnees (une par mesure)
    global diff_gd;#hume matrice (variable indicée) des différences gauche-droite
    
    # ----------------------------------------------------
    # Nom du FICHIER et son CHEMIN (path) CORRESPONDANT
    # ----------------------------------------------------
    # Chercher sa route dans un fichier de configuration:
    # ... dans le dossier ASI (chemin), ... et dans le dossier CLIC (route)
    # dans le dossier ASI
    set chemin "[pwd]/dossier_asi.txt"
    #
    # dans le dossier CLIC
    set route $S(dirclic)/dossier_CLIC.txt
    #
    # On lit d'abord le fichier dossier_CLIC, s'il existe
    # sinon
    # on lit le fichier du script, s'il existe
    #
    # 1. verifier que les fichiers de routage existent
    if {[file exists $route] == 0} {
        # dans le cas ou il n'y a pas de fichier de routage dans le dossier CLIC
        set candidat_dossier_CLIC [pwd]
        catch [
	       set fid [open $route w+]
	       puts -nonewline $fid "Dernier dossier de donnees:\n"
	       puts -nonewline $fid [pwd]
	       close $fid
	      ] ici
    } else  {
        # 3. si le fichier de routage existe dans le dossier CLIC, reccueillir le dernier chemin parcouru
        # open le fichier pour lire
        catch [
	       set fid [open $route r]
	       set candidat_dossier_CLIC [read -nonewline $fid]
	       close $fid
	      ] ici
        set candidat_dossier_CLIC [lindex [split $candidat_dossier_CLIC \n] 1]
    }
    # 2. s'il n'existe pas, voir ci-dessus
    if {[file exists $chemin] == 0} {
        set candidat_dossier [pwd]
    } else  {
        # 3. si le fichier de routage du MODULE existe reccueillir le dernier chemin parcouru
        # open le fichier pour lire
        catch [
	       set fid [open $chemin r]
	       set candidat_dossier [read -nonewline $fid]
	       close $fid
	      ] ici
        set candidat_dossier [lindex [split $candidat_dossier \n] 1]
    }
    
    # A present, on a le choix entre candidat_dossier_CLIC et candidat_dossier
    if {$candidat_dossier_CLIC != "./"} {
        # si le dossier_CLIC n'est pas le dossier en cours (CLIC_XX)
        set candidat_dossier $candidat_dossier_CLIC
        #
        catch [
	       set fid [open $chemin w+]
	       puts -nonewline $fid "Dernier dossier de donnees utilise par ASI:\n"
	       puts -nonewline $fid $candidat_dossier
	       close $fid
	      ] ici
    } else  {
        # sinon, on adopte le candidat_dossier
        set candidat_dossier_CLIC $candidat_dossier
        catch [
	       set fid [open $route w+]
	       puts -nonewline $fid "Dernier dossier de donnees:\n"
	       puts -nonewline $fid $candidat_dossier_CLIC
	       close $fid
	      ] ici
    }
    #
    # On va chercher les donnees
    #
    set typeList {
        {{"Text files"} {.txt .nts .TXT .NTS} }
    }
    set infile [tk_getOpenFile -filetypes $typeList -initialdir $candidat_dossier]
    #infile contient le chemin et le nom du fichier des donnees
    set dossier "[file dirname $infile]/";#repertoire des donnees
    # 4 si ce n'est pas le chemin indique par candidat_dossier, il faut reecrire le fichier de configuration
    if {$dossier != $candidat_dossier} {
        # open le fichier de conf pour ecrire
        catch [
	       set fid [open $chemin w+]
	       puts -nonewline $fid "Dernier dossier de donnees utilisees par ASI:\n"
	       puts -nonewline $fid $dossier
	       close $fid
	      ] ici
        # open le fichier de routage pour ecrire
        catch [
	       set fid [open $route w+]
	       puts -nonewline $fid "Dernier dossier de donnees :\n"
	       puts -nonewline $fid $dossier
	       close $fid
	      ] ici
    }
    #
    set sentier [file dirname $infile]
    # -----------------------------------------------------------
    # USAGE DE LA PROCEDURE loadfile POUR CREER LA MATRICE pop_pw
    # -----------------------------------------------------------
    global version; # le numero de cette version ASI
    set queueinfile [file tail $infile]
    set dirinfile [file dirname $infile];# meme contenu que sentier
    set longdirinfile [llength $dirinfile]
    #
    if {$longdirinfile > 2} {
        set dirinfile "[lrange $dirinfile 0 2]/.../..."
    } else  {
        set dirinfile "$dirinfile/..."
    }
    wm title . "$version  -  $dirinfile$queueinfile"
    #
    # usage de loadfile pour chercher les donnees
    if {$infile != ""} {
        set pop_pw [loadfile "$infile"]
	# pop_pw une matrice de Hume
    } else  {
        return
    }
    #
    bleu "\n\nArchivo"
    rouge "\t$infile"
    rouge "\n$references\n\n"
    # ----------------------------------------------------
    # QUELQUES STATISTIQUES SUR LA MATRICE pop_pw UTILISEE
    # ----------------------------------------------------
    # Nombre de lignes des fichiers separes (un fichier par mesure)
    set lig_pw [mrows $pop_pw]
    set col_pw [mcols $pop_pw]
    # Nombre de lignes des fichiers separes (un fichier par mesure)
    set lig_mes_pw [expr {$lig_pw / $mediciones}]
    
    # Il faudrait ensuite separer le fichier en autant de mesures, p. ex. en 2
    # et placer cela dans mes(1) et mes(2)
    # pour ensuite passer a STATIS
    
    # Separation du fichier pop_pw en autant de fichiers que de mesures
    set k 0
    for {set i 0} {$i < $mediciones} {incr i} {
        set mes($i) "2 $lig_mes_pw $col_pw "
        for {set j [expr {$k + 0}]} {$j < [expr {$k + $lig_mes_pw}]} {incr j} {
            append mes($i) "[lrange [mrange $pop_pw 0 end $j $j] 3 end] "
        }
        set k [expr {$k + $lig_mes_pw}]
    }
    
    #debug "pop_pw\n[show $pop_pw]"
    #debug "mes (1) \n[show $mes(0)]"
    #debug "mes (2) \n[show $mes(1)]"

    for {set i 0} {$i < $mediciones} {incr i} {
        # On va extirper de mes($i) une matrice 'lr' des valeurs a gauche 'left' et une matrice des valeurs a droite 'right'
        # ... a ce stade, la gauche et la droite sont des lignes successives
        set gauche ""
        set droite ""
        for {set z 0} {$z < [mrows $mes($i)]} {incr z 2} {
            append gauche "[vtrim [mrange $mes($i) 0 end $z $z]] "
            #debug "ligne gauche [expr {$z+1}]:\n[vtrim [mrange $mes($i) 0 end $z $z]] "
            set dr [expr {$z+1}]
            append droite "[vtrim [mrange $mes($i) 0 end $dr $dr]] "
            #debug "ligne droite [expr {$dr+1}]:\n[vtrim [mrange $mes($i) 0 end $dr $dr]] "
        }
        #debug "pre-matrice gauche :\n$gauche\n[llength $gauche]"
        #debug "pre-matrice droite :\n$droite\n[llength $droite]"
        set hume "2 [expr {$lig_mes_pw * 1/2}] $col_pw "
        set gauche "$hume $gauche"
        set droite "$hume $droite"
        #debug "\n&&&&&\nGAUCHE\n[show $gauche]\nDROITE\n[show $droite]"
        #set gd [msub $gauche $droite]
	set diff_gd($i) [msub $gauche $droite]
        set gd [join_rows $gauche $droite]
        #debug "\n&&&&&\n[show $diff_gd($i)]"
        #
    }
    # fin de 'On va extirper de mes($i) une matrice 'lr ... etc.'
    
    
    # - ASI_ACP des differences D G des PW, sans distinction des sessions de mesures...
    if {$gd != ""} {
        #debug [show $gd]
	#debug "\npop_pw\n[show $pop_pw]\n"
        global p individus fectifs
        #acp $gd
        asi_acp [mrange $pop_pw 0 end 0 end];# 'asi_'acp car 'silent'
        set fectifs ""
        for {set e 0 }  {$e < [expr {[mrows $pop_pw] * 1/2}]} {incr e} {
            append fectifs "2 "
        }
        #debug "Nombre de lignes... pw\t[mrows $pop_pw]\n effectifs: [llength $fectifs]\nfectifs\nINDIVIDUS\n$individus"
        global lignes colonnes asi_individus
set asi_individus $individus
        if {[info exists individus] == 1} {global effectifs; set effectifs $fectifs;numind $individus}
    }
    # ------
    
    # Nombre d'individus (non de lignes)
    set nbindiv [expr {($lig_mes_pw)/2}];# '2' parce que gauche et droite
    set inputval "$nbindiv $mediciones"
}
#
# Procedure GET_PW ---------------- fin

# Procedure GET_pwcs ---------------- debut
#
proc get_pwcs {mediciones} {
    
    global pop_pwcs p S infile sentier references inputval 
    global mes;# mes() est le tableau indexe des matrices de donnees (une par mesure)
    global diff_gd;#hume matrice (variable indicée) des différences gauche-droite
    
    # ----------------------------------------------------
    # Nom du FICHIER et son CHEMIN (path) CORRESPONDANT
    # ----------------------------------------------------
    # Chercher sa route dans un fichier de configuration:
    # ... dans le dossier ASI (chemin), ... et dans le dossier CLIC (route)
    # dans le dossier ASI
    set chemin "[pwd]/dossier_asi.txt"
    #
    # dans le dossier CLIC
    set route $S(dirclic)/dossier_CLIC.txt
    #
    # On lit d'abord le fichier dossier_CLIC, s'il existe
    # sinon
    # on lit le fichier du script, s'il existe
    #
    # 1. verifier que les fichiers de routage existent
    if {[file exists $route] == 0} {
        # dans le cas ou il n'y a pas de fichier de routage dans le dossier CLIC
        set candidat_dossier_CLIC [pwd]
        catch [
	       set fid [open $route w+]
	       puts -nonewline $fid "Dernier dossier de donnees:\n"
	       puts -nonewline $fid [pwd]
	       close $fid
	      ] ici
    } else  {
        # 3. si le fichier de routage existe dans le dossier CLIC, reccueillir alors le dernier chemin parcouru
        # open le fichier pour lire
        catch [
	       set fid [open $route r]
	       set candidat_dossier_CLIC [read -nonewline $fid]
	       close $fid
	      ] ici
        set candidat_dossier_CLIC [lindex [split $candidat_dossier_CLIC \n] 1]
    }
    # 2. s'il n'existe pas, voir ci-dessus
    if {[file exists $chemin] == 0} {
        set candidat_dossier [pwd]
    } else  {
        # 3. si le fichier de routage du MODULE existe reccueillir le dernier chemin parcouru
        # open le fichier pour lire
        catch [
	       set fid [open $chemin r]
	       set candidat_dossier [read -nonewline $fid]
	       close $fid
	      ] ici
        set candidat_dossier [lindex [split $candidat_dossier \n] 1]
    }
    
    # A present, on a le choix entre candidat_dossier_CLIC et candidat_dossier
    if {$candidat_dossier_CLIC != "./"} {
        # si le dossier_CLIC n'est pas le dossier en cours (CLIC_XX)
        set candidat_dossier $candidat_dossier_CLIC
        #
        catch [
	       set fid [open $chemin w+]
	       puts -nonewline $fid "Dernier dossier de donnees utilise par ASI:\n"
	       puts -nonewline $fid $candidat_dossier
	       close $fid
	      ] ici
    } else  {
        # sinon, on adopte le candidat_dossier
        set candidat_dossier_CLIC $candidat_dossier
        catch [
	       set fid [open $route w+]
	       puts -nonewline $fid "Dernier dossier de donnees:\n"
	       puts -nonewline $fid $candidat_dossier_CLIC
	       close $fid
	      ] ici
    }
    #
    # On va chercher les donnees
    #
    set typeList {
        {{"Text files"} {.txt .nts .TXT .NTS} }
    }
    set infile [tk_getOpenFile -filetypes $typeList -initialdir $candidat_dossier]
    #infile contient le chemin et le nom du fichier des donnees
    set dossier "[file dirname $infile]/";#repertoire des donnees
    # 4 si ce n'est pas le chemin indique par candidat_dossier, il faut reecrire le fichier de configuration
    if {$dossier != $candidat_dossier} {
        # open le fichier de conf pour ecrire
        catch [
	       set fid [open $chemin w+]
	       puts -nonewline $fid "Dernier dossier de donnees utilisees par ASI:\n"
	       puts -nonewline $fid $dossier
	       close $fid
	      ] ici
        # open le fichier de routage pour ecrire
        catch [
	       set fid [open $route w+]
	       puts -nonewline $fid "Dernier dossier de donnees :\n"
	       puts -nonewline $fid $dossier
	       close $fid
	      ] ici
    }
    #
    set sentier [file dirname $infile]
    # --------------------------------------------------------
    # USAGE DE LA PROCEDURE loadfile POUR CREER LA MATRICE pop_pwcs
    # --------------------------------------------------------
    global version; # le numero de cette version ASI
    set queueinfile [file tail $infile]
    set dirinfile [file dirname $infile];# meme contenu que sentier
    set longdirinfile [llength $dirinfile]
    #
    if {$longdirinfile > 2} {
        set dirinfile "[lrange $dirinfile 0 2]/.../..."
    } else  {
        set dirinfile "$dirinfile/..."
    }
    wm title . "$version  -  $dirinfile$queueinfile"
    #
    # usage de loadfile pour chercher les donnees
    if {$infile != ""} {
        set pop_pwcs [loadfile "$infile"]
	# pop_pwcs une matrice de Hume
    } else  {
        return
    }
    #
    bleu "\n\nArchivo"
    rouge "\t$infile"
    rouge "\n$references\n\n"
    # -------------------------------------------------
    # QUELQUES STATISTIQUES SUR LA MATRICE pop UTILISEE
    # -------------------------------------------------
    # Nombre de lignes des fichiers separes (un fichier par mesure)
    set lig_pwcs [mrows $pop_pwcs]
    set col_pwcs [mcols $pop_pwcs]
    # Nombre de lignes des fichiers separes (un fichier par mesure)
    set lig_mes_pwcs [expr {$lig_pwcs / $mediciones}]
    
    # Il faudrait ensuite separer le fichier en autant de mesures, p. ex. en 2
    # et placer cela dans mes(1) et mes(2)
    # pour ensuite passer a STATIS
    
    # Separation du fichier pop_pwcs en autant de fichiers que de mesures
    set k 0
    for {set i 0} {$i < $mediciones} {incr i} {
        set mes($i) "2 $lig_mes_pwcs $col_pwcs "
        for {set j [expr {$k + 0}]} {$j < [expr {$k + $lig_mes_pwcs}]} {incr j} {
            append mes($i) "[lrange [mrange $pop_pwcs 0 end $j $j] 3 end] "
        }
        set k [expr {$k + $lig_mes_pwcs}]
    }
    
    #debug "pop_pwcs\n[show $pop_pwcs]"
    #debug "mes (1) \n[show $mes(0)]"
    #debug "mes (2) \n[show $mes(1)]"

    for {set i 0} {$i < $mediciones} {incr i} {
        # On va extirper de mes($i) une matrice 'lr' des valeurs a gauche 'left' et une matrice des valeurs a droite 'right'
        # ... a ce stade, la gauche et la droite sont des lignes successives
        set gauche ""
        set droite ""
        for {set z 0} {$z < [mrows $mes($i)]} {incr z 2} {
            append gauche "[vtrim [mrange $mes($i) 1 end $z $z]] "
            #debug "ligne gauche [expr {$z+1}]:\n$gauche\n[llength $gauche]"
            set dr [expr {$z+1}]
            append droite "[vtrim [mrange $mes($i) 1 end $dr $dr]] "
            #debug "ligne droite [expr {$dr+1}]:\n$droite\n[llength $droite]"
        }
        #debug "pre-matrice gauche :\n$gauche\n[llength $gauche]"
        #debug "pre-matrice droite :\n$droite\n[llength $droite]"
        set hume "2 [expr {$lig_mes_pwcs * 1/2}] [expr {$col_pwcs - 1}] "
        set gauche "$hume $gauche"
        set droite "$hume $droite"
        #debug "\n&&&&&\nGAUCHE\n[show $gauche]\nDROITE\n[show $droite]"
        #set gd [msub $gauche $droite]
	set diff_gd($i) [msub $gauche $droite]
        set gd [join_rows $gauche $droite]
        #debug "\n&&&&&\n[show $diff_gd($i)]"
        #
    }
    # fin de 'On va extirper de mes($i) une matrice 'lr ... etc.'
    
    
    # - ASI_ACP des differences D G des PW, sans distinction des sessions de mesures...
    if {$gd != ""} {
        #debug [show $gd]
	#debug "\npop_pw_cs\n[show $pop_pwcs]\n"
        global p individus fectifs
        #acp $gd
        asi_acp [mrange $pop_pwcs 1 end 0 end];# 'asi_'acp car 'silent'
        set fectifs ""
        for {set e 0 }  {$e < [expr {[mrows $pop_pwcs] * 1/2}]} {incr e} {
            append fectifs "2 "
        }
        #debug "Nombre de lignes... pw_cs\t[mrows $pop_pwcs]\n effectifs: [llength $fectifs]\nfectifs\nINDIVIDUS\n$individus"
        global lignes colonnes asi_individus
set asi_individus $individus
        if {[info exists individus] == 1} {global effectifs; set effectifs $fectifs;numind $individus}
    }
    # ------
    
    # Nombre d'individus (non de lignes)
    set nbindiv [expr {($lig_mes_pwcs)/2}];# '2' parce que gauche et droite
    set inputval "$nbindiv $mediciones"
}
#
# Procedure GET_PWCS ---------------- fin



# Procedure GETFILES
#
proc get_files {mediciones} {
    #
    global p S infile sentier references version mes filerep1 filerep2;# mes() le tableau indexe des matrices de donnees
    #
    # OUVERTURE des fichiers en 2 etapes
    #
    # ------------------------------------
    # Ouverture du premier fichier
    # ------------------------------------
type "Please open the first file with size measurements" 
    set infile [chemin]
    set sentier [file dirname $infile]
    set i 0;# pour permettre le premier indice de mes(i)
    #
    # version le numero de version de cet ASI
    set queueinfile [file tail $infile]
    set dirinfile [file dirname $infile];# meme contenu que sentier
    set longdirinfile [llength $dirinfile]
    # 
    if {$longdirinfile > 2} {
	set dirinfile "[lrange $dirinfile 0 2]/.../..."
    } else  {
	set dirinfile "$dirinfile/..."
    }
    wm title . "$version  -  $dirinfile$queueinfile"
    # ------------
    # USAGE DE LA PROCEDURE loadfile POUR CREER LA MATRICE mes($i)
    # -------------
    # usage de loadfile pour chercher les donnees
    set mes($i) [loadfile "$infile"]
    set filerep1 [file tail $infile];# pour R
    if {$mes($i) == ""} {
	type "You did not select a file; exiting"
	exit
    } else  {
	bleu "\n\nArchivo"
	rouge "\t$infile"
	bleu "\nMedicion numero [expr {$i+1}] (Measurement number: [expr {$i+1}])"
	bleu "\nComentario del ARCHIVO (first row of the file):\n$references\n\n"	
	# ---------
	# QUELQUES STATISTIQUES SUR LA MATRICE pop UTILISEE
	# ----------
	set ouaip 0
	set lignes [mrows $mes($i)]
	set colonnes [mcols $mes($i)]
    }
    
    
    # ------------------------------------
    # Ouverture du ou des fichiers suivant
    # ------------------------------------
type "Please open now the second file, the one with repeated measurements\n(same individuals, same order)" 
    
    for {set i 1} {$i < $mediciones} {incr i} {
        #
	set infile [chemin]
        set sentier [file dirname $infile]

        # --------------------------------------------------------
        # USAGE DE LA PROCEDURE loadfile POUR CREER LA MATRICE mes($i)
        # --------------------------------------------------------
        global version
        # version le numero de version de cet ASI
        set queueinfile [file tail $infile]
        set dirinfile [file dirname $infile];# meme contenu que sentier
        set longdirinfile [llength $dirinfile]
        #
        if {$longdirinfile > 2} {
            set dirinfile "[lrange $dirinfile 0 2]/.../..."
        } else  {
            set dirinfile "$dirinfile/..."
        }
        wm title . "$version  -  $dirinfile$queueinfile"
        #
        # usage de loadfile pour chercher les donnees
        set mes($i) [loadfile "$infile"]
	set filerep2 [file tail $infile];# pour R
        if {$mes($i) == ""} {
            type "You did not select a file; exiting"
            exit
        } else  {
	    bleu "\n\nArchivo"
	    rouge "\t$infile"
	    bleu "\nMedicion numero [expr {$i+1}] (Measurement number: [expr {$i+1}])"
	    bleu "\nComentario del ARCHIVO (first row of the file):\n$references\n\n"	
	    # ---------
	    # QUELQUES STATISTIQUES SUR LA MATRICE pop UTILISEE
	    # ----------
	    set ouaip 0
	    set lignes [mrows $mes($i)]
	    set colonnes [mcols $mes($i)]
        }
    };# Fin de la boucle qui cherche un nombre de fichiers correspondant au nombre de mesures
    return [expr {$lignes * 1/2}]
}
#
# Procedure GET_FILES ----- fin


# Procedure CORRELATION - debut
#
proc correlation {corps} {
    global p
    set lignes [mrows $corps]
    set pasdecorr 0
    # CALCULER MOYENNES ET DEVIATIONS STANDARD DES VARIABLES, et ...
    mnorms_br corps corpsmeans corpssigmas
    #debug "nouveau corps ? \n[show $corps] \nmoyennes ? $corpsmeans std ? $corpssigmas"
    # DEBUG correction pour sigma nul
    set sigmanul [lrange $corpssigmas 3 end]
    foreach item $sigmanul {
        if {$item < 0.0001} {set pasdecorr 1}
    }
    if {$pasdecorr == 0} {
        # NORMALISER LES DONNEES (MOYENNES = 0, VARIANCES = 1)
        set Z [mnormalize $corps $corpsmeans $corpssigmas]
        #debug "donnees normalisees  ? \n[show $Z]"
        # MATRICE des VARIANCES - COVARIANCES
        set ZZ [mmult [transpose $Z] $Z]
        #debug "\nzTz ... \n [show $ZZ %8.3f]"
        set denom [expr {$lignes - 1}]
        set R [mscale $ZZ [expr {1.0 / $denom}] ]
    } else  {
        set R "2 2 0 8888 9999"
    }
    # on montre
    return $R
}
#
# Procedure CORRELATION - fin


# Procedure PICASSO - debut
#
# Table des couleurs
proc picasso {} {
    global magritte
    # Creer des couleurs
    for {set i 0} {$i < 40} {incr i} {
        set rcol [expr int(rand()*65535)]
        set gcol [expr int(rand()*65535)]
        set bcol [expr int(rand()*65535)]
        set magritte($i) [format "#%.4x%.4x%.4x" $rcol $gcol $bcol]
    }
}
# Procedure PICASSO - fin

# Procedure ASIDRAW_REGGRISE - debut
#
proc asidraw_reggrise {igrec ixe args} {
    #
    global axes axeX sizetransf
    set axes $args
    #debug "args ou axes = $axes"
    global S w p gr pad verjpd adda_lignes cv1 cv2 size
    global numero;# pas certain
    global asi_effectifs magritte
    #
    update; flush stdout
    #
    if {[info exists gr] == 1} {global gr}
    #
    destroy .muestreo.entree
    destroy .muestreo.boutonBac
    destroy .progres
    destroy .subd
    #
    set lignes [llength $igrec]
    
    # VALEUR MINIMUM de l'axe des X (ixe)
    # ------------------------------------
    set axeX $ixe
    set abscisse($S(numero)) $ixe
    #tk_messageBox -message "axeX \n$axeX" -type ok
    set ordre [lsort -real $axeX]
    set minaxeX [lindex $ordre 0]
    set maxaxeX [lindex $ordre end]
    
    # VALEUR MINIMUM de l'axe des Y (igrec1)
    # ----------------------------------
    set axeY $igrec
    set ordre [lsort -real $axeY]
    set minaxeY [lindex $ordre 0]
    set maxaxeY [lindex $ordre end]
    # axe X: Translation des VALEURS SUR ZERO
    # ---------------------------------------
    set enliste ""
    foreach item $axeX {
        lappend enliste [expr $item - $minaxeX]
    }
    set centaxeX $enliste
    # axe YTranslation des VALEURS SUR ZERO
    # -------------------------------------
    set enliste ""
    foreach item $axeY {
        lappend enliste [expr $item - $minaxeY]
    }
    set centaxeY $enliste
    
    #VALEUR MAXIMUM de l'axe X centre
    # -------------------------------
    set ordre [lsort -real $centaxeX]
    set maxcentaxeX [lindex $ordre end]
    set mincentaxeX [lindex $ordre 0]
    #VALEUR MAXIMUM de l'axe Y centre
    # -------------------------------
    set ordre [lsort -real $centaxeY]
    set maxcentaxeY [lindex $ordre end]
    set mincentaxeY [lindex $ordre 0]
    
    set corrx [expr {300.0/$maxcentaxeX}]
    set corry [expr {300.0/$maxcentaxeY}]
    
    # -----------------------------------------------
    # Preparer la formule de la droite de regression
    set ixe2 ""
    set igrec2 ""
    set ixeigrec ""
    update
    set Sx 0
    set Sx2 0
    foreach item $centaxeX {
        set Sx [expr $Sx + $item];# somme des X
        lappend ixe2 [expr pow($item,2)]
        set Sx2 [expr $Sx2 + [expr pow($item,2)]];# somme des X carres
    }
    set Sy 0
    set Sy2 0
    foreach item $centaxeY {
        set Sy [expr $Sy+$item];# somme des Y
        lappend igrec2 [expr pow($item,2)]
        set Sy2 [expr $Sy2 + [expr pow($item,2)]];# somme des Y carres
    }
    set Sxy 0
    for  {set i 0} {$i < $lignes} {incr i} {
        set lun [lindex $centaxeX $i]
        set lautre [lindex $centaxeY $i]
        set lunlautre [expr $lun * $lautre]
        lappend ixeigrec $lunlautre
        set Sxy [expr $Sxy + $lunlautre];# somme des XY
    }
    set denom [expr [expr $lignes * $Sx2] - [expr pow($Sx,2)] ]
    set divis [expr 1.0 / $denom]
    set a [expr ($Sy*$Sx2 - $Sx*$Sxy) * $divis]
    set b [expr ($lignes*$Sxy - $Sx*$Sy) * $divis]
    # Prediction des Y
    set predictY ""
    set long [llength $igrec]
    for {set i 0} {$i < $long} {incr i} {
        set indx [ lindex $centaxeX $i ]
        lappend predictY [ expr $a + ($b * $indx) ]
    }
    # VALEUR MAXIMUM de l'axe des Y predits
    set ordre [lsort -real $predictY]
    set maxpredictY [lindex $ordre end]
    set minpredictY [lindex $ordre 0]
    # Translation des VALEURS sur ZERO pour l'axe des Y predits
    # ---------------------------------------------------------
    set enliste ""
    foreach item $predictY {
        lappend enliste [expr $item - $minpredictY]
    }
    set centpredictY $enliste
    # Preparer la droite de regression, fin.
    # --------------------------------------
    
    # -----------------------------------------------
    # Preparer la formule de la correlation, DEBUT
    # -----------------------------------------------
    set denomG [expr [expr $lignes * $Sx2] - [expr pow($Sx,2)] ]
    set denomD [expr [expr $lignes * $Sy2] - [expr pow($Sy,2)] ]
    set denom [expr sqrt($denomG*$denomD)]
    set divis [expr 1.0 / $denom]
    set r [expr ($lignes*$Sxy - $Sx*$Sy) * $divis]
    set r2 [expr $r * $r]
    #debug "\nr = $r\nr2=$r2"
    set ddl [expr $lignes - 2]
    set t [expr ( $r*sqrt($ddl) ) / sqrt(1-$r2) ]
    # -----------------------------------------------
    # Formule de la correlation, FIN
    # -----------------------------------------------
    
    # -----------------------------------------------------------------------
    # Preparation du GRAPHIQUE de la procedure ASIDRAW_REGRISE
    # ------------------------------------------------------------------------
    set w .plottasireggrise_
    set w $w$S(numero)
    toplevel $w
    set positionx [expr {205 + $S(numero)*5}]
    set positiony [expr {10 + $S(numero)*5}]
    wm geometry $w 550x500+[expr {205 + $S(numero)*20}]+[expr {10 + $S(numero)*25}]
    wm title $w "ASI"
    wm iconname $w "ASI"
    #
    set Font {Helvetica 10}
    #
    frame $w.buttons
    pack $w.buttons -side bottom -fill x -pady 2m
    #
    button $w.buttons.cancel -foreground white -text "Exit" -command "destroy $w"
    #
    button $w.buttons.ps -text "Save (.eps)" -bg white -command {
        global S infile
        # NAME THE FILE
        set file_types {
            {"All Files" * }
        }
        set dirinfile "[file dirname $infile]/";#repertoire des donnees
        set tailfile [file tail $infile];#fichier nu, avec l'extension
        set rootfile [file rootname $tailfile];#fichier nu, sans l'extension
        #debug "chemin = $dirinfile\nroot = $rootfile\ntail = $tailfile"
        #
        set filename [tk_getSaveFile -filetypes $file_types -initialdir $dirinfile]
        wm title . $filename
        # OPEN THE FILE
        set outputGraphe [open $filename w]
        puts -nonewline $outputGraphe [.plottasireggrise_$S(numero) postscript]
        close $outputGraphe
    }
    #
    button $w.buttons.xy -text "Y / X" -bg white -command {
        set avis "Y is Right minus Left size \nX is size \
                \n\nL-R should be unaffected by size variation"
        affiche 400 400 $avis
    }
    #
    pack $w.buttons.cancel $w.buttons.xy -side left -expand 1
    #
    # Monter le cannevas
    #
    #canvas  $c  -width 450 -height 450 -relief raised -background "light yellow" -cursor dot
    canvas  $w.c  -width 450 -height 450 -relief raised -background "light yellow" -cursor dot
    pack $w.c -side top -expand yes -fill both
    
    set c $w.c
    
    #debug
    #$p insert end "\ncanvas $c\n"
    
    set plotFont {Helvetica 16}
    
    ################################
    # Le plot est carre de toutes facons; dessin des axes
    set blup 300.0
    set xgauche 100
    set plic [expr (400 - $blup)/2]
    $c create line 100 [expr 400 - $plic] 400 [expr 400 - $plic] -width 1 ;\
	# axe des x croit de g a droite
    $c create line 100 [expr 400 - $plic] 100 [expr (400 - $plic - $blup)] -width 1 ;\
	# axe des y decroit de bas en ht
    set corry [expr ($blup/$maxcentaxeY)]
    set corrPy [expr ($blup/$maxpredictY)]
    for {set i 0} {$i <= 10} {incr i} {
        set x [expr {100 + ($i*30)}]
        $c create line $x [expr 400 - $plic] $x [expr 395 - $plic] -width 1
    }
    set hauteurY [expr ($blup)]
    set subdivisionY [expr $hauteurY/10]
    for {set i 0} {$i <= 10} {incr i} {
        set y [expr {400 - $plic - ($i*$subdivisionY)}]
        $c create line 100 $y 105 $y -width 1
    }
    
    ## L'instruction ci-dessous permet d'ecrire sur le canvas, en bas
    ## ajout August 2002
    update
    set textFont {Helvetica 8}
    
    # Table des gris
    set gris(0) white
    set gris(1) black
    set gris(2) grey86
    set gris(3) grey21
    set gris(4) grey55
    set gris(5) grey39
    set gris(6) grey70
    
    set coul 0
    foreach item [split $asi_effectifs] {
        set grisatre($coul) $gris($coul)
        incr coul
    }
    
    # Dessin des points OBSERVES sur le graphique
    set long [llength $centaxeX]
    # Creer des couleurs
    set coul 0
    foreach item [split $asi_effectifs] {
        set rcol [expr int(rand()*65535)]
        set gcol [expr int(rand()*65535)]
        set bcol [expr int(rand()*65535)]
        set name($coul) [format "#%.4x%.4x%.4x" $rcol $gcol $bcol]
        incr coul
    }
    # Preparer la fragmentation de l'echantillon total
    set coul 0
    set quidam 0
    #tk_messageBox -message "asi_effectifs $asi_effectifs" -type ok
    set legion [lindex $asi_effectifs $coul]
    # plotter
    for {set i 0} {$i < $long} {incr i} {
        incr quidam
        if {$quidam > $legion} {incr coul; incr legion [lindex $asi_effectifs $coul]}
        set indpc1 [ lindex $centaxeX $i ]
        set indpc2 [ lindex $centaxeY $i ]
        set x [ expr { $xgauche + ( $corrx * $indpc1 ) } ]
        set y [ expr { 400 - $plic - ( $corry * $indpc2 ) } ]
        set item [$c create rectangle [expr {$x-3}] [expr {$y-3}] [expr {$x+3}] [expr {$y+3}] \
		      -width 1 -outline black -fill $grisatre($coul)]
        $c addtag point withtag $item
    }
    # Dessin de la DROITE DE REGRESSION
    set x1 $mincentaxeX
    set y1 [ expr $a + ($b * $x1) ]
    set x2 $maxcentaxeX
    set y2 [ expr $a + ($b * $x2) ]
    set x1 [ expr { $xgauche + ( $corrx * $x1 ) } ]
    set y1 [ expr { 400 - $plic - ( $corry * $y1 ) } ]
    set x2 [ expr { $xgauche + ( $corrx * $x2 ) } ]
    set y2 [ expr { 400 - $plic - ( $corry * $y2 ) } ]
    set item [$c create line $x1 $y1 $x2 $y2 -width 1 -fill brown]
    $c addtag point withtag $item
    # Nommer les axes
    $c create text 50 45 -text "$axes" -font $plotFont
    $c create text 380 380 -text "$sizetransf" -font $plotFont
    $c create text 260 410 -text "$S(numero): Contribution of size \[as (R+L)/2\] to asymmetry =  [expr int($r * $r *100)]%\
            \n(R and L corrected or not, as indicated by the X axis legend)"
    $c create text 260 440 -text "(Y = [lrange $axeX 0 3] etc.)"
    # Indiquer sur le cote et dans l'ordre les differentes couleurs choisies
    for {set i 0} {$i < [expr $coul+1]} {incr i} {
        set item [$c create rectangle 55 [expr 71 + ($i * 20)]  65 [expr 81 + ($i * 20)] \
		      -width 1 -outline black -fill $grisatre($i)]
        $c addtag point withtag $item
        $c create text 25 [expr 66 + ($i * 20)] -text [lindex $asi_effectifs $i] -anchor n\
	    -font 12
    }
}
#
# Procedure ASIDRAW_REGGRISE - debut

# Procedure MATOP - debut
#
proc matop {mat op } {
    #
    # 'op' de chaque element matriciel ('op' est une fonction mathematique sans les parentheses, par ex. 'log')
    set liste [vtrim $mat]
    #
    set result ""
    foreach item $liste {
        append result "[expr ($op ($item))] "
    }
    set opmat "2 [mrows $mat] [mcols $mat] $result"
    return $opmat
}
#
# Procedure MATOP - fin


proc print-histogram {counts limits} {
    foreach count $counts limit $limits {
	if { $limit != {} } {
	    #debug [format "<%12.4g\t%d" $limit $count]
	    set prev_limit $limit
	} else {
	    #debug [format ">%12.4g\t%d" $prev_limit $count]
	}
    }
}

# TCLLIBHIST  -- debut
#
# librarie tcllib: histogramme
#
proc tcllibhist {data cannevas} {
    # data <- signed differences
    #
    #console show
    set cannevas [expr {$cannevas+1}]
    toplevel .h$cannevas
    canvas .h$cannevas.plot
    pack   .h$cannevas.plot
    #
    #debug "Basic statistics:\n$data"
    set data [vtrim $data];# BUG fasym Mtpl May 2014
    set b [::math::statistics::basic-stats $data]
    foreach label {mean min max number stdev var} v $b {
	#debug "$label\t$v"
	append basic "\n$label\t$v"
    }
    #
    #debug "Print and Plot histograms"
    set min [lindex $b 1]
    set max [lindex $b 2]
    
    set limits [::math::statistics::minmax-histogram-limits $min $max]
    set histogram_data [::math::statistics::histogram $limits $data]
    set ordre [lsort -r -inc $histogram_data]
    set gauche [lindex $ordre 0]
    set droite [lindex $ordre end]
    #debug "Basic statistics: \n$basic\nlimits $limits, \nhistogram_data $histogram_data\ngauche $gauche droite $droite"
    ::math::statistics::plot-scale .h$cannevas.plot  $min $max $gauche $droite
    
    #debug "histogram_data $histogram_data"
    print-histogram $histogram_data $limits
    ::math::statistics::plot-histogram .h$cannevas.plot $histogram_data $limits
    return [list $basic, $limits, $histogram_data]
}





# Procedure SIGNEDIFF --- debut
#
# Appelee par STATIS, elle-meme appelee par la procedure 'PRINCIPAL'
# La procedure 'PRINCIPAL' 
# 1. entre le nb de sessions de mesures, et 2. appelle les donnees par GET_PWCS
#
proc signediff {pop sujet args} {
    global p S valores infile  sigdif_c sizest_c sizetransf_c
    
    #
    # sujet is either "Variable $v" or "Centroid size (CS)"
    #
    set caract [lindex $sujet 1]
    set S(nbdecol) $args
    #debug "SIGNEDIFF $S(nbdecol) \nsujet $sujet \tcolonne $caract \tpop :\n[show $pop]"
    set valores($S(nbdecol)) $pop
    #
    append S(sizereport) "\n$infile\n\n-----------\n$sujet\n"
    #
    # Appel (une deuxieme fois, helas) a la procedure SIZESCALING
    set correction [sizescaling $pop signediff]
    # sizescaling renvoie une liste de matrices: return_pop sigdif sizest sizetransf
    set pop_c [lindex $correction 0];# pas de correction
    set sigdif_c [lindex $correction 1]
    set sizest_c [lindex $correction 2]
    set sizetransf_c [lindex $correction 3]
    #
    
    # # -----------------------------------------------------------------------
    # # EMBEDDED BUTTON FOR SIZE / ASYMMETRY RELATIONSHIPS
    # # -----------------------------------------------------------------------
    
    # bouton cache car il ne fonctionne que pour la derniere variable d'un tableau
    # et pas le temps de programmer plus finement (Quito 2014)

    # $p insert end "\n\n"
    # destroy $p.embed_assiplot
    # button $p.embed_assiplot -text "Plot of asymmetry against size" -command {
    #     global sigdif_c sizest_c asi_effectifs sizetransf_c
    #     # On s'occupe d'abord du cannevas
    #     destroy .size_dep
    #     toplevel .size_dep
    #     # une frame en haut pour le canvas
    #     frame .size_dep.haut
    #     # une frame en bas pour les commentaires (texte)
    #     frame .size_dep.bas  -bg grey22
    #     set d .size_dep.bas
    #     # On pose les frames
    #     pack .size_dep.haut .size_dep.bas -side bottom -expand yes
    #     #
    #     global effectifs
    #     global magritte
    #     picasso
    #     #
    #     # Je prends les differences avec signe (Palmer & Strobeck l'indiquent ainsi)
    #     #
    #     set S(numero) "1"
    #     set sigdif $sigdif_c
    #     set sizest $sizest_c
    #     set sizetransf $sizetransf_c
    #     set asi_effectifs [mrows $sizest_c]
    #     set cannevas ".size_dep.haut.plot"
    #     #
    #     set xdata [vtrim $sigdif]
    #     set ydata [vtrim $sizest]
    #     set cannevas ".size_dep.haut.plot"
    #     asidraw_reggrise $ydata $xdata {(R-L)}
    #     #
    #     # Second graphique: raw pop
    #     #
    #     global pop_raw sigdif_raw sizest_raw sizetransf_raw
    #     #
    #     set S(numero) "2"
    #     set sigdif $sigdif_raw
    #     set sizest $sizest_raw
    #     set sizetransf $sizetransf_raw
    #     set xdata [vtrim $sigdif]
    #     set ydata [vtrim $sizest]
    #     set cannevas ".size_dep.haut.plot"
    #     asidraw_reggrise $ydata $xdata {(R-L)}
    
    # }
    # # fin de l'embedded button
    # #
    # $p insert end "\n\n\n"
    # $p window create {end lineend} -window $p.embed_assiplot
    # bind $p.embed_assiplot <Motion> {$p.embed_assiplot configure -cursor hand1}
    # $p insert end "\n"
    
    # # -----------------------------------------------------------
    # # EMBEDDED BUTTON FOR HISTOGRAM
    # # ------------------------------------------------------------
    # destroy $p.embed_eucl
    # $p insert end "\n"
    # button $p.embed_eucl -text "Histogramme des différences avec signe" -bg "light yellow" -command {
    #     # cannevas
    #     set numero "1"
    #     destroy .signed$numero
    #     toplevel .signed$numero
    #     wm geometry .signed$numero 550x450+[expr {405 + $numero*20}]+[expr {10 + $numero*25}]
    #     set c ".signed$numero"
    #     frame $c.f
    #     pack $c.f
    #     canvas $c.can -background "light yellow"
    #     label $c.f.labe -text "CS: signed diff. distrib." -fg blue
    #     button $c.f.data -text " d a t a " -command {affiche 600 200 [show $sigdif_raw]}
    #     button $c.f.ex -text " e x i t " -command {destroy .signed1}
    #     pack $c.f.labe $c.f.data $c.f.ex -side left -fill x -expand yes
    #     # Histogramme num. 1: valeurs entrees
    #     set sigdif $sigdif_raw
    #     set sizest $sizest_raw
    #     set sizetransf $sizetransf_raw
    #     set data1 [vtrim $sigdif]
    #     set data2 [vtrim $sigdif]
    #     if {[info exists valclass] == 0} {set valclass 10}
    #     # appel a la procedure histogramme
    #     histograbe_asi $data1 $valclass $c.can $sizetransf
    #     #
    #     # cannevas
    #     set numero "2"
    #     destroy .signed$numero
    #     toplevel .signed$numero
    #     wm geometry .signed$numero 550x450+[expr {405 + $numero*20}]+[expr {10 + $numero*25}]
    #     set c ".signed$numero"
    #     frame $c.f
    #     pack $c.f
    #     canvas $c.can -background "light yellow"
    #     label $c.f.labe -text "CS: signed diff. distrib." -fg blue
    #     button $c.f.data -text " d a t a " -command {affiche 600 200 [show $sigdif_raw]}
    #     button $c.f.ex -text " e x i t " -command {destroy .signed2}
    #     pack $c.f.labe $c.f.data $c.f.ex -side left -fill x -expand yes
    #     #
    #     # Histogramme num. 2: valeurs corrigees
    #     set sigdif $sigdif_c
    #     set sizest $sizest_c
    #     set sizetransf $sizetransf_c
    #     set data1 [vtrim $sigdif]
    #     set data2 [vtrim $sigdif]
    #     if {[info exists valclass] == 0} {set valclass 10}
    #     # appel a la procedure histogramme
    #     histograbe_asi $data1 $valclass $c.can $sizetransf
    
    # }
    # $p window create {end lineend} -window $p.embed_eucl
    # bind $p.embed_eucl <Motion> {$p.embed_eucl configure -cursor hand1}
    # -----------------------------------------------------------------
}
#
# Procedure SIGNEDIFF --- fin


# Procedure SIZESCALING --- debut
#
# plusieurs transformations de la taille,
# choix de la transformation qui minimise la correlation entre asymetrie et taille
#
proc sizescaling {pop args} {
    #
    # sujet is either "..." or "Centroid size (CS)"
    #
    global p S infile pt_raw
    #
    #
    #
    set sujet "Size "
    #
    set pop_g [mrange $pop 0 0 0 end]
    set pop_d [mrange $pop 1 1 0 end]
    set sigdif [msub $pop_g $pop_d]
    set sizest [mscale [madd $pop_g $pop_d] 0.5]
    set pop [join_cols $pop_g $pop_d]
    #if {$args != "signediff"} {debug "\nPOP non corrige -> sigdif $sigdif\nsizest -> $sizest"}
    
    global pop_raw sigdif_raw sizest_raw sizetransf_raw
    set pop_raw $pop
    set sigdif_raw $sigdif
    set sizest_raw $sizest
    set sizetransf_raw "raw size"
    
    # Transformations
    #
    # Diviser Ri et Li par une estimation de la taille - mscale the matrix
    #
    set ss "";# future matrice 'ss' de correction
    foreach {left right} "[vtrim $pop_g] [vtrim $pop_d]" {
        append ss "[expr {1 * 1 / ($left + $right)}] "
    }
    set ss "2 [mrows $pop_g] [mcols $pop_g] $ss "
    set ss_pop_g [mprod $ss $pop_g];# scaled left side
    set ss_pop_d [mprod $ss $pop_d];# scaled right side
    set ss_sigdif [msub $ss_pop_g $ss_pop_d];# signed side differences between scaled sizes
    # Une estimation de la taille (R+L)/2 doit etre faite pour chaque individu
    set ss_sizest [mscale [madd $ss_pop_g $ss_pop_d] 0.5];# scaled estimate of size
    set ss_pop [join_cols $ss_pop_g $ss_pop_d];# left ad right scaled sizes as a two-columns matrix
    #if {$args != "signediff"} {debug "ss -> sigdif $ss_sigdif\nsizest \n-> \n[show $ss_pop]"}
    #
    # Logarithme de chaque element matriciel, appel a la procedure MATOP
    set pop_gLN [matop $pop_g log]
    set pop_dLN [matop $pop_d log]
    set sigdifLN [msub $pop_gLN $pop_dLN]
    set sizestLN [mscale [madd $pop_gLN $pop_dLN] 0.5]
    set popLN [join_cols $pop_gLN $pop_dLN]
    #if {$args != "signediff"} {debug "LN of raw -> sigdif $sigdifLN\nsizest -> $sizestLN"}
    #
    # Square root of each element in matrix, appel a la procedure MATOP
    set pop_gSQRT [matop $pop_g sqrt]
    set pop_dSQRT [matop $pop_d sqrt]
    set sigdifSQRT [msub $pop_gSQRT $pop_dSQRT]
    set sizestSQRT [mscale [madd $pop_gSQRT $pop_dSQRT] 0.5]
    set popSQRT [join_cols $pop_gSQRT $pop_dSQRT]
    #if {$args != "signediff"} {debug "SQRT of raw -> sigdif $sigdifSQRT ->sizest -> $sizestSQRT"}
    #
    # Transformations
    
    # Correlations des matrices transformees (ou NON) avec la taille - start -
    #
    #---------------------------- pop non corrige ------- start
    set sizest [mscale [madd $pop_g $pop_d] 0.5]
    set corps [join_cols $sigdif $sizest]
    set R [correlation $corps]
    #----------------------------------------- raw ------- start
    set ss_sizest [mscale [madd $ss_pop_g $ss_pop_d] 0.5]
    set corps [join_cols $ss_sigdif $ss_sizest]
    set Rss [correlation $corps]
    #----------------------------------------- log ------- start
    #
    set sizestLN [mscale [madd $pop_gLN $pop_dLN] 0.5]
    set corps [join_cols $sigdifLN $sizestLN]
    set Rln [correlation $corps]
    #----------------------------------------- sqrt ------- start
    #
    set sizestSQRT [mscale [madd $pop_gSQRT $pop_dSQRT] 0.5]
    set corps [join_cols $sigdifSQRT $sizestSQRT]
    set Rsqrt [correlation $corps]
    #
    set rawR [vtrim [mrange $R 1 1 0 0]]
    set rawR2 [expr {pow($rawR,2)}]
    set ssR [vtrim [mrange $Rss 1 1 0 0]]
    set ssR2 [expr {pow($ssR,2)}]
    set lnR [vtrim [mrange $Rln 1 1 0 0]]
    set lnR2 [expr {pow($lnR,2)}]
    set sqrR [vtrim [mrange $Rsqrt 1 1 0 0]]
    set sqrR2 [expr {pow($sqrR,2)}]
    #
    # Statistical significance
    # 
    set ddl [expr [mrows $pop] - 2]
    set t_raw [expr ( $rawR*sqrt($ddl) ) / sqrt(1-$rawR2) ]
    set t_raw [format %1.2f $t_raw]
    set pt_raw [format %1.2f [expr {1 - [::math::statistics::cdf-students-t $ddl $t_raw]}]]
    set t_ss [expr ( $ssR*sqrt($ddl) ) / sqrt(1-$ssR2) ]
    set t_ss [format %1.2f $t_ss]
    set pt_ss  [format %1.2f [expr {1 - [::math::statistics::cdf-students-t $ddl $t_ss]}]]
    set t_ln [expr ( $lnR*sqrt($ddl) ) / sqrt(1-$lnR2) ]
    set t_ln [format %1.2f $t_ln]
    set pt_ln  [format %1.2f [expr {1 - [::math::statistics::cdf-students-t $ddl $t_ln]}]]
    set t_sqrt [expr ( $sqrR*sqrt($ddl) ) / sqrt(1-$sqrR2) ]
    set t_sqrt [format %1.2f $t_sqrt]
    set pt_sqrt [format %1.2f [expr {1 - [::math::statistics::cdf-students-t $ddl $t_sqrt]}]]
    #
    # Comments
    set comment "\n\nCorrelation coefficient (r) between signed differences (R-L) and size estimate (R+L)/2), with $ddl degrees of freedom\
            \nraw size values R and L: \t[show [vtrim [mrange $R 1 1 0 0]] %1.6f]\t t = $t_raw \t P = $pt_raw \
            \nscaled size values R/((R+L)/2)) and L/((R+L)/2)): \t[show [vtrim [mrange $Rss 1 1 0 0]] %1.6f]\t t = $t_ss \t P = $pt_ss\
            \nnatural logarithm ln(R) and ln(L): \t[show [vtrim [mrange $Rln 1 1 0 0]] %1.6f]\t t = $t_ln \t P = $pt_ln \
            \nsquare root sqrt(R) and sqrt(L)): \t[show [vtrim [mrange $Rsqrt 1 1 0 0]] %1.6f]\t t = $t_sqrt \t P = $pt_sqrt \n"
    append S(sizereport) "$sujet: ---- \n[show [mrange $pop 0 0 0 4]]...etc.\n$comment"
    #debug $S(sizereport)
    #if {$args != "signediff"} {bleu $comment; append S(sizereport) $comment}
    #
    
    # Choix de la transformation de la correlation la plus petite / sigdif le plus interessant pour le graphique
    #
    
    # valeurs absolues pour le choix du coefficient 't' le plus petit
    set a_rawR [expr {abs([vtrim [mrange $R 1 1 0 0]])}]
    set a_ssR [expr {abs([vtrim [mrange $Rss 1 1 0 0]])}]
    set a_lnR [expr {abs([vtrim [mrange $Rln 1 1 0 0]])}]
    set a_sqrR [expr {abs([vtrim [mrange $Rsqrt 1 1 0 0]])}]
    # selection des valeurs associees au choix, et retour a l'ANOVA
    set listeR [list "$a_rawR raw" "$a_ssR ss" "$a_lnR ln" "$a_sqrR sqrt"]
    set ordre_listeR [lsort -r -index 0 $listeR]
    set choix [lindex [lindex $ordre_listeR 0] 1]
    #choix à présent fixé = non $choix mais $a_rawR"
    set choix "raw"
    # CHOIX, RETURN
    # switch -- $choix {
    #     raw {
             set sizest $sizest
             set sigdif $sigdif
             set sizetransf "raw size"
             set r $rawR
             set return_pop $pop
             return [list $return_pop $sigdif $sizest $sizetransf]
    #     }
    #     ss {set sizest $ss_sizest
    #         set sigdif $ss_sigdif
    #         set sizetransf "raw size values divided by size"
    #         set r $ssR
    #         set return_pop $ss_pop
    #         return [list $return_pop $sigdif $sizest $sizetransf]
    #     }
    #     ln {set sizest $sizestLN
    #         set sigdif $sigdifLN
    #         set sizetransf "raw size values divided by size"
    #         set r $lnR
    #         set return_pop $popLN
    #         return [list $return_pop $sigdif $sizest $sizetransf]
    #     }
    #     sqrt {set sizest $sizestSQRT
    #         set sigdif $sigdifSQRT
    #         set sizetransf "raw size values divided by size"
    #         set r $sqrR
    #         set return_pop $popSQRT
    #         return [list $return_pop $sigdif $sizest $sizetransf]
    #     }
    #     default {type "Bug in Sizescaling procedure, at selecting smallest R"; exit}
    # }
}
#
# Procedure SIZESCALING --- fin

#
proc pause {time} {
    set wait 0
    after [expr {$time*1000}] {set ::wait 1}
    vwait wait
}



# Procedure VARCOV - debut
#
proc varcov  {newpop} {
    set newcolonnes [mcols $newpop]
    set newlignes [mrows $newpop]
    # ... CREER UN VECTEUR AVEC AUTANT DE 1 QUE DE VARIABLES ...
    # IL SERVIRA POUR CREER UNE MATRICE CENTREE au lieu de NORMALISEE
    set popones "2 1 $newcolonnes 1"
    set x 1
    set y $newcolonnes
    while {$x < $y} {lappend popones 1; set x [expr {$x + 1} ] }
    # CALCULER MOYENNES ET DEVIATIONS STANDARD DES VARIABLES, et ...
    mnorms_br newpop newpopmeans newpopsigmas
    #
    # CENTERED DATA
    #
    set Z [mnormalize $newpop $newpopmeans $popones]
    #
    # DEGREES of FREEDOM
    #
    set denom [expr {$newlignes -1}]
    set scale [expr {1.0 / $denom}]
    #
    # MATRICE des SUMS of SQUARES
    #
    set ZZ [mmult [transpose $Z] $Z]
    #
    # MATRICE des VARIANCES - COVARIANCES du GROUPE
    #    
    set ZZsc [mscale $ZZ $scale]
    return $ZZsc
}
#
# Procedure VARCOV - fin

# Procedure STATIS --- debut
#
proc statis {indimed} {
    #
    # "indimed" est une liste qui contient deux elements: 'nb. indiv.' et 'nb. de sessions de mesures'
    #
    global p S infile
    global mes;# variable indicee (une par digitalisation)
    global nbindiv r mediciones n lado c
    global partialw nbvar
    #
    $p insert end  "\n[lindex $indimed 0] \tIndividuals\n[lindex $indimed 1]\tDigitizations  \n"
    #
    # Sokal & Rohlf, Two-way ANOVA with replication, Biometry Third Edition, p.324
    #
    # 'mixed' two-way ANOVA because 'side' is fixed, 'individual' is not
    #
    # Nombre d'individus dans nbindiv, ou 'r' pour suivre les symboles de S&R
    set nbindiv [lindex $indimed 0]
    set r $nbindiv
    # Nombre de mesures dans 'mediciones',  ou 'n' pour suivre les symboles de S&R
    set mediciones  [lindex $indimed 1]
    set n $mediciones
    # Nombre de cotes (droite et gauche) dans  'lado',
    set lado 2
    set c $lado
    
    # Compter le nombre de variables a traiter (cfr. p. ex. un fichier _pw_cs.txt)
    set nbvar [mcols $mes(1)]
    
    rouge "mes(1) \n[show $mes(1)]"
    
    # Selectionner les variables une a une, chacune dans une matrice separee, doublement indexee
    for {set j 0} {$j < $mediciones} {incr j} {
        for {set i 0} {$i < $nbvar} {incr i} {
            set mat_var($i,$j) [mrange $mes($j) $i $i]
            #debug "\nmat_var($i,$j) \n[show $mat_var($i,$j)]"
        }
    }
    
    # Extraire une matrice pop de chaque variable
    #
    #debug "Nombre de variables $nbvar"
    for {set v 0} {$v < $nbvar} {incr v} {
        #
        set cote 0
        for {set indiv 0} {$indiv < $nbindiv} {incr indiv} {
            #debug "\n\nINDIV = $indiv (sur les $nbindiv)\n"
            for {set j 0} {$j < $mediciones} {incr j} {
                #debug  "\nJ DIGITIZATION = $j \n\nVariable \n[show $mat_var($v,$j)]\n\n"
                for {set m 0} {$m < $lado} {incr m} {
                    set q [expr $m + $cote]
                    #debug " <- side = $cote q = $q \nmrange mat_var($v,$j) 0 0 $q $q\n[show [mrange $mat_var($v,$j) 0 0 $q $q]]\n"
                    set extrait  [mrange $mat_var($v,$j) 0 0 $q $q]
                    #debug "\nextrait [lindex $extrait 3] "
                    append pre_pop($v) "[lindex $extrait 3] "
                }
            }
            incr cote 2
        }
        #$p insert end "\n"
        set pre_mat "2 [expr {$nbindiv*$mediciones}] 2 "
        #set pre_mat "2 $nbindiv 2 "
        
        set partialw $v
        set pop $pre_mat$pre_pop($v)
        #
        #debug "\n Nouvelle matrice\n$pop"
	if {[string first "_PW_CS" $infile] > 0} {type "You should enter a file containing PARTIAL WARPS only (not PW and CS)";exit}
	# S(pwcs), S(pw) sont définis aux boutons d'input Single (shape) Single(landmarks)
	# 
	
	if {$S(pwcs)==1} {
            if {$partialw > 0} {
                set sujet "Partial Warp $partialw"
            } else {
                set sujet "SIZE"
                # Calcul des differences avec signe, leur moyenne et leur distribution (usage de statistics)
                signediff $pop $sujet 0
            }
        } else  {
	    # either Separate (size) or Single(shape) buttons
	    if {$S(pw)==1} {
		set sujet "Shape variable $partialw"
	    } else {
		#set sujet "Variable $v"
		set sujet "Variable [expr {$v+1}]"
		# Calcul des differences avec signe, leur moyenne et leur distribution (usage de statistics)
		signediff $pop $sujet $v
	    }
	}
        #
        # Faire l'ANOVA 
        anova $pop $sujet;# procedure suivante
        #
        # Passer a la variable suivante
    }
    
}
#
# Procedure STATIS - fin


# Procedure ANOVA -- debut
#
proc anova {pop args} {
    
    global p S nbindiv r mediciones n lado c infile partialw nbvar
    global procrustes_ss_model procrustes_ss_indiv procrustes_ss_side procrustes_ss_interact procrustes_ss_residual
    global pt_raw
    #
    if {[string first "Corrected" $args] > 0} {
        #debug "ANOVA args = $args, $pt_raw"
        if {$pt_raw > 0.05} {return}
    }
    # On continue les statistiques generales
    set colonnes [mcols $pop]
    set lignes [mrows $pop]
    #
    #
    rouge "\nMIXED TWO-WAY ANOVA on $args"
    #$p insert end "\nind1.side1 \t ind1.side2  \t <- first measurement\
	\nind1.side1 \t ind1.side2  \t <- repeated measurement\
	\nind2.side1 \t ind2.side2  \t <- first measurement\
	\nind2.side1 \t ind2.side2  \t <- repeated measurement\
	\n... etc\
	"
    #$p insert end "\n(please adapt reading if more than two sessions of measurements)"
    rouge "\n[show $pop]\n"
    
    # Moyennes generales des colonnes
    mnorms_br pop popmeans popsigmas
    
    # Total de chaque colonne
    set total_col [msum $pop]
    #bleu "\nTotal de chaque colonne "
    #$p insert end "\n[show $total_col %8.3f]"
    
    # Total general
    set total [msum [msum $pop]]
    set total_general [lindex $total 3 end]
    #bleu "\n\nTotal general :"
    $p insert end "\nTotal general : \n[show $total %8.3f]"
    
    # Et pour respecter les symboles de Sokal & Rohlf, Two-way ANOVA with replication, Biometry Third Edition, p.324
    set y2b [expr {$total *  1.0/($r*$c*$n)}]
    bleu "\n\nGrand mean / Promedio total / Moyenne generale"
    bleu "\n$y2b\n"
    #$p insert end  "\n$y2b\ni.e. Total ([show $total %8.3f])/ ($r * $c * $n)\n"
    
    #bleu "\n\nMoyennes des colonnes"
    #$p insert end "\n[show $popmeans %8.3f]"
    #bleu "\nVariances des colonnes"
    #$p insert end "\n[show $popsigmas %8.3f]\n"
    #
    #
    
    # ==================================================
    # Rappel theorique -- start - Sokal & Rohlf, Biometry, Third edition, p.324
    # ==================================================
    # quantity 1 : moyenne totale (1/r.c.n) . (SrScSnY)
    #     SrY signifie la somme des y pour les r individus, i.e. 2 dans S&R, p. 324 ...
    #     ScY signifie la somme des y pour les c cotes de l'insecte, i.e. 2 ...
    #     SnY signifie la somme des y pour les n mesures (repetees) , i.e. 3 dans S&R, p. 324 ...
    # quantity 2 : SS_subgr = n.Src (moy.subgr/cote - moy.tot.)2
    #     pour ASI, le nombre de subgr correspond au nombre d'individus
    #     les subgr/cote sont ici les moyennes de chaque individu pour chaque cote (de l'insecte)
    #     sur l'ensemble des mesures (repetitions)
    #         s'il y a 2 individus, il y a quatre SS_subgr/cote
    #         soit 'r' individus multiplies par les 2 cotes : 'rc'
    #     Src signifie la somme sur les r individus et leurs c cotes, i.e. 2x2=4 dans S&R, p. 324 ...
    # quantity 3 : SS_within = Src Sn(Y-moy.subgr/cote)
    #
    # quantity 4 : SS_total = SS_subgr + SS_within
    #
    # quantity 5 : SS_rows = Sr c.n.(moy.indiv - moy.tot.)
    #     Cette SS correspond aux individus (subgroups), on retire la moy. totale
    #     des moyennes de chaque individu sur la totalite des repetitions et les deux cotes
    # quantity 6 : SS_columns = Sc r.n.(moy.cotes - moy.tot.)
    #     Cette SS correspond aux deux cotes de l'insecte, les moyennes gauche et droite,
    #     (moyennes sur toutes les repetitions et tous les individus), dont on retire la moy. totale
    # quantity 7 : SS_interact
    #     la SS correspondant a l'interaction "side X indiv" s'obtient en faisant la difference
    #     entre la SS_subgr (celle correspondant au 'MODEL' des ouputs STATA)
    #     et la somme des SSside (SS_columns) et SSindiv (SS_rows)
    #
    # ==================================================
    # Rappel theorique -- fin
    # ==================================================
    
    
    
    # ----------- EXTRACTION DES SOUS-GROUPES -----------------------------
    # Pour la droite, et pour la gauche, autant de sous-groupes que d'individus,
    #     ou 'r' pour suivre les symboles de Sokal & Rohlf, Two-way ANOVA with replication, Biometry Third Edition, p.324
    # Chaque sous-groupe contient un nombre 'mediciones', ou 'n', de lignes.
    # Donc r*n c'est le nombre d'individus multiplie par le nombre de mesures,
    #     ou 'nbindiv' * 'mediciones', ou encore 'lignes' <- c'est sur ca ?? et la g et la d alors ??
    for {set i 0} {$i < [expr {$r*$n}]} {incr i $n} {
        #rouge "\nProcedure ANOVA\tmrange pop 1 1 $i [expr {$i+ ($n-1)}]"
        set subgroup_d($i) [mrange $pop 1 1 $i [expr {$i+ ($n-1)}] ]
        #debug "\nIndividu [expr {$i*1/$n}], cote droit: \n$subgroup_d($i) <- $lignes ligne=[expr {$r*$n}]"
    }
    for {set i 0} {$i < $lignes} {incr i $n} {
        set subgroup_g($i) [mrange $pop 0 0 $i [expr {$i+($n-1)}]]
        #debug "\nIndividu [expr {$i*1/$mediciones}], cote gauche: \n$subgroup_g($i) <- $lignes lignes=[expr {$r*$n}]"
    }
    # ------------ EXTRACTION DES SOUS-GROUPES -----------------------------
    
    
    
    # ------- GROUP MEANS, ou moyennes des sous-groupes ------------- debut
    #
    # A gauche, puis a droite, moyenne des mesures par individu
    #
    # # A gauche
    for {set i 0} {$i < [expr {$r*$n}]} {incr i $n} {
        mnorms_br subgroup_g($i) subgr_mean_g($i) subgr_sigma_g($i)
    }
    # # A droite
    for {set i 0} {$i < [expr {$r*$n}]} {incr i $n} {
        mnorms_br subgroup_d($i) subgr_mean_d($i) subgr_sigma_d($i)
    }
    
    set subgroups_means "2 $nbindiv $lado "
    for {set i 0} {$i < [expr {$r*$n}]} {incr i $n} {
        append subgroups_means "[lindex $subgr_mean_g($i) 3] [lindex $subgr_mean_d($i) 3] "
    }
    #debug "\n\nGroup means / Promedios por grupos / Moyennes par groupes\nsoit ici par individus, a gauche (mes 1 + mes 2 + ... + mes n)/n  et a droite (mes 1 + mes 2 + ... + mes n)/n"
    #$p insert end "\n[show $subgroups_means]"
    #
    # ---------------- GROUP MEANS, ou moyennes des sous-groupes ---------- fin
    
    
    
    
    # -------- ROW means, ou moyennes des moyennes des sous-groupes ----- debut
    #
    # Il s'agit des moyennes correspondant a chaque individu (1 indiv = 1 subgr),
    # moyenne sur la totalite des repetitions et incluant les deux cotes
    for {set i 0} {$i < [expr {$r*$n}]} {incr i $n} {
        set gauche [lindex $subgr_mean_g($i) 3]
        set droite [lindex $subgr_mean_d($i) 3]
        set row_means($i) [expr {($gauche + $droite) * 1.0 / $colonnes}]
    }
    #bleu "\n\nRow means / promedios de los promedios de grupos / Moyennes des moyennes des groupes\
	(par individu, la moyenne des deux colonnes ci-dessus)"
    #for {set i 0} {$i < [expr {$r*$n}]} {incr i $n} {
    #    rouge "\ngroupe (individu) [expr {$i*1/$n}]:\t"
    #    $p insert end "$row_means($i)"
    #}
    # En faire une matrice pour le calcul, plus loin, de SS_rows
    # ATTENTION append ...
    set row_m "2 $nbindiv 1 "
    for {set i 0} {$i < [expr {$r*$n}]} {incr i $n} {
        append row_m "$row_means($i) "
    }
    #debug
    #$p insert end "\nPour le calcul de SS_rows: liste des moyennes de moyennes des sous-groupes\n$row_m\n"
    #
    # ------ROW means, ou moyennes des moyennes des sous-groupes --------- fin
    
    
    
    # ---------------- SS_subgr  ---------------- debut
    #
    # Exercice
    # 1. ENLEVER un scalaire de chaque element matriciel
    set op "expr $y2b -"
    set pre_ss_subgr [mat_unary_op $subgroups_means $op]
    #debug "\n\nEXERCICE: Moins $y2b partout dans la matrice des 'subgroups_means'...\n[show $pre_ss_subgr %8.3f]\n"
    
    # # Exercice
    # 2. Elever au CARRE chaque element d'une matrice
    set pro_ss_subgr [mat_binary_op $pre_ss_subgr  $pre_ss_subgr  {*}]
    #debug "\n\nEXERCICE: Elever au CARRE chaque element de la matrice, soit\n[show $pro_ss_subgr]"
    
    # # Exercice
    # 3. Faire la somme de tous les elements d'une matrice (Total general)
    set total [msum [msum $pro_ss_subgr]]
    #debug "\n\nEXERCICE: faire la somme de tous les elements d'une matrice: \n$total <- S(y1b - y2b)"
    
    #bleu "\n\nSS_subgr = n * S(y1b - y2b) <- n is the number of measurements, i.e. the number of lines within each subgroup"
    set ss_subgr [expr $n * $total]
    
    rouge "\n\n-------------------------------------------------------------"
    rouge "\nSS_subgr: (Model)"
    #rouge "\n'Grand mean' has been substracted from 'Sub-group means', \
	#        \nthe differences have been squared, then summed,\
	#        \nand this sum has been multiplied by number of measurements\
	#        \n('Sub-group means' are the average across measurements for each side of one individual)"
    $p insert end "\n$ss_subgr"
    rouge "\n------------------------------------------------------------"
    
    #
    # ---------------- SS_subgr  ---------------- fin
    
    # --------------- SS_within ----------------- debut
    #
    set sub_ss_w_d 0
    set sub_ss_w_g 0
    for {set i 0} {$i < [expr {$r*$n}]} {incr i $n} {
	#bleu "\n\nIndividu [expr {($i*1/$n)+1}], cote droit:"
	#$p insert end "\n[show $subgroup_d($i)]"
	#bleu "\nIndividu [expr {($i*1/$n)+1}], cote gauche:"
	#$p insert end "\n[show $subgroup_g($i)]"
	
	#rouge "\nOn retire [lindex $subgr_mean_d($i) 3] a droite et  [lindex $subgr_mean_g($i) 3] a gauche"
	
	# Exercice
	# 1. ENLEVER un scalaire de chaque element matriciel de chaque matrice
	#
	set op "expr [lindex $subgr_mean_d($i) 3] -"
	set pre_ss_w_d($i) "[mat_unary_op $subgroup_d($i) $op] "
	#debug "\npre_ss_w_d($i) "
	#$p insert end "\n[show $pre_ss_w_d($i)]\nOn a enleve \n[show $subgr_mean_d($i)] de \n[show $subgroup_d($i)] "
	#
	set op "expr [lindex $subgr_mean_g($i) 3] -"
	set pre_ss_w_g($i) "[mat_unary_op $subgroup_g($i) $op] "
	#debug "\npre_ss_w_g($i) "
	#$p insert end "\n[show $pre_ss_w_g($i)]\nOn a enleve \n[show $subgr_mean_g($i)] de \n[show $subgroup_g($i)] "
	
	# # Exercice
	# 2. Elever au CARRE chaque element d'une matrice
	#rouge "\nOn eleve au carre"
	set pro_ss_w_d($i) [mat_binary_op $pre_ss_w_d($i)  $pre_ss_w_d($i)  {*}]
	set sub_ss_w_d [expr $sub_ss_w_d + [msum $pro_ss_w_d($i)]]
	#bleu "\npro_ss_w_d($i) et sub_ss_w_d"
	#$p insert end "\n[show $pro_ss_w_d($i)]\nEt somme = $sub_ss_w_d"
	set pro_ss_w_g($i) [mat_binary_op $pre_ss_w_g($i)  $pre_ss_w_g($i)  {*}]
	set sub_ss_w_g [expr $sub_ss_w_g + [msum $pro_ss_w_g($i)]]
	#bleu "\npro_ss_w_g($i) et sub_ss_w_g"
	#$p insert end "\n[show $pro_ss_w_g($i)]\nEt somme = $sub_ss_w_g"
    }
    
    
    set ss_w [expr {$sub_ss_w_g + $sub_ss_w_d}]
    #rouge "\n-------------------------------------------------------------"
    rouge "\nSS_within: (residual, i.e. error) \t"
    $p insert end "$ss_w"
    rouge "\n------------------------------------------------------------------"
    #
    # -------------- --------------- SS_within ----------------- fin
    
    
    
    # --------------- SS_total ----------------- debut
    #
    set ss_tot [expr {$ss_subgr + $ss_w}]
    #rouge "\n------------------------------------------------------------"
    rouge "\nSS_total : (Total) \t"
    $p insert end "$ss_tot"
    rouge "\n-------------------------------------------------------------"
    #
    # --------------- SS_total ----------------- fin
    
    
    
    # --------------- SS_rows ----------------- debut
    #
    # SS (due to individuals) - Utilise la matrice des moyennes des moyennes de sous-groupes (row_m)
    #
    
    # Exercice
    # 1. ENLEVER un scalaire de chaque element matriciel d'une matrice
    #
    set op "expr $y2b -"
    set pre_ss_rows "[mat_unary_op $row_m $op] "
    #debug "\npre_ss_rows  \n[show $pre_ss_rows]"
    
    # Exercice
    # 2. Elever au CARRE chaque element d'une matrice
    #rouge "\nOn eleve au carre"
    set pro_ss_rows [mat_binary_op $pre_ss_rows  $pre_ss_rows  {*}]
    #debug "\npro_ss_rows  \n[show $pro_ss_rows]"
    
    # Exercice 1 modifie
    # 1modifie: multiplier par un scalaire chaque element d'une matrice
    set cn [expr {$colonnes * $n}];# n = nombre de mesures faites, ou nombre de lignes par sous-groupe.
    set op "expr $cn *"
    set presque_ss_rows "[mat_unary_op $pro_ss_rows $op] "
    #debug "\npresque_ss_rows  \n[show $presque_ss_rows]"
    
    # # Exercice
    # 3. Faire la somme de tous les elements d'une matrice a une colonne
    #
    #debug "\nICI -->> \n[show $presque_ss_rows]"
    #
    set ss_rows [msum $presque_ss_rows]
    #rouge "\n-----------------------------------------------------------------"
    rouge "\nSS_rows: (indiv) \t"
    $p insert end "$ss_rows"
    rouge "\n------------------------------------------------------------------"
    #
    # --------------- SS_rows ------(indiv)----------- fin
    
    
    
    
    # --------------- SS_cols -------(side)---------- fin
    #
    # SS (due to individuals) - Utilise la matrice des moyennes des moyennes de sous-groupes (row_m)
    #
    # #
    # Il s'agit des moyennes correspondant a chaque cote (1 cote = 1 colonne, cfr. p.324 de S&R),
    # moyenne sur la totalite des repetitions et incluant tous les individus
    
    # Exercice
    # 1. ENLEVER un scalaire de chaque element matriciel d'une matrice
    #
    set op "expr $y2b -"
    set pre_ss_cols "[mat_unary_op $popmeans $op] "
    #debug "\npre_ss_cols  $y2b - $popmeans \n[show $pre_ss_cols]"
    
    # Exercice
    # 2. Elever au CARRE chaque element d'une matrice
    #rouge "\nOn eleve au carre"
    set pro_ss_cols [mat_binary_op $pre_ss_cols  $pre_ss_cols  {*}]
    #debug "\npro_ss_cols  \n[show $pro_ss_cols]"
    
    # Exercice 1 modifie
    # 1modifie: multiplier par un scalaire chaque element d'une matrice
    set rn [expr {$r * $n}];# r = nombre d'individus, soit le nombre de sous-groupes.
    set op "expr $rn *"
    set presque_ss_cols "[mat_unary_op $pro_ss_cols $op] "
    #debug "\npresque_ss_cols  \n[show $presque_ss_cols]"
    
    # # Exercice
    # 3. Faire la somme de tous les elements d'une matrice a une colonne
    set ss_cols [msum $presque_ss_cols]
    #rouge "\n-----------------------------------------------------------"
    rouge "\nSS_cols:  (side)   \t"
    $p insert end "$ss_cols"
    rouge "\n------------------------------------------------------------"
    #
    # ---------------------------- SS_columns ------------(side)------- fin
    
    
    # ---------------------------- SS_interaction ----(side*indiv)---- debut
    #
    # quantity 2 - quantity 5 - quantity 6, soit : SS_subgr - SS_rows - SS_cols
    #
    # l'interaction entre "indiv" (5) et "side" (6) est evaluee par "2 (Model) - 5 (indiv) - 6 (side)"
    # je rappelle que le Model, c'est SSsubgr soit (la moy. de chaque gr. - la moy. totale )2 multiplie par 'n' (cfr. 602)
    #
    set ss_interact [expr {$ss_subgr - $ss_rows - $ss_cols}]
    #rouge "\n-------------------------------------------------"
    rouge "\nSS_interact: (side*indiv) \t"
    $p insert end "$ss_interact"
    #rouge "\n-------------------------------------------------"
    
    #
    # ---------------------------- SS_interaction ----(side*indiv)---- fin
    
    #$p configure -tabs "3c left 6c left 16c left"
    
    set ddf_model  [expr {($nbindiv * $lado) - 1}]
    set ddf_indiv  [expr {$nbindiv - 1}]
    set ddf_side  [expr {$lado - 1}]
    set ddf_interact  [expr {($nbindiv - 1) * ($lado-1) }]
    set ddf_residual [expr {$nbindiv * $lado * ($mediciones - 1)}]
    
    rouge "\n-------------------------------------degrees of freedom-"
    $p insert end "\nddf for MODEL"
    bleu "\n$ddf_model\t(rc-1) "
    $p insert end "\nddf for individuals"
    bleu "\n$ddf_indiv\t(r-1)"
    $p insert end "\nddf for sides (here, directional asymmetry)"
    bleu "\n$ddf_side\t(c-1)"
    $p insert end "\nddf for interaction (actually here, non-directional asymmetry)"
    bleu "\n$ddf_interact\t(r-1) * (c-1)"
    $p insert end "\nddf for error"
    bleu "\n$ddf_residual\trc(n-1)"
    rouge "\n----------------------------------------------------------"
    
    set ms_model  [expr {$ss_subgr * 1.0 / $ddf_model}]
    set ms_indiv  [expr {$ss_rows * 1.0 / $ddf_indiv}]
    set ms_side [expr {$ss_cols * 1.0 / $ddf_side}]
    set ms_interact [expr {$ss_interact * 1.0 / $ddf_interact}]
    set ms_residual [expr {$ss_w * 1.0 / $ddf_residual}]
    
    bleu "\n-------------------------------------------Mean squares---"
    $p insert end "\nMS for MODEL"
    rouge "\n$ms_model\t(rc-1) = $ddf_model"
    $p insert end "\nMS for individuals"
    rouge "\n$ms_indiv\t(r-1) = $ddf_indiv"
    $p insert end "\nMS for sides (here, directional asymmetry)"
    rouge "\n$ms_side\t(c-1) = $ddf_side"
    $p insert end "\nMS for interaction (here, non-directional asymmetry)"
    rouge "\n$ms_interact\t(r-1) * (c-1) = $ddf_interact"
    $p insert end "\nMS for error"
    rouge "\n$ms_residual\trc(n-1) = $ddf_residual"
    bleu "\n------------------------------------------------------------"
    
    #
    if {$S(pw)==1 || $S(pwcs) == 1} {
	# on engrange les sommes procrusteennes
	if {$partialw > 0} {
	    set procrustes_ss_model [expr {$procrustes_ss_model+ $ss_subgr}]
	    set procrustes_ss_indiv [expr {$procrustes_ss_indiv+ $ss_rows}]
	    set procrustes_ss_side [expr {$procrustes_ss_side+ $ss_cols}]
	    set procrustes_ss_interact [expr {$procrustes_ss_interact+ $ss_interact}]
	    set procrustes_ss_residual [expr {$procrustes_ss_residual+ $ss_w}]
	} else  {
	    set procrustes_ss_model 0
	    set procrustes_ss_indiv 0
	    set procrustes_ss_side 0
	    set procrustes_ss_interact 0
	    set procrustes_ss_residual 0
	}
    }
    
    set f_model [format %8.2f [expr {$ms_model * 1.0 / $ms_residual}]]
    set f_indiv [format %8.2f [expr {$ms_indiv * 1.0 / $ms_interact}]]
    #set f_side [format %8.2f [expr {$ms_side * 1.0 / $ms_residual}]]
    set f_side [format %8.2f [expr {$ms_side * 1.0 / $ms_interact}]];# Palmer & Strobec, 1986 p. 406
    set f_interact [format %8.2f [expr {$ms_interact * 1.0 / $ms_residual}]]
    
    bleu "\n----------------------------------F statistic----"
    $p insert end "\nF for MODEL"
    rouge "\n$f_model (with df = $ddf_model)"
    $p insert end "\nF for individuals"
    rouge "\n$f_indiv (with df = $ddf_indiv)"
    $p insert end "\nF for sides (directional asymmetry)"
    rouge "\n$f_side (with df = $ddf_side)"
    $p insert end "\nF for interaction (non-directional asymmetry)"
    rouge "\n$f_interact (with df = $ddf_interact)"
    bleu "\n-------------------------------------------------"
    
    # Calcul des significations statistiques
    set a $ddf_model
    set b $ddf_residual
    set cc [expr {$ms_model * 1.0 / $ms_residual}]
    #debug "a = $a\nb=$b\ncc=$cc\n------------------------\n"
    set P_model  [::math::statistics::Cdf-toms322 $a $b $cc]
    #	
    set a $ddf_indiv
    set b $ddf_residual
    set cc [expr {$ms_indiv * 1.0 / $ms_interact}]
    #debug "a = $a\nb=$b\ncc=$cc\n------------------------\n"
    set P_indiv  [::math::statistics::Cdf-toms322 $a $b $cc]
    #	
    set a $ddf_side
    set b $ddf_interact;# Palmer & Strobec, 1986 p. 406
    set cc [expr {$ms_side * 1.0 / $ms_interact}];# Palmer&Strobecq,1986 p.406
    #debug "a = $a\nb=$b\ncc=$cc\n------------------------\n"
    set P_side  [::math::statistics::Cdf-toms322 $a $b $cc]
    #
    set a $ddf_interact
    set b $ddf_residual
    set cc [expr {$ms_interact * 1.0 / $ms_residual}]
    #debug "a = $a\nb=$b\ncc=$cc\n------------------------\n"
    set P_interact  [::math::statistics::Cdf-toms322 $a $b $cc]
    
    $p configure -tabs "3c left 6c center 7c center 10c center 12c left"
    $p insert end "\n"
    $p insert end "\n----------- ANOVA output ------ $args ----------------"
    $p insert end "\nSource \t SS \t df \t MS \t F \t Signification"
    $p insert end "\n------------------------------------------------------"

    $p insert end "\nModel \t [format %8.4f $ss_subgr] \t $ddf_model \t  [format %8.6f $ms_model] \t  $f_model \t [format %8.4f [expr {1-$P_model}]]"

    $p insert end "\nindiv \t [format %8.4f $ss_rows] \t $ddf_indiv \t  [format %8.6f $ms_indiv] \t  $f_indiv \t [format %8.4f [expr {1-$P_indiv}]]"

    $p insert end "\nside \t [format %8.4f $ss_cols] \t $ddf_side \t  [format %8.6f $ms_side] \t  $f_side \t [format %8.4f [expr {1-$P_side}]]"

    $p insert end "\nside*i \t [format %8.4f $ss_interact] \t $ddf_interact \t  [format %8.6f $ms_interact] \t  $f_interact \t [format %8.4f [expr {1-$P_interact}]]"

    $p insert end "\nResidu\t [format %8.4f $ss_w] \t  $ddf_residual \t [format %8.6f $ms_residual] \t"

    $p insert end "\n------------------------------------------------------"
    #debug "$args ... ??"
    if {[string first "SIZE" $args] >= 0 || [string first "values" $args] > 0 || [string first "Variable" $args] >= 0} {
	#$p configure -tabs "3c left 6c center 7c center 10c center 12c left"
	append S(sizereport) "\n"
	append S(sizereport) "\n----------------- ANOVA output ---- $args -"
	append S(sizereport) "\nSource \t SS \t df \t MS \t F \t Signific."
	append S(sizereport) "\n-------------------------------------------"
	append S(sizereport)  "\nModel \t [format %8.4f $ss_subgr] \t $ddf_model \t [format %8.6f $ms_model] \t$f_model \t[format %8.4f [expr {1-$P_model}]]"
	append S(sizereport)  "\nindiv \t [format %8.4f $ss_rows] \t $ddf_indiv \t [format %8.6f $ms_indiv] \t$f_indiv \t[format %8.4f [expr {1-$P_indiv}]]"
	append S(sizereport)  "\nside \t [format %8.4f $ss_cols] \t $ddf_side \t [format %8.6f $ms_side] \t$f_side \t[format %8.4f [expr {1-$P_side}]]"
	append S(sizereport)  "\nside*i \t [format %8.4f $ss_interact] \t $ddf_interact \t [format %8.6f $ms_interact] \t$f_interact \t[format %8.4f [expr {1-$P_interact}]]"
	append S(sizereport)  "\nResidu\t [format %8.4f $ss_w] \t $ddf_residual \t [format %8.6f $ms_residual] \t"
	append S(sizereport)  "\n------------------------------------------"
    }
    
    #
    if {$S(pw)==1 || $S(pwcs)==1} {
	# Ecriture des sommes procrusteennes
	#debug "partialw $partialw <-> nbvar $nbvar"
	if {[expr {$partialw + 1}] == $nbvar} {
	    # Procrustes ddf
	    if {$S(pwcs)==1} {
		set procrustes_ddf_model [expr {$partialw * $ddf_model}];# oui si PWCS
		set procrustes_ddf_indiv [expr {$partialw * $ddf_indiv}]
		set procrustes_ddf_side [expr {$partialw * $ddf_side}]
		set procrustes_ddf_interact [expr {$partialw * $ddf_interact}]
		set procrustes_ddf_residual [expr {$partialw * $ddf_residual}]
	    } else {
		set procrustes_ddf_model [expr {$nbvar * $ddf_model}];# oui si PW
		set procrustes_ddf_indiv [expr {$nbvar * $ddf_indiv}]
		set procrustes_ddf_side [expr {$nbvar * $ddf_side}]
		set procrustes_ddf_interact [expr {$nbvar * $ddf_interact}]
		set procrustes_ddf_residual [expr {$nbvar * $ddf_residual}]
	    }
	    # Procrustes MS
	    set procrustes_ms_model [expr {$procrustes_ss_model * 1.0/ $procrustes_ddf_model}]
	    set procrustes_ms_indiv [expr {$procrustes_ss_indiv * 1.0/ $procrustes_ddf_indiv}]
	    set procrustes_ms_side [expr {$procrustes_ss_side * 1.0/ $procrustes_ddf_side}]
	    set procrustes_ms_interact [expr {$procrustes_ss_interact * 1.0/ $procrustes_ddf_interact}]
	    set procrustes_ms_residual [expr {$procrustes_ss_residual * 1.0/ $procrustes_ddf_residual}]
	    # Procrustes F (but non parametric methods are recommended)
	    set procrustes_f_model [expr {$procrustes_ms_model * 1.0/ $procrustes_ms_residual}]
	    set procrustes_f_indiv [expr {$procrustes_ms_indiv * 1.0/ $procrustes_ms_interact}]
	    set procrustes_f_side [expr {$procrustes_ms_side * 1.0/ $procrustes_ms_interact}];# Palmer & Strobec, 1986 p. 406
	    set procrustes_f_interact [expr {$procrustes_ms_interact * 1.0/ $procrustes_ms_residual}]
	    
	    # Calcul des significations statistiques
	    set a $procrustes_ddf_model
	    set b $procrustes_ddf_residual
	    set cc $procrustes_f_model
	    set P_model  [::math::statistics::Cdf-toms322 $a $b $cc]
	    #
	    set a $procrustes_ddf_indiv
	    set b $procrustes_ddf_residual
	    set cc $procrustes_f_indiv
	    set P_indiv  [::math::statistics::Cdf-toms322 $a $b $cc]
	    #		
	    set a $procrustes_ddf_side
	    set b $procrustes_ddf_interact
	    set cc $procrustes_f_side
	    set P_side  [::math::statistics::Cdf-toms322 $a $b $cc];#Palmer&Strobecq,1986 p.406
	    #
	    set a $procrustes_ddf_interact
	    set b $procrustes_ddf_residual
	    set cc $procrustes_f_interact
	    set P_interact  [::math::statistics::Cdf-toms322 $a $b $cc]
	    
	    # output Table for SHORT REPORT
	    set S(shapereport) "\n"
	    append S(shapereport) "\n\n----\nP R O C R U S T E S - Simetria de la conformacion - Shape symmetry \n(Klingenberg & McIntyre, 1998, Evolution 52, 5)"
	    rouge $S(shapereport)
	    $p configure -tabs "3c left 6c center 8c center 10c center 12c left"
	    $p insert end "\n"
	    rouge "\n------------------------------------------------------"
	    $p insert end "\n\t\t \t Procrustes ANOVA output "
	    rouge "\n------------------------------------------------------"
	    $p insert end "\nSource \t  SS \t df \t MS \t F \tSignificance"
	    rouge "\n------------------------------------------------------"
	    $p insert end "\nModel \t [format %8.4f $procrustes_ss_model] \
                    \t $procrustes_ddf_model \t [format %8.6f $procrustes_ms_model] \t [format %8.2f $procrustes_f_model] \t [format %8.4f [expr {1-$P_model}]]"
	    $p insert end "\nindiv \t [format %8.4f $procrustes_ss_indiv] \
                    \t $procrustes_ddf_indiv \t [format %8.6f $procrustes_ms_indiv] \t [format %8.2f $procrustes_f_indiv] \t [format %8.4f [expr {1-$P_indiv}]]"
	    $p insert end "\nside \t [format %8.4f $procrustes_ss_side] \
                    \t $procrustes_ddf_side \t [format %8.6f $procrustes_ms_side] \t [format %8.2f $procrustes_f_side] \t [format %8.4f [expr {1-$P_side}]]"
	    $p insert end "\nside*i \t [format %8.4f $procrustes_ss_interact] \
                    \t $procrustes_ddf_interact \t [format %8.6f $procrustes_ms_interact] \t [format %8.2f $procrustes_f_interact] \t [format %8.4f [expr {1-$P_interact}]]"
	    $p insert end "\nResidu\t [format %8.4f $procrustes_ss_residual] \
                    \t $procrustes_ddf_residual \t [format %8.6f $procrustes_ms_residual] \t"
	    rouge "\n------------------------------------------------------"
	    
	    
	    # For SHORT REPORT (output Table)
	    append S(shapereport) "\n--------------------------------------"
	    append S(shapereport) "\n\t\t \t Procrustes ANOVA output "
	    append S(shapereport) "\n--------------------------------------"
	    append S(shapereport) "\nSource \t  SS \t df \t MS \t  F \t Signific."
	    append S(shapereport) "\n--------------------------------------"
	    append S(shapereport) "\nModel \t [format %8.4f $procrustes_ss_model] \
                    \t $procrustes_ddf_model \t [format %8.6f $procrustes_ms_model] \t[format %8.2f $procrustes_f_model] \
                    \t[format %8.4f [expr {1-$P_model}]]"
	    append S(shapereport) "\nindiv \t [format %8.4f $procrustes_ss_indiv] \
                    \t $procrustes_ddf_indiv \t [format %8.6f $procrustes_ms_indiv] \t[format %8.2f $procrustes_f_indiv] \
                    \t[format %8.4f [expr {1-$P_indiv}]]"
	    append S(shapereport) "\nside \t [format %8.4f $procrustes_ss_side] \
                    \t $procrustes_ddf_side \t [format %8.6f $procrustes_ms_side] \t[format %8.2f $procrustes_f_side] \
                    \t[format %8.4f [expr {1-$P_side}]]"
	    append S(shapereport) "\nside*i \t [format %8.4f $procrustes_ss_interact] \
                    \t $procrustes_ddf_interact \t [format %8.6f $procrustes_ms_interact] \t[format %8.2f $procrustes_f_interact] \
                    \t[format %8.4f [expr {1-$P_interact}]]"
	    append S(shapereport) "\nResidu\t [format %8.4f $procrustes_ss_residual] \t $procrustes_ddf_residual \
                    \t [format %8.6f $procrustes_ms_residual] \t"
	    
	    
	    tk_messageBox -message "Done.\nFirst ANOVA output is for Centroid size, last one is the Procrustes ANOVA for SHAPE\n(see end of report)\n\nUse the \"SHORT REPORT\" button to see the essential" -icon warning -type ok
	}
	###################################################################
	$p insert end "\n"
	destroy $p.embed_plot
	#$p window create {end lineend} -window $p.embed_plot
	button $p.embed_plot -text "PCA on left-rigth PW values for sucessive sessions" -command {
	    global asi_individus
	    dessine $asi_individus
	}
	$p window create {end lineend} -window $p.embed_plot
	###################################################################
    }
    
    ## VOILA LES BOUTONS
    #pack .buttons.dyk .buttons.r .buttons.fascores .buttons.informe .buttons.exitanyway -side left -fill y -expand 1
    pack .buttons.r .buttons.fascores .buttons.informe .buttons.exitanyway -side left -fill y -expand 1
    destroy .buttons.pwcs .buttons.cs .buttons.exit
    
}
#
# Proc ANOVA --- fin



# Procedure ASI_ACP -- debut
#
# c'est en fait un 'silent_'acp (sauf factor map), un peu comme COPRI?
#
proc asi_acp {x} {
    
    global p pop lignes colonnes
    
    set pop $x
    
    set colonnes [mcols $pop]
    set lignes [mrows $pop]
    set cellules [expr {$lignes * $colonnes}]
    
    #CREER n-1 POUR CALCULER (plus tard) LES VARIANCES (eigenvalues)
    set obsminusone [expr {$lignes - 1}]
    
    # ------------------------------------------
    # ACP de la MATRICE OBSERVEE (procedure ACP)
    # ------------------------------------------
    
    # CALCULER MOYENNES ET DEVIATIONS STANDARD DES VARIABLES, et ...
    mnorms_br pop popmeans popsigmas
    
    # ... CREER UN VECTEUR AVEC AUTANT DE 1 QUE DE VARIABLES ...
    # IL SERVIRA POUR CREER UNE MATRICE CENTREE au lieu de NORMALISEE
    set popones "2 1 $colonnes 1"
    set x 1
    set y $colonnes
    while {$x < $y} {lappend popones 1; set x [expr {$x + 1} ] }
    
    # CENTRER LES DONNEES PLUTOT QUE DE LES NORMALISER
    # ------------------------------------------------
    set Z [mnormalize $pop $popmeans $popones]
    
    # ---------------------------------------------------
    # essai de la methode naive, qui je suppose prend + de temps ....
    # ---------------------------------------------------
    # SINGULAR VALUE DECOMPOSITION SUR LA MATRICE varcov
    # ---------------------------------------------------
    
    #######################################################################
    #     bleu "\nPromedios\n"
    #     $p insert end "[show $popmeans %8.6f]"
    #     bleu "\n\nDesviaciones estandard\n"
    #     $p insert end "[show $popsigmas %8.6f]\n"
    #     $p insert end "\nLes donnees sont ensuite 'centrees' (chaque colonne par sa moyenne)"
    #     $p insert end "\n[show $Z]"
    #######################################################################
    set ZZ [mmult [transpose $Z] $Z]
    #######################################################################
    #     $p insert end "\n\nThe varcov matrix, or matrix of mean square VCM = (Zt)(Z)/(n-1)\
	#             \n\tIl s'agit d'une multiplication de Z (la matrice des donnees centrees) par sa transposee (Zt),\
	#             \n\tAttention, dans cette multiplication, la transposée (Zt) est à gauche: \t Zt by Z"
    #     $p insert end "\n[show $ZZ]"
    #     $p insert end  "\n\nChaque element de la matrice resultante (ici : [mrows $ZZ] x [mcols $ZZ]) est divisé ensuite \
	#             par le nombre d'individus moins un (perte de un degré de liberté)\nOn obtient alors la matrice de \
	#             variances - covariances\n"
    #######################################################################
    set nless1 [expr {1./$obsminusone}]
    #######################################################################
    #     vert "\nVariance - covariance matrix"
    #     $p insert end "\nVariances are the diagonal elements of the variance-covariance matrix"
    #     vert "\n[show [set vcm [mscale $ZZ $nless1]] %8.6f]"
    #######################################################################
    set vcm [mscale $ZZ $nless1]
    # Continuing the naive example use VARCOV for the eigensolution
    set V $vcm
    mevsvd_br V evals
    #set obsevalues $evals
    # debug
    #######################################################################
    #     $p insert end "\n\nEIGENVALUES  (evals <- ' mevsvd_br V evals ' )\n[lrange $evals 3 end]"
    #     vert "\nThe sum of eigenvalues is the sum of the diagonal elements of the variance-covariance matrix"
    #######################################################################
    set long [llength $evals]
    set sumevals 0
    for {set i 3} {$i < $long} {incr i} {set sumevals [expr {[lindex $evals $i] + $sumevals} ] }
    #######################################################################
    #     vert "\nSum of EIGENVALUES "
    #     $p insert end "[format %1.6f $sumevals]\n"
    #     $p insert end "\nNumber of EIGENVALUES"
    #     $p insert end " [expr {$long - 3}]"
    #######################################################################
    
    #debug
    set I [mident 3]
    #######################################################################
    #     rouge "\n\nIdentity matrix \n[show $I]"
    #######################################################################
    
    set propevals(name) 0
    for {set i 3} {$i < $long} {incr i} {
	set prop [expr {[lindex $evals $i] / $sumevals} ]
	set name [expr {$i - 3}]
	set propevals($name) $prop
    }
    $p insert end "\n"
    #######################################################################
    #     global nbcp_nonull
    #     for {set i 3} {$i < $long} {incr i} {
    #         set name [expr {$i - 3}]
    #         bleu "\ncontribucion pc[expr {$name + 1} ]"
    #         $p insert end " [format %1.2f  $propevals($name)] \t\t($propevals($name))"
    #         if  {$propevals($name) < 0.0000001} {set nbcp_nonull $name} else {set nbcp_nonull [expr {$long-3}]}
    #     }
    #######################################################################
    
    # Eigenvectors - presentation
    #
    #######################################################################
    #     bleu "\n\nCoeficientes para los 3 primeros componentes principales\
	#             \no ' E I G E N V E C T O R E S' (V de <- mevsvd_br V evals)\
	#             \n(au depart V c'est la matrice de variances-covariances) \n"
    #     bleu "CP1\t\tCP2\t\tCP3\n"
    #     bleu "---\t\t---\t\t---\n"
    #######################################################################
    #######################################################################
    #     set Vpres [lrange $V 3 end]
    #     set compteur -1
    #     foreach item $Vpres {
    #         incr compteur
    #         if {$compteur < 3} {
    #             set item [format %1.3f $item]
    #             $p insert end "$item\t\t"
    #         }
    #         if {$compteur == [expr {$colonnes-1} ] } {
    #             set compteur -1
    #             $p insert end "\n"
    #         }
    #     }
    #######################################################################
    
    #######################################################################
    #     bleu "\n\nCoeficientes para "
    #     rouge "todos "
    #     bleu "los componentes principales o ' E I G E N V E C T O R E S ' ([mrows $V] x [mcols $V])\n"
    #     $p insert end "[show $V %1.3f]"
    #######################################################################
    
    # COMPOSANTES PRINCIPALES (cp) (procedure ACP)
    # ---------------------------------------------
    global cp
    set cp [mmult $pop $V ]
    
    #######################################################################
    #     bleu "\n\nPrincipal components\n[show $cp]"
    #######################################################################
    
    # CENTRER LES COMPOSANTES PRINCIPALES ... les individus sont pourtant...
    mnorms_br cp cpmeans cpsigma
    set copri [mnormalize $cp $cpmeans $popones]
    #
    #$p insert end "\n lignes $lignes <-> copri [mrows $copri]\n"
    
    #########################################################################
    #     bleu "\nComponentes principales (centrados)\n"
    ##########################################################################
    global cpcentrados
    set cpcentrados [show $copri]
    ##########################################################################
    #     set prescopri [lrange $copri 3 end]
    #     set compteur -1
    #     foreach item $prescopri {
    #         incr compteur
    #         set item [format %1.7f $item]
    #         $p insert end "$item\t"
    #         if {$compteur == [expr {$colonnes-1} ] } {set compteur "-1";$p insert end "\n"}
    #     }
    #######################################################################
    
    global individus
    set individus ""
    for {set u 0} {$u < $lignes} {incr u} {
	lappend individus [mrange $copri 0 end $u $u]
    }
    
    #######################################################################
    #     # Correlations entre CP1 et chaque variable ...
    #     # ----------------------------------------------
    #     set corps [mrange $cp 0 0 0 end]
    #     set corps [join_cols $corps $pop]
    #     set R [correlation "$corps"]
    #     set Rc [mrange $R 0 0 1 end]
    #     bleu "\n\nCorrelaciones entre CP1 y las variables:"
    #     $p insert end "\n[show $Rc %1.6f]"
    #######################################################################
    
    #######################################################################
    
    #######################################################################
    #     $p insert end "\n"
    #     destroy $p.embed_plot
    #     $p window create {end lineend} -window $p.embed_plot
    #     button $p.embed_plot -text "PCA on left-rigth PW values for sucessive sessions" -command {
    #         dessine $individus
    #     }
    #
    #     $p window create {end lineend} -window $p.embed_plot
    #     bind $p.embed_plot <Motion> {$p.embed_plot  configure -cursor hand2}
    
    #ajouté Anillaco February 2008
    return $cp;#cp non centrées (cfr. 'copri'), non standardisées
    #return $copri;#cp centrée
}
#
# Procedure ASI_ACP - end

# Procedure FASCORES_ACP -- debut
#
# c'est en fait un 'silent_'acp (sauf factor map), un peu comme COPRI?
#
proc fascores_acp {x} {
    
    global p pop lignes colonnes
    
    set pop $x
    
    set colonnes [mcols $pop]
    set lignes [mrows $pop]
    set cellules [expr {$lignes * $colonnes}]
    
    #CREER n-1 POUR CALCULER (plus tard) LES VARIANCES (eigenvalues)
    set obsminusone [expr {$lignes - 1}]
    
    # ------------------------------------------
    # ACP de la MATRICE OBSERVEE (procedure ACP)
    # ------------------------------------------
    
    # CALCULER MOYENNES ET DEVIATIONS STANDARD DES VARIABLES, et ...
    mnorms_br pop popmeans popsigmas
    
    # ... CREER UN VECTEUR AVEC AUTANT DE 1 QUE DE VARIABLES ...
    # IL SERVIRA POUR CREER UNE MATRICE CENTREE au lieu de NORMALISEE
    set popones "2 1 $colonnes 1"
    set x 1
    set y $colonnes
    while {$x < $y} {lappend popones 1; set x [expr {$x + 1} ] }
    
    # CENTRER LES DONNEES PLUTOT QUE DE LES NORMALISER
    # ------------------------------------------------
    set Z [mnormalize $pop $popmeans $popones]
    
    # ---------------------------------------------------
    # essai de la methode naive, qui je suppose prend + de temps ....
    # ---------------------------------------------------
    # SINGULAR VALUE DECOMPOSITION SUR LA MATRICE varcov
    # ---------------------------------------------------
    set Z $pop
    set ZZ [mmult [transpose $Z] $Z]
    set nless1 [expr {1./$obsminusone}]
    set vcm [mscale $ZZ $nless1]
    set V $vcm
    mevsvd_br V evals
    set long [llength $evals]
    set sumevals 0
    for {set i 3} {$i < $long} {incr i} {set sumevals [expr {[lindex $evals $i] + $sumevals} ] }
    set I [mident 3]
    set propevals(name) 0
    for {set i 3} {$i < $long} {incr i} {
	set prop [expr {[lindex $evals $i] / $sumevals} ]
	set name [expr {$i - 3}]
	set propevals($name) $prop
    }
    $p insert end "\n"
    
    # COMPOSANTES PRINCIPALES (cp) (procedure ACP)
    # ---------------------------------------------
    global cp
    set cp [mmult $pop $V ]

    # CENTRER LES COMPOSANTES PRINCIPALES ... les individus sont pourtant...
    mnorms_br cp cpmeans cpsigma
    set copri [mnormalize $cp $cpmeans $popones]
    #
    global cpcentrados
    set cpcentrados [show $copri]    
    global individus
    set individus ""
    for {set u 0} {$u < $lignes} {incr u} {
	lappend individus [mrange $copri 0 end $u $u]
    }
    
    #ajouté Anillaco February 2008
    #return $cp;#cp non centrées (cfr. 'copri'), non standardisées
    return $copri;#cp centrée
}
#
# Procedure FASCORES_ACP - end

# Procedure MP3 - start
#
# Moore Penrose pseudo inverse
#
proc mp3 {X} {
    #
    # Maximum 7 DECIMALES, pas plus!!
    set Z $X
    set Z "[lrange $Z 0 2] [show $Z %8.7f]";# pas > 7 decimales
    set Ztcllib [::math::linearalgebra::from_LA $Z]
    set svd_tcllib [::math::linearalgebra::determineSVD $Ztcllib]
    set U [::math::linearalgebra::to_LA [lindex $svd_tcllib 0]]
    set S [::math::linearalgebra::to_LA [lindex $svd_tcllib 1]]
    set V [::math::linearalgebra::to_LA [lindex $svd_tcllib 2]]
    # 
    set el 0
    set Splus [vtrim $S]
    foreach item $Splus {
        #
	if {$item > 0.00000000149} {  
	    set Splus [lreplace $Splus $el $el [expr {1.0/$item}]]
	} else {
	    set Splus [lreplace $Splus $el $el 0.000]
	}
	incr el
    }
    set Splus "2 [mrows $S] [mcols $S] $Splus"
    set Splus_diag [mdiag $Splus]
    set regardez [mmult $Splus_diag [transpose $U]]
    set pinvZ [mmult $V $regardez]
    #
    set verification [mmult $pinvZ $Z]
    #debug "verification! \n[show $verification %8.4f]"
    #
    return $pinvZ
}
#
# Procedure MP3 (moorepenrose) - fin

proc centering {x} {
    #
    # x est une matrice de HUME
    #
    #$m1 add command -label "X-> centered X" -command {}
    global S dossier
    # set typeList {
    # 	{{"Text files"} {.txt .nts .TXT .NTS} }
    # }
    # set infile [chemin]
    # if {$infile == ""} {
    # 	return
    # } else  {
    # 	set x [loadfile $infile]
    # }
    # #set x [loadfile $infile]
    # entrada $x $infile
    set colonnes [mcols $x]
    set pop $x
    # ... CREER UN VECTEUR AVEC AUTANT DE 1 QUE DE VARIABLES ...
    # IL SERVIRA POUR CREER UNE MATRICE CENTREE au lieu de NORMALISEE
    set popones "2 1 $colonnes 1"
    set x 1
    set y $colonnes
    while {$x < $y} {lappend popones 1; set x [expr {$x + 1}]}
    #	
    # Compute the means and standard deviations of each column.
    #
    mnorms_br pop popmeans popsigma
    #	
    # CENTRER LES DONNEES PLUTOT QUE LES NORMALISER
    # ------------------------------------------------
    set C [mnormalize $pop $popmeans $popones]
    #$p insert end "\nCenter each column by substracting the corresponding mean.\n"
    #rouge [show $C %8.7f]
    #
    # global infile
    # set sub "_"
    # set end ".txt"
    # set quoi "CENTERED"
    # set ajout $sub$quoi$end
    # set infilesub "[file root $infile]$ajout"
    # #
    # rouge "\n\nCentered matrix saved as:\n$infilesub\n"
    # set fid [open $infilesub w]
    # puts -nonewline $fid "Centered matrix of $infile\n"
    # puts -nonewline $fid [show $C %8.7f]
    # close $fid
    #$p insert end "\nSaved as:\n[file dirname $infilesub]\n[file tail $infilesub]"
    return $C
}

# Procedure PRINCIPAL --- debut
#
proc principal {} {
    #
    global p S version infile
    #
    set tcl_precision 17
    #
    # VENTANA de COMBATE - start
    #
    wm geometry . 880x650+128+10
    wm title . "$version - dujardinbe@gmail.com"
    
    set aide_input "\t\t\tP o s s i b l e     I N P U T     F I L E S\n\n--- ---\n'Separate (size) files'\n--- ---\n\nTwo separate files, containing size only (..._CS.txt, or ..._ARE.txt, or ...PER.txt). \nA two-way mixed ANOVA is performed (Palmer & Strobecq, 1986).\n\nor\n\n--- ---\n'Single (shape) files'\n--- ---\n\nSingle file containing shape variables (..._PW.txt, ..._FOGnefcp.txt)\nfor successive digitizations. \nA Procrustes ANOVA is performed (Klingenberg McIntyre, 1998).\n\nor\n\n--- ---\n'Single (landmarks) file'\n--- ---\n\nA single file containing raw coordinates of landmarks for successive digitizations. \nThus, a Procrustes superimposition is performed before to apply \na two way mixed ANOVA for size (Palmer & Strobecq, 1986), \nand \na Procrustes ANOVA for shape (Klingenberg McIntyre, 1998). \nMoreover, the symmetric and asymmetric components, as well as \nthe individual shape FA scores (Klingenberg & Monteiro, 2005), \nare computed and saved in separate files.\n\n--- ---\n\n"
    

    set  aide_en_shape  "\n\nAbout the BUTTON: 'single (shape) file'...\n--- ---\nThe file contains shape variables of left and right sides, computed from two sucessive digitizations.\nThe internal arrangement as above.\n"

    set  aide_fr_pw_cs  "About the BUTTON: 'single (landmarks) file'...\n--- ---\n\n\
    - ligne 1 = digitization 1 indiv 1 side 1 (d1 i1 s1)\n\
    - ligne 2 = digitization 1 indiv 1 side 2 (d1 i1 s2)\n\
    - ligne 3 = d1 i2 s1\n\
    - ligne 4 = d1 i2 s2\n\
    - ligne 5 = d1 i3 s1\n\
    - ligne 6 = d1 i3 s2\n\
    - ...\n\
    - ligne (r*2)-1 = d1 i(r) s1\n\
    - ligne (r*2) = d1 i(r) s2\n\
    - ligne 1 = d2 i1 s1 <- starting the second digitization (d2)\n\
    - ligne 2 = d2 i1 s2\n\
    - ligne 3 = d2 i2 s1\n\
    - ligne 4 = d2 i2 s2\n\
    - ligne 5 = d2 i3 s1\n\
    - ligne 6 = d2 i3 s2\n\
    - ...\n\
    - ligne (r*2)-1 = d2 i(r) s1\n\
    - ligne (r*2)   = d2 i(r) s2\n\
    \n ------------------------------------------------------- \
    \nIMPORTANT: THE ABOVE FILE IS BUILT USING the 'TET' module\
    \n(menu : Utilities -> From 4 files to ASI), \
    \nWHERE 4 FILES ARE ASKED TO BE ENTERED: \
    \ns1d1, s1d2, s2d1, s2d2\
    \n ------------------------------------------------------- \
    \n\n"
    
    set aide_fr_1mes1file "\nAbout the BUTTON: \"Separate (size) files\"...\
    \n---   ---\nSize files as produced by CLIC are ending by  '...CS.txt',  '..._PER.txt' or  '..._ARE.txt'\
    \nOne file by measurement can be entered, it is about size only. \
    \nA maximum of TWO measurements (i.e., just one repetition)\
    \nThe internal arrangement of each file as above...\
    \nEach file contains one column or various ones, i.e., you can enter the measurements \nof various characters, as a table.\n---   ---"
    #"\n-----------------------------------------------------------------\n"
    
    set aide_fr_att "\nATTENTION: le nombre d'individus d'un fichier n'est pas le nombre de lignes ('r*2')\nmais bien le nombre d'individus ('r') dont les deux cotes sont compares  ('cote 1' et 'cote 2')\n"
    
    set aide_es_att "\nOJO: el numero de individuos de un archivo NO es el numero de lineas ('r*2')\n\
    pero si el numero de individuos ('r') cuyos lados se comparan ('lado 1' y 'lado 2')\n"
    
    set aide_en_att "ATTENTION: the number of individuals is NOT the number of rows ('r*2')\n\
    but it is the number of individuals ('r') whose sides are compared ('side 1' and 'side 2')\n"
    
    set aide_es "Votre jeu de fichier doit etre organise comme suit\
    - Un fichier par mesure\
    - Le meme ordre des individus doit etre strictement suivi dans chaque fichier\
    - Les valeurs de chaque individu sont ordonnees selon les cotes, de la meme facon pour chaque individu\
    (Par exemple, gauche-droite, gauche-droite, etc.)\
    Attention: le nombre d'individus d'un fichier n'est pas le nombre de lignes\
    mais bien le nombre d'individus dont les deux cotes sont compares \
    (Un fichier qui compare la droite et la gauche de 30 individus contiendra 60 lignes, puisque chaque individu a deux valeurs, la droite et la gauche)"
    
    set aide_en "Votre jeu de fichiers doit etre organise comme suit\
    - Un fichier par session de mesure\
    Le meme ordre des individus doit etre strictement suivi dans chaque fichier\
    - Les valeurs de chaque individu sont ordonnees selon les cotes, de la meme facon pour chaque individu\
    (Par exemple, gauche-droite, gauche-droite, etc.)\
    Attention: le nombre d'individus d'un fichier n'est pas le nombre de lignes\
    mais bien le nombre d'individus dont les deux cotes sont compares \
    (Un fichier qui compare la droite et la gauche de 30 individus contiendra 60 lignes, puisque chaque individu a deux valeurs, la droite et la gauche)"
    
    destroy .buttons;# au cas ou on rappelle la procedure principale (cfr. UNOT)
    
    frame .buttons;## EN BAS DE CETTE FENETRE, UNE FRAME POUR LOGER des BOUTONS
    pack .buttons -side bottom -fill x -pady 2m
    
    button .buttons.salir -bg red -fg white -text "EXIT\n- - -" -command {exit}
    
    button .buttons.r -text "R" -command {
	global S infile
	# Ouvrir un terminal R a chaque fois? Oui si je peux le refermer...
	if {[info exists S(erela)]} {
	    # travailler
	} else  {
	    erela;# <- R est la
	    # travailler
	}
	#
	#...fenetre d'aide au R dans la variable globale S(ere)
	#
	faar
	
	# ------------------- TENTATIVE CLIC+ - start
	# Chaque intruction ci-dessous est stockee dans une variable SCRIPT, ensuite ecrite dans un fichier
	set script ""
	set ifile [file tail [file rootname $infile]].txt
	#
	# ------------------- TENTATIVE CLIC+ - end
	
	if {$S(rasi) == 1} {
	    set analyse "r_asi_pwcs.txt"
	} else {
	    set analyse "r_asi_cscs.txt"
	}
	
	$S(ere) insert end "Please put yourself in the working folder\n"


	set aide "\nsetwd(\"[file dirname $infile]\");"
	append script "\n[string range $aide 1 end]";#<- pour clic+
	r_rouge $aide
	$S(ere) insert end "\nTo verify the folder content, type"
	set aide "\n> list.files()"
	r_brun $aide
	$S(ere) insert end "\nEnter the data:"
	if {$S(rasi) == 1} {
	    set aide "\ndataset <- read.table(\"[file tail $infile]\", skip=1,header=FALSE, sep=\"\", na.strings=\"NA\", dec=\".\", strip.white=TRUE)"
	    append script "\n[string range $aide 1 end]";#<- pour clic+
	    r_rouge $aide
	} else {
	    global filerep1 filerep2
	    set aide "\n rep1 <- read.table(\"$filerep1\", skip=1,header=FALSE, sep=\"\", na.strings=\"NA\", dec=\".\", strip.white=TRUE)"
	    append script "\n[string range $aide 2 end]";#<- pour clic+
	    r_rouge $aide
	    set aide "\n rep2 <- read.table(\"$filerep2\", skip=1,header=FALSE, sep=\"\", na.strings=\"NA\", dec=\".\", strip.white=TRUE)"
	    append script "\n[string range $aide 2 end]";#<- pour clic+
	    r_rouge $aide
	}
	$S(ere) insert end "\nTo install a new package, under INTERNET connection,just call it using the 'install.packages()' function"
	set aide "\n> install.packages(\"packagename\")"
	r_brun $aide
	$S(ere) insert end "\nAlternatively, \
            \n(1) download the source file (ending by ...tar.gz) in your working directory\
            \n(2) use the following instruction:"
	set aide "\n> install.packages(\"...tar.gz\")"
	r_brun $aide
	$S(ere) insert end "\nSome R packages are necessary to perform morphometric analyses. Their intallation is made once for all. However, you also have to upload it into your current R session, like"
	set aide "\nlibrary(e1071);library(pixmap);library(utils);library(base);library(stats);"
	r_rouge $aide
	append script "\n[string range $aide 1 end]";#<- pour clic+
	$S(ere) insert end "\nThe R script used here for symmetry analyses is inspired from Claude J. (2008). Morphometrics with R. Springer"
	set aide "\n> source(\"http://www.isem.univ-montp2.fr/recherche/files/2012/01/Rfunctions1.txt\");"
	r_brun $aide
	# $S(ere) insert end "\n...or, if not presently connected"
	# set aide "\n> source(\"[file join $S(dirclic) $S(rhelp) Rfunctions.txt]\");"
	# r_brun $aide
	# append script "\n[string range $aide 3 end]";#<- pour clic+
	# $S(ere) insert end "\nPlease call also the following function(s):"
	# set aide "\n> source(\"[file join $S(dirclic) $S(rhelp) clic2r.txt]\");"
	# r_rouge $aide
	# append script [string range $aide 3 end];#<- pour clic+
	
	r_vert "\n\n--- --- --- --- --- --- \nYOUR ATTENTION PLEASE, "
	$S(ere) insert end "DIFFERENT OPTIONS ARE AVAILABLE!"
	r_vert "\n--- --- --- --- --- --- \n\n"
	$S(ere) insert end "\n1./" 
	r_vert " An R SCRIPT inspired from Claude's book, see hereunder the blue script\n"
	$S(ere) insert end "\n2./" 
	r_vert "The use of the GEOMORPH package (bilat.symm function). This option makes use of ANOTHER INPUT FILE (actually the raw coordinates in the 'coord_\[date\]_format.txt)\n\n"
	
	r_vert "\n\n--- --- --- --- --- --- \n"
	$S(ere) insert end "FIRST OPTION"
	r_vert "\n--- --- --- --- --- --- \n"
	r_vert "\n\nThe instructions above in one (blue) set:\n"
	r_bleu  "$script"
	if {$S(rasi) == 1} {
	    $S(ere) insert end "\n---\nThe following instructions as R script for the [file tail $infile] data symmetry analysis\n"
	} else {
	    $S(ere) insert end "\n\n\n---\nThe following instructions as R script for the two files($filerep1 and $filerep2), corresponding to the two measurements sessions on both sides of the same specimens\n"
	}
	fiar $analyse


	r_vert "\n\n--- --- --- --- --- --- \n"
	$S(ere) insert end "SECOND OPTION"
	r_vert "\n--- --- --- --- --- --- \n"
	$S(ere) insert end "\nThis option makes use of the GEOMORPH package, and of another INPUT file\n"
	set option_deux "\
            \nlibrary(geomorph)
            \nsetwd(\"[file dirname $infile]\");# if not done yet;
            \n# Please enter here the LANDMARKS file ('coord_\[date\]_format.txt'), i.e. the raw coordinates (NOT ending by '_PW_CS') containing the two sides and two measurements arranged as for the ASI module;
            \ndataset <- read.table(\"[file dirname $infile]/ ... ...\", skip=1,header=FALSE, sep=\"\", na.strings=\"NA\", dec=\".\", strip.white=TRUE);
            \nsource(\"[file join $S(dirclic) $S(rhelp) clic2r.txt]\");
            \ndata_Sformat <- clic2array(dataset);
            \ndataset <-as.matrix(dataset;
            \nnbrows <-nrow(dataset);
            \nnbindiv<-nbrows/4;
            \nlado<-1:2;
            \nlado<-rep(lado,nbrows/2);
            \nlado;
            \nindiv <- c(1,1);
            \nfor(i in 2:nbindiv-1) {indiv<-c(indiv,i+1,i+1)};
            \nindiv<-c(indiv,indiv);
            \nmes1<-rep(1,nbrows/2);
            \nmes2<-rep(2,nbrows/2);
            \nrepet <-c(mes1,mes2);
            \nbilat.symmetry(data_Sformat,ind=indiv,side=lado,replicate=repet,object.sym=FALSE);
        "
	r_rouge $option_deux
	##########################################################################
	#     .rwidget.b configure -command {
	#
	#         # ------------------- TENTATIVE CLIC+ - start
	#         #
	#         # Appel a la procedure CLIC+ RIN ROUT (laquelle appelle WRITEINFO)
	#         debug "\nThe CLIC+ button asks the system to quietly run the whole set of R instructions (rin_mog.txt). \
	    #                 \nGraphs (grout_asi.png) and text (rout_asi.txt) output files are in your working folder:\
	    #                 \n[file dirname $infile])\n"
	#         set rin [file join [file dirname $infile] rin_asi.txt]
	#         set rout [file join [file dirname $infile] rout_asi.txt]
	#         clic+ $script $rin $rout
	#         #
	#         # ------------------- TENTATIVE CLIC+ - end
	#     }
	##################################################################
    }
    
    
    button .buttons.dyk -text "Did you know" -command {
	set rd0 "If there are directional (DA) and non-directional asymmetries (NDA) both significant,\
    NDA cannot be used to estimate fluctuating asymmetry (FA) or antisymmetry (AS),\
    \n(their definition requires the mean of signed differences to be zero).\
    \nYou have to scale for directional asymmetry first."
	set rd1 "C. P. Klingenberg, M. Barluenga, and A. Meyer.\
    Shape analysis of symmetric structures: Quantifying variation among individuals and asymmetry.\
    Evolution 56 (10): 1909-1920, 2002."
	set rd2 "Asymmetry is intra-individual variance"
	set rd3 "Symmetry has an expected outcome: perfection"
	set rd4 "Symmetry may be bilateral, or radial"
	set rd5 "C. D. Schlichting and M. Pigliucci.\
    Phenotypic evolution: a reaction norm perpective.\
    Sinauer Associates, Inc. Sunderland, MA, 387 pp, 1998."
	set rd6 "Palmer, R. A. and Strobeck, C. 1986. Fluctuating asymmetry: Measurement, Analysis, Patterns. \
    Ann. Rev. Ecol. Syst. 17: 391--421."
	set rd7 "Index FA10 = (MSside*indiv - MSerror)/(number of repeated measurements)\
    \nTo compare asymmetries among populations, use their MS values and corresponding df."
	set rd8 "Fluctuating asymmetry, as well as antisymmetry, \
    are supposed to be modulated mainly by environmental influences."
	set rd7 "The mean sum of square (MS) of the ANOVA output \
    \nis obtained by dividing the corresponding \
    \ntotal SS by the corresponding degrees of freedom."
	set rd9 "The F value of the MS side*indiv is obtained by dividing the corresponding\
    \nmean sum of square (MS) by the residual MS (error)."
	set rd10 "ANOVA output : 'side*indiv' \nrefers to the interaction of each side with individuals, \
    \n\nIt represents the NON-DIRECTIONAL ASYMMETRY (NDA)"
	set rd11 "ANOVA output : 'side' \nrefers to the variation between sides, \
    \nit represents the DIRECTIONAL ASYMMETRY (DA)"
	set rd12 "ANOVA output : 'Residual' ... \nis your measurement error"
	set rd13 "ANOVA output: 'indiv' refers to the interindividual variation; \
    \nBe careful: if too different \
    \n(because of outliers, for instance), \
    \nit will be hard for you \
    \nto detect other sources of variation."
	set rd14 "ANOVA output : 'side' \nrefers to the variation between sides (DA), \
    \nIts statistical significance is computed by \nF = (MS sides / MS side*indiv)"
	set rd15 "ANOVA for asymmetry testing is a mixed model ANOVA ('side' = fixed, 'indiv' = random)"
	# Appel a la librairie Tcllib (::math::statistics)
	set nb [expr {int([::math::statistics::random-uniform 0 15 1])}]
	# usage de upvar pour que la valeur d'une variable ('ici') devienne le nom d'une variable (rd1, rd9)
	#     lequel est place dans une autre variable: x
	set ici "rd$nb"
	upvar #0 $ici x
	tk_messageBox -message "$nb: $x" -icon warning -type ok
	update
	flush stdout
    }
    
    focus .buttons.dyk
    
    # bind ...
    bind .buttons.dyk <Return> {
	.buttons.dyk invoke
	update
        flush stdout
    }
    
    button .buttons.sr -text "Short report" -command {
	global p S
	destroy .sr
	#
	#destroy .sr.scrollSR ;# au cas ou on rappelle la procedure principale (cfr. UNOT)
	#
	toplevel .sr
	wm geometry .sr 820x630+205+66
	set short [text .sr.t -wrap word -height 22 -font "courier 9" \
		       -bg "light yellow" \
		       -fg blue\
		       -tabs "3c left"\
		       -yscrollcommand ".sr.v_scroll set" \
		       -xscrollcommand ".sr.h_scroll set"]
	
	scrollbar .sr.v_scroll -command ".sr.t yview"
	scrollbar .sr.h_scroll -command ".sr.t xview" -orient horizontal
	pack .sr.v_scroll -side right -fill y
	pack .sr.h_scroll -side bottom -fill x
        pack .sr.t -side bottom -fill both -expand 1
#debug "$S(sizereport)"
        if {[info exists S(sizereport)]} {
            #
            $short insert end "SHORT REPORT \n$S(sizereport)";#S(sizereport) contient Correlations et ANOVA
            #
            $short insert end "\n\nGraphical output\n"
            # destroy .sr.embed_assiplot - bouton cache car ne fonctionne que pour la derniere variable
            # button .sr.embed_assiplot -text "Plot of asymmetry against size" -font "courier 9" -command {
            #     $p.embed_assiplot invoke
            # }
            #
            #$short window create {end lineend} -window .sr.embed_assiplot
            destroy .sr.embed_eucl
            button .sr.embed_eucl -text "Histogram of signed differences" -font "courier 9" -command {
                global S valores          
                #$short insert end "\nRappeler l'existence et le contenu des differentes variables, ou alors au moins le contenu des differences avec signe"
                for {set i 0} {$i < [expr {$S(nbdecol)+1}]} {incr i} {
		    set pop $valores($i)
		    #debug "---"
		    set pop_g [mrange $pop 0 0 0 end]
		    set pop_d [mrange $pop 1 1 0 end]
		    set sigdif [msub $pop_g $pop_d]
		    set llf [::math::statistics::lillieforsFit [vtrim $sigdif]]
		    $short insert end "\n\n\nREPORT about the SIGNED DIFFERENCES distribution\n\nLilliefors (variable [expr {$i+1}]) : $llf \n (n = [mrows $sigdif])\n-----------\n"
		    set sigli [::math::statistics::test-normal [vtrim $sigdif] 0.95]
		    if {$sigli < 1} {
			$short insert end "NOT significant. Could not reject normality. \n-----------\n"
		    } else {
			$short insert end "Distribution of signed difference is not normal (P < 0.05)\n-----------\n"
		    }
		    set histrap [tcllibhist $sigdif $i]
		    foreach item $histrap {
			$short insert end "\n$item\n"
		    }
		}
	    }
	    #
	    $short window create {end lineend} -window .sr.embed_eucl
	}
	#
	$short insert end "\n"

	#
	if {[info exists S(shapereport)]} {
	    #
	    $short insert end $S(shapereport)
	    $short insert end "\n"
	    destroy .sr.embed_plot
	    button .sr.embed_plot -text "PCA on left-rigth PW values for sucessive sessions" -font "courier 9" -command {
		
		global asi_individus
		numind $asi_individus
		# global pop_pwcs
		# global fectifs
		# #$p.embed_plot invoke
		# set fectifs ""
		# for {set e 0 }  {$e < [expr {[mrows $pop_pwcs] * 1/2}]} {incr e} {
		#     append fectifs "2 "
		# }
		# #debug "Nombre de lignes... pw_cs\t[mrows $pop_pwcs]\n effectifs: [llength $fectifs]\nfectifs"
		# global lignes colonnes
		# if {[info exists individus] == 1} {global effectifs; set effectifs $fectifs;numind $individus}
		$short configure -fg black
		set aide "\
                        \nPlot of PC1 (horizontal) and PC2 (vertical) of shape (PW) showing for each individual (each square)\
                        \nthe left and right sides (the extremities of the line crossing the square)\
                        \nHow to read the factor map:\
                        \na/ the apparent asymmetry is the « size » of the crossing line,\
                        \nb/ the apparent measurement error is the distance between homologous squares..."
		affiche 600 100 $aide
		$short insert end "\
                        \nPlot of PC1 (horizontal) and PC2 (vertical) of shape (PW) showing for each individual (each square)\
                        \nthe left and right sides (the extremities of the line crossing the square)\
                        \nHow to read the factor map:\
                        \na/ the apparent symmetry is the « size » of the crossing line,\
                        \nb/ the apparent measurement error is the distance between homologous squares..."
	    }
	    $short window create {end lineend} -window .sr.embed_plot
	}
    }

    button .buttons.exitanyway -text "Salir (Exit)" -command {
	set yesnocancel [tk_messageBox -message "Did you see the \"Short Report\"?\nDid you save the report?" -type yesnocancel]
	switch -- $yesnocancel {
	    yes {exit}
	    no {
		.buttons.informe invoke
		exit
	    }
	    cancel {}
	}
    }

    button .buttons.informe -text "Informe (Report)" -command {
	global infile
	savelisting
	set reportage [concat [file rootname $infile] "_INFO.txt"]
	tk_messageBox -message "[file tail $reportage] \nis saved in\n [file dirname $reportage]" -type ok
	exit
	#destroy .buttons.informe .buttons.exitanyway
	#pack .buttons.salir -side left -padx 2 -fill x -expand yes
    }

    button .buttons.pw -text "Single (shape) file" -fg brown -bg "light yellow" -command {
    	global S
    	set S(rasi) "1"
    	set S(pwcs) 0
	set S(pw) 1
    	# petite fenetre qui demande le nombre de mesures
    	destroy .symmetry
    	toplevel .symmetry -background grey66
    	set bmg .symmetry
    	wm geometry $bmg 390x180+500+210
    	#Une etiquette decorative, au-dessus
    	set aviso "Procrustes ANOVA on shape variables"
    	set label0  [label $bmg.etiquette0 -background blue -fg white -text $aviso]
    	pack $label0 -side top -fill both -expand 1
    	set label2 [label $bmg.etiquette2 -background red -foreground white -text "How many Measurements / Cuantas Mediciones  "]
    	set med [entry $bmg.entree2 -background grey77 -textvariable inputval2];#le nombre de mesures dans "inputval2"
    	pack $label2 $med -side left -fill both -expand 1
    	#
    	focus -force $med
    	# On appuie ensuite sur return, on va dans GET_PW placer les donnees dans "pop_pw",
    	# 'nbindiv mediciones' dans "inputval", et on est envoye a la procedure STATIS
    	bind $med <Return>  {
    	    if {$inputval2 > 0} {
    		global S
    		destroy $bmg
    		get_pw $inputval2;# 'nbindiv mediciones' dans "inputval" et les donnees dans "pop_pwcs"
    		#
    		.buttons.fascores configure -state disabled
    		if {[info exists inputval]} {
    		    statis $inputval
    		} else  {
    		    return
    		}
    	    } else  {
    		type "You did not enter an acceptable number of measurements"
    		destroy .symmetry
    	    }
    	}
    }
     focus .buttons.pw
     # bind ...
     bind .buttons.pw <Return> {
     	.buttons.pw invoke
    }

    # menubutton .buttons.menu -bg "light yellow" -text "Single (landmarks) file" -menu .buttons.menu.1
    # set m1 [menu .buttons.menu.1 ]
    # #
    # $m1 add command -label "raw coordinates ('..._format.txt')" 

    button .buttons.menu -bg "light yellow" -fg "brown" -text "Single (landmarks) file" -command {
	#tk_messageBox -message  "Please open a single file containing a concatenation of \n ..._format.txt <- dig. 1\n ..._format.txt <- dig. 2\netc." -type ok
	#
	destroy .avis
	global S
	set S(rasi) "1"
	set S(pwcs) 1
	set S(pw) 0
	# petite fenetre qui demande le nombre de mesures
	destroy .symmetry
	toplevel .symmetry -background grey66
	set bmg .symmetry
	wm geometry $bmg 390x180+500+210
	#Une etiquette decorative, au-dessus
	set aviso "RAW coordinates of landmarks...\nPlease open a single file containing a concatenation of \nsuccessive digitizations\n ..._format.txt <- dig. 1\n ..._format.txt <- dig. 2\netc.\ni.e., raw coordinates of successive digitization"
	set label0  [label $bmg.etiquette0 -background blue -fg white -text $aviso]
	pack $label0 -side top -fill both -expand 1
	set label2 [label $bmg.etiquette2 -background red -foreground white -text "How many Measurements / Cuantas Mediciones  "]
	set med [entry $bmg.entree2 -background grey77 -textvariable inputval2];#le nombre de mesures dans "inputval2"
	pack $label2 $med -side left -fill both -expand 1
	#
	focus -force $med
	# On appuie ensuite sur return, on va dans GET_LM placer les donnees dans "pop_pwcs",
	# 'nbindiv mediciones' dans "inputval", et on est envoye a la procedure STATIS
	bind $med <Return>  {
	    if {$inputval2 > 0} {
		global S
		destroy $bmg
		get_lm $inputval2
		#
		.buttons.fascores configure -state active
		if {[info exists inputval]} {
		    statis $inputval
		} else  {
		    return
		}
	    } else  {
		type "You did not enter an acceptable number of measurements"
		return
		#destroy .symmetry
	    }
	}
    }
    # #
    # $m1 add command -label "shape and size ('..._PW_CS.txt', or '..._ProcrCp_CS')" -command {
    # 	tk_messageBox -message  "hehe"
    # 	global S
    # 	set S(rasi) "1"
    # 	set S(pwcs) 1
    # 	# petite fenetre qui demande le nombre de mesures
    # 	destroy .symmetry
    # 	toplevel .symmetry -background grey66
    # 	set bmg .symmetry
    # 	wm geometry $bmg 390x180+500+210
    # 	#Une etiquette decorative, au-dessus
    # 	set aviso "First column is SIZE, next columns are SHAPE..."
    # 	set label0  [label $bmg.etiquette0 -background blue -fg white -text $aviso]
    # 	pack $label0 -side top -fill both -expand 1
    # 	set label2 [label $bmg.etiquette2 -background red -foreground white -text "How many Measurements / Cuantas Mediciones  "]
    # 	set med [entry $bmg.entree2 -background grey77 -textvariable inputval2];#le nombre de mesures dans "inputval2"
    # 	pack $label2 $med -side left -fill both -expand 1
    # 	#
    # 	focus -force $med
    # 	# On appuie ensuite sur return, on va dans GET_PWCS placer les donnees dans "pop_pwcs",
    # 	# 'nbindiv mediciones' dans "inputval", et on est envoye a la procedure STATIS
    # 	bind $med <Return>  {
    # 	    if {$inputval2 > 0} {
    # 		global S
    # 		destroy $bmg
    # 		get_pwcs $inputval2;# 'nbindiv mediciones' dans "inputval" et les donnees dans "pop_pwcs"
    # 		#
    # 		.buttons.fascores configure -state active
    # 		if {[info exists inputval]} {
    # 		    statis $inputval
    # 		} else  {
    # 		    return
    # 		}
    # 	    } else  {
    # 		type "You did not enter an acceptable number of measurements"
    # 		destroy .symmetry
    # 	    }
    # 	}
    # }
    

    # Bouton C: on entre un fichier par mesure
    #
    button .buttons.cs -fg brown -bg "light yellow" -text "Separate (size) files" -command {
.buttons.menu configure -state disabled
	global S infile
destroy .avis
	set S(rasi) "2"
	set S(pwcs) 0
	set S(pw) 0
	# petite fenetre ?
	# non!
	# essai d'annulation de la petite fenetre car on connait
	# forcement le nombre de mesures (cfr. 'two separate files' ...) !
	# et on connaitra le nombre d'individus (cfr la procedure modifiée GET_FILES)
	set nbind [get_files 2]
	#
	set inputval "$nbind 2"
	statis $inputval
	return
	# # en principe le bouton se termine ici...
	# #
	# destroy .symmetry
	# toplevel .symmetry -background grey66
	# set bmg .symmetry
	# wm geometry $bmg 490x180+500+210
	
	# set aviso "After filling the hereunder entries, \
	    #             \nand after pressing ENTER, \
	    #             \nyou will be asked \
	    #             \nto open the files corresponding to \
	    #             \nthe successive sets of (size) measurements"
	# set label0  [label $bmg.etiquette0 -background red -fg white -text $aviso]
	# pack $label0 -side top -fill both -expand 1
	
	# set label_ind [label $bmg.labind -bg red]
	# pack $label_ind -side top -fill both -expand 1
	
	# set label1 [label $bmg.labind.etiquette1 -background blue -foreground white -text "Individuals  (half the number of rows) /  Individuos  (mitad lineas)"]
	# set indi [entry $bmg.labind.entree1 -background grey88 -textvariable inputval1]
	# pack $label1 $indi -side left -fill x -expand 1
	
	# set label_med [label $bmg.labmed -bg red]
	# pack $label_med -side top -fill both -expand 1
	
	# set label2 [label $bmg.labmed.etiquette2 -background blue -foreground white \
	    # 		    -text "Measurements / Mediciones "]
	# set med [entry $bmg.labmed.entree2 -background grey88 -textvariable inputval2]
	# button $bmg.go -background "light yellow" -text GO! -command {
	# 	if {$inputval1 > 0} {
	# 	    if {$inputval2 > 0} {
	# 		global S infile
	# 		destroy $bmg
	# 		set inputval "$inputval1 $inputval2"
	# 		get_files $inputval2
	# 		statis $inputval
	# 	    } else  {
	# 		type "You did not enter an acceptable number of measurements"
	# 	    }
	# 	} else  {
	# 	    type "You did not enter an acceptable number of individuals"
	# 	}
	# }
	# pack $label2 $med $bmg.go -side left -fill x -expand 1
	
	# focus -force $indi
	
	# # On appuie ensuite sur RETURN,
	# # ces groupes sont enregistres dans inputval,
	# # et on est envoye a la procedure statis
    }
    # .buttons.cs -- FIN


    button .buttons.fascores -text "FA scores" -state disabled -command {
	#
	global p S infile 
	global pop_pwcs inputval2 
	global diff_gd
	#
.buttons.fascores configure -state disabled
#
	set repet $inputval2;# donnée entrée au départ
	set nbrows [mrows $pop_pwcs]
	set nbind [expr {$nbrows * 0.5/$repet}]
	set nbcols [mcols $pop_pwcs]
	#

	# Moyenne des différences bilaterales : asymetric component
	set fichier [file rootname $infile]
	set fichier [string replace [file rootname $infile] end-4 end "ALIGNED.txt"]
	#

	# On va chercher les donnees
	#
	# global(aligned) est une LISTE de MATRICES à deux colonnes (x,y) 
	# (pour chaque matrice, autant de lignes que de LM)
	# (et pour global(), autant de matrices que d'individus)
	if {[info exists global(aligned)]} {
	    # On va en faire une grande matrice avec en lignes les individus, en colonnes les coordonnées
	    set table_rows [llength $global(aligned)];# au moins 4 fois le nombre d'individus
	    set table_cols [lindex [lindex $global(aligned) 0] 1];#nombre de LM de la première matrice
	    set table_cols [expr {$table_cols *2}];# deux coordonnees par LM
	    for {set i 0} {$i < $table_rows} {incr i} {
		append pre_table_proc "[vtrim [lindex $global(aligned) $i]] "
	    }
	    set hume "2 $nbrows $table_cols "
	    set table_proc "$hume $pre_table_proc"
	    set hume_proc "[lrange $table_proc 0 2] [show $table_proc %8.6f]";# < 7 decimales
	    #
	} else {
	    # si global(aligned) n'existe pas, donc si on a entré pw_cs, on va chercher le fichier des aligned
	    #
	    if { [catch {
		set fid [open $fichier r]
		set proc [read -nonewline $fid]
		close $fid
		#debug "Procrustes coordinates (aligned specimens) $proc"
	    } result]} {
		set typeList {
		    {{"Procrustes coordinates: '..._ALIGNED.txt ?'"} { .txt .TXT} }
		    {{"Text files"} {.txt .txt .nts .TXT .NTS .TPS} }
		    {{"All files"} {*} }
		}
		set infile_aligned [tk_getOpenFile -filetypes $typeList -initialdir [file dirname $infile]]
		#infile_aligned contient le chemin et le nom du fichier des donnees
		set answer [type_yn "Does \"[file tail $infile_aligned]\" contain Procrustes coordinates?"]
		if {$answer == "no"} {
		    exit
		} else {
		    #
		    # with proc = Procrustes coordinates
		    set fid [open $infile_aligned r]
		    set proc [read -nonewline $fid]
		    close $fid
		    #
		    set table_proc [lrange [split $proc "\n"] 1 end]
		    # Enlever les curved brackets
		    set table_proc [regsub -all {\{|\}} $table_proc ""]
		    set table_rows [mrows $pop_pwcs]
		    set table_cols [expr {[mcols $pop_pwcs] + 3}];# ne pas compter la taille
		    
		    set hume_proc "2 $table_rows $table_cols $table_proc"
		    set hume_proc "[lrange $hume_proc 0 2] [show $hume_proc %8.6f]";# < 7 decimales
		    #debug "\nHUME TABLE (hume_proc) with Procrustes coordinates, $table_rows X $table_cols, \n$hume_proc [show $hume_proc]"
		    #
		}
		#
	    } else {
		type "$fichier is supposed to contain Procrustes coordinates "
	    }
	}

	# with ind_dg = le nombre d'individus droits et gauches
	set ind_dg [expr {$table_rows * 1/$repet}]
	set ind [expr {$ind_dg * 1/2}]
	# 1. séparer les ($repet) digitalisations
	set compteur 0
	for {set i 0} {$i < $repet} {incr i} {
	    set table_dig($i) [mrange $hume_proc 0 end $compteur [expr {$compteur + $ind_dg - 1}]]
	    incr compteur $ind_dg
	}
	#for {set i 0} {$i < $repet} {incr i} {debug "\n[expr {$i+1}]\n[show $table_dig($i)]"}
	#
	# From here, please compute the SYMMETRIC COMPONENT and SAVE IT
	#
	# d'abord la moyenne deux par deux dans table_dig(1), et dans table_dig(2), etc.
	for {set i 0} {$i < $repet} {incr i} {
	    for {set j 0} {$j < [mrows $table_dig($i)]} {incr j 2} {
		set a [mrange $table_dig($i) 0 end $j $j]
		set b [mrange $table_dig($i) 0 end [expr {$j+1}] [expr {$j+1}]]
                set ab_total [madd $a $b];# somme d et g
		set ab_mean [mscale $ab_total 0.5];# moyenne d et g
		#debug "mean\n [show $ab_mean]\n"
		append ici "[show $ab_mean] "
	    }
	    set dig_m($i) "2 [expr {[mrows $table_dig($i)]/2}] $table_cols $ici"
	    #
	    set ici "";# because of the append above
	};# fin de la boucle du calcul des moyennes d g
	#
	#
	# puis la moyenne des (deux) matrices dig_m($i) set s_mean [] etc
	#
	set m_total [madd $dig_m(0) $dig_m(1)];# les deux premières matrices
	for {set i 2} {$i < $repet} {incr i} {
	    set m_total [madd $m_total $dig_m($i)];# si > 2 matrices
	}
	#$p insert end "\nmtotal \n[show $m_total]"
	set s_mean [mscale $m_total [expr {1 * 1.0/$repet}]]
	#debug "\n\nmmean \n[show [mscale $m_total [expr {1 * 1.0/$repet}]]]"
	#
	# "Symmetric component yet to save"
	#
	# OPEN THE FILE
	set megainfo "Symmetric component\n"
	set filename "[file rootname $infile]_SymComp.txt"
	set fileid [open $filename w]
	puts -nonewline $fileid $megainfo
	puts -nonewline $fileid [show $s_mean]
	# CLOSE THE FILE
	close $fileid
	type "SYMMETRIC COMPONENT saved in $filename"
	# --------------------------------------------------------
	# 
	#
	# From here, please compute the Asymmetric COMPONENT and SAVE IT
	#
	# with lr = left - right (asymmetric component)
	for {set j 0} {$j < $repet} {incr j} {
	    for {set i 0} {$i < [expr {($ind*2)}]} {incr i 2} {
		set left [mrange $table_dig($j) 0 end $i $i]
		set cote [expr {$i+1}]
		set right [mrange $table_dig($j) 0 end $cote $cote]
	        append lr($j) [vtrim [msub $left $right]]
		append lr($j) "\n";# attention, lr() est une liste
		#debug "dig [expr {$j+1}], ind [expr {$i+1}]\n[show $left]\n[show $right]"
	    }
	    #debug "\nDIGITIZATION $j, diff \n$lr($j)\n"
	}
	# de liste à matrice (Hume)
	set hume "2 $ind $table_cols"
	for {set j 0} {$j < $repet} {incr j} {
	    set lr($j) "$hume $lr($j)"
	}
	#
	# Moyenne des matrices (correspondant à chaque dig)
	#
	set m_total [madd $lr(0) $lr(1)]
	for {set i 2} {$i < $repet} {incr i} {
	    set m_total [madd $m_total $lr($i)]
	}
	#debug "\nmtotal \n[show $m_total]"
	set m_mean [mscale $m_total [expr {1 * 1.0/$repet}]]
	# FILE TO SAVE WITH ASYMMETRIC COMPONENTS
	# "\n\nasymmetric component \n[show $m_mean]"
	#
	# OPEN THE FILE
	set megainfo "Asymmetric components\n"
	set filename "[file rootname $infile]_AsymComp.txt"
	set fileid [open $filename w]
	puts -nonewline $fileid $megainfo
	puts -nonewline $fileid [show $m_mean]
	# CLOSE THE FILE
	close $fileid
	type "ASYMMETRIC COMPONENT saved in $filename"
	#
	#
	# A présent les distances de Procrustes (shape FA scores) % à la moyenne
	# 
	# 1. Centrer
	mnorms_br m_mean mm_means sm_means
	set popones "2 1 $colonnes 1"
	set x 1
	set y $table_cols
	while {$x < $y} {lappend popones 1; set x [expr {$x + 1} ] }
	#
	#debug "moyennes [show $mm_means]"
	set cm_mean [mnormalize $m_mean $mm_means $popones];# centrer...
	#debug "centrer\n[show $cm_mean]"
	# 2. Square
	set cm_mean_squared [mprod $cm_mean $cm_mean]
	#debug "squared\n[show $cm_mean_squared]"
	# 3. Sum
	set pre_fascores [msum [transpose $cm_mean_squared]]
	#debug "sum of squares\n[show $pre_fascores]"
	# 4. sqrt
	proc sqrt x { return [expr sqrt($x)] }
	set fascores [mat_unary_op $pre_fascores sqrt]
	#debug "PROCRUSTES FA SCORES\n[show $fascores]"
	#
	# On enregistre
	set hume "2 [llength [vtrim $fascores]] 1 "
	set scoresfa "$hume [vtrim $fascores]"

	# #
	# # OPEN THE FILE
	# set megainfo "FA scores are shape deviations (shape, not size) from the group mean ([file tail $infile]) in a isotropic space.\n"
	# set filename "[file rootname $infile]_FAscores.txt"
	# set fileid [open $filename w]
	# puts -nonewline $fileid $megainfo
	# puts -nonewline $fileid [show $scoresfa]
	# # CLOSE THE FILE
	# close $fileid
	# type "FA scores saved in $filename"
	# -------------------------
	# tentative de correlation FA scores Centroid Size
	# -------------------------
	#
	# le S(centroide) contient les tailles de toutes les configurations, il faut en tirer la taille moyenne de chaque individu, moyenne entre la droite et la gauche, moyenne entre les digitalisations successives
	set indnb [mrows $scoresfa]
	set confignb [mrows $S(centroide)]
	set dignb [expr {$confignb * 1/$indnb * 1/2}]
	if {$dignb != $repet} {type "souci : dignb et repet"}
	set dg [expr {$indnb*2}]
	#
	set compteur 0
	for {set i 0} {$i < $confignb} {incr i $dg} {
	    set centroid($compteur) [mrange $S(centroide) 0 0 $i [expr {$i+$dg-1}]]
	    incr compteur
	}
	#debug "centroid(0)\n[show $centroid(0)]\ncentroid(1)\n[show $centroid(1)]"
	#
	# puis la moyenne des (deux) matrices centroid($i) set c_mean [] etc
	#
	set c_total [madd $centroid(0) $centroid(1)];# les deux premières matrices
	for {set i 2} {$i < $repet} {incr i} {
	    set c_total [madd $c_total $centroid($i)];# si > 2 digitizarions
	}
	#debug "\nctotal \n[show $c_total]"
	set c_mean [mscale $c_total [expr {1 * 1.0/$repet}]]
	#debug "\n\nmean size\n[show $c_mean]"

	# A présent les moyennes entre droite et gauche
	for {set i 0} {$i < $dg} {incr i 2} {
	    #debug "$i et [expr {$i+1}]"
	    set a [mrange $c_mean 0 0 $i $i]
	    #debug "a $a"
	    set b [mrange $c_mean 0 0 [expr {$i+1}] [expr {$i+1}]]
	    #debug "b $b\na+b [madd $a $b]"
	    set ab [madd $a $b]
	    #set ab2 [mscale $ab 0.5]
	    #debug "ab2 = $ab2"
	    set ab [expr {[vtrim $ab] * 1/2}]
	    append mdg "$ab "
	}
	#debug "mdg $mdg"
	set hume "2 $indnb 1" 
	set mdg "$hume $mdg"
	#debug $mdg
	#debug [show $mdg]

	#debug "mdg\n$mdg\nfascores\n$scoresfa"
	set corps [join_cols $mdg $scoresfa]
	set R [correlation $corps]
	#debug "R = $R"
	# -------------------------
	# tentative de plotting
	# -------------------------
	#reggrisexy [mrange $corps 1 1 0 end] [mrange $corps 0 0 0 end] "size" "FAscores"




	# -------------------------
	# tentative de correlation size and shape FA scores
	# -------------------------
	#
	# le S(centroide) contient les tailles de toutes les configurations, il faut en tirer le vecteur centré des différences bilatérales
	set indnb [mrows $scoresfa]
	set confignb [mrows $S(centroide)]
	set dignb [expr {$confignb * 1/$indnb * 1/2}]
	if {$dignb != $repet} {type "souci : dignb et repet"}
	set dg [expr {$indnb*2}]
	#
	set compteur 0
	for {set i 0} {$i < $confignb} {incr i $dg} {
	    set centroid($compteur) [mrange $S(centroide) 0 0 $i [expr {$i+$dg-1}]]
	    incr compteur
	}
	# A présent, centroid(0) et centroid(...) contiennent les tailles droite et gauche des digitalisations successives
	#debug "centroid(0)\n[show $centroid(0)]\ncentroid(1)\n[show $centroid(1)]"
	#
	# On calcule la moyenne des (deux) matrices centroid($i) set c_mean [] etc
	#
	set c_total [madd $centroid(0) $centroid(1)];# les deux premières matrices
	for {set i 2} {$i < $repet} {incr i} {
	    set c_total [madd $c_total $centroid($i)];# si > 2 digitizarions
	}
	#debug "\nctotal \n[show $c_total]"
	set c_mean [mscale $c_total [expr {1 * 1.0/$repet}]]
	#debug "\n\nmean size\n[show $c_mean]"

	# A présent les DIFFERENCES entre droite et gauche
	for {set i 0} {$i < $dg} {incr i 2} {
	    #debug "$i et [expr {$i+1}]"
	    set a [mrange $c_mean 0 0 $i $i]
	    #debug "a $a"
	    set b [mrange $c_mean 0 0 [expr {$i+1}] [expr {$i+1}]]
	    #debug "b $b\na+b [msub $a $b]"
	    set ab [msub $a $b]
	    #set ab2 [mscale $ab 0.5]
	    #debug "ab2 = $ab2"
	    #set ab [expr {[vtrim $ab] * 1/2}]
	    append ddg "[vtrim $ab] "
	}
	#debug "ddg $ddg"
	set hume "2 $indnb 1" 
	set ddg "$hume $ddg"
	#debug $ddg
	#debug "ddg \n[show $ddg]"

	# Individual scores of fluctuating asymmetry (FA scores for size)
	set ddg [centering $ddg]
	rouge "\n\nIndividual scores of size fluctuating asymmetry (FA scores for size)\n(centered vector of averaged bilateral differences)"
	$p insert end "\n[show $ddg]"
	$p see end

	#debug "mdg\n$ddg\nfascores\n$scoresfa"
	set corps [join_cols $ddg $scoresfa]
	set R [correlation $corps]
	#debug "\ncorps \n[show $corps]\nR = $R"
	# -------------------------
	# tentative de plotting
	# -------------------------
	set transit $effectifs
	unset effectifs
	#set effectifs "[mrows $corps] [mrows $corps]"
	reggrisexy [vtrim [mrange $corps 1 1 0 end]] [vtrim [mrange $corps 0 0 0 end]] " = size FA scores" " = shape FA scores"
	global effectifs
	set effectifs $transit
	# set S(numero) 1
	# global asi_effectifs
	# set asi_effectifs [mrows $corps]
	# asidraw_reggrise [mrange $corps 1 1 0 end] [mrange $corps 0 0 0 end] "size FA scores" "shape FA scores"

	# On joint les scores individuels de FA shape et size dans un seul tableau
	set newscoresfa [join_cols $scoresfa $ddg]
	# On M O N T R E les scores de ce tableau correspondant à l'asymétrie fluctuante de la conformation
	# Individual scores of fluctuating asymmetry (FA scores for SHAPE)
	rouge "\n\nIndividual scores of SHAPE fluctuating asymmetry (shape FA scores)\n(Procrustes distance of each individual bilateral asymmetry to the mean bilateral difference, applied to the centered differences)"
	$p insert end "\n[show [mrange $newscoresfa 0 0 0 end]]"
	$p see end
	# On E N R E G I S T R E ce tableau ("FA SCORES"
	#
	# OPEN THE FILE
	set megainfo "shape FA scores are shape deviations (shape, not size) from the group mean ([file tail $infile]) in a isotropic space.  size FA scores are the centered vector of averaged bilateral differences of size\n"
	set filename "[file rootname $infile]_FAscores.txt"
	set fileid [open $filename w]
	puts -nonewline $fileid $megainfo
	puts -nonewline $fileid [show $newscoresfa]
	# CLOSE THE FILE
	close $fileid
	type "FA scores saved in $filename"

















	# # -------------------------
	# # tentative RETIREE d'histogramme sur la moyenne des scores
	# # -------------------------
	# set scores_m $fascores
	# # canvas c
	# toplevel .fasc
	# set c [canvas  .fasc.ici  -width 550 -height 550 -relief raised -background "light yellow" -cursor dot]
	# pack $c -side top -expand yes -fill both
	# set valclass $ind
	# set scores_m [vtrim $scores_m];# doit etre une liste
	# histograbe_asi $scores_m $valclass $c
    };# button fascores - fin
    
    
    
    button .buttons.exit -text "EXIT (Salir)" -command {exit}

    ## VOILA LES BOUTONS
    #pack .buttons.informe .buttons.pwcs .buttons.permuterD .buttons.adda .buttons.cs -side left -fill y -expand 1
    #pack .buttons.pwcs .buttons.menu .buttons.cs .buttons.sr  .buttons.exit -side left -fill y -expand 1
    pack  .buttons.cs .buttons.pw .buttons.menu .buttons.sr .buttons.exit -side left -fill y -expand 1

    balloonhelp_for .buttons.menu "A single file containing RAW coordinates of landmarks; see o.help"
    balloonhelp_for .buttons.cs "Only SIZE asymmetry\nPlease open 2 separate files containing SIZE (CS,PER,ARE,etc.); see o.help"
    balloonhelp_for .buttons.fascores "Klingenberg & Monteiro, 2005. Systematic Biology, 54: 679-688"
    balloonhelp_for .buttons.r "Two options for Procrustes ANOVA with R"

    destroy .scrollA ;# au cas ou on rappelle la procedure principale (cfr. UNOT)

    set scb [scrollbar .scrollA -command ".debut yview"];
    ## A GAUCHE DE LA FENETRE, UNE SCROLLBAR
    pack $scb -side left -fill y

    destroy .debut ;# au cas ou on rappelle la procedure principale (cfr. UNOT)

    set pres [text .debut -background white -font "helvetica 12" -wrap none -yscrollcommand ".scrollA set"];
    ## PUIS ENFIN LE TEXTE il n'est plus necessaire d'indiquer -SIDE, mais -FILL BOTH
    pack $pres -fill both -expand 1

    global lignes
    global colonnes
    set p .debut
    ## LE NOM DE LA SOUS-FENETRE DE TEXTE (.debut) DANS UNE VARIABLE (p)
    $p tag configure bleu -foreground blue
    $p tag configure rouge -foreground red
    $p tag configure vert -foreground "dark green"
    update
    rouge $aide_input
    bleu $aide_fr_pw_cs
    vert $aide_fr_1mes1file
    bleu $aide_en_shape
    #rouge "\n---------------------------------------------------------\n"
    #$p insert end $aide_es_att
    #rouge "\n---------------------------------------------------------\n"
    #$p insert end $aide_en_att
    #rouge "\n---------------------------------------------------------\n"
    #$p insert end $aide_fr_att

    # Ajout d'un message suppliant
    #set missive "Blue button for _PW_CS.txt file as produced by MOG, \
	you will be asked to enter the number of measurements.\
	\n\n\nRed button for separate CS files\
	\n you will be asked to enter the number of measurements\
	\nAND\n the number of individuals.\
	\n\n Please note that the number of individuals is half the number of rows ..."
    #tk_messageBox -message $missive -icon warning -type ok
    
    set avis "The 'Single (landmarks) file' as input file \
        \nmust combine the successive digitizations. \
        \nPLEASE USE the 'TET' module of CLIC to combine them \
	\n(Menu 'Utilities-> From 4 files to ASI') \
        \n\nThis TET utility produces a file ending by '..._toasi' \
        \nwhich has to be called by the 'Single (landmarks) file' button, \
	\n\nHere is its internal format \
        \nwith d = digitization, i = individual, s = side\
	\n\n\
	- row 1 =       d1 i1 s1\n\
	- row 2 =       d1 i1 s2\n\
	- row 3 =       d1 i2 s1\n\
	- row 4 =       d1 i2 s2\n\
	- row 5 =       d1 i3 s1\n\
	- row 6 =       d1 i3 s2\n\
	- ...\n\
	- row (r*2-1) = d1 i(r) s1\n\
	- row (r*2) =   d1 i(r) s2\n\
	\n\
	- row 1 =       d2 i1 s1\n\
	- row 2 =       d2 i1 s2\n\
	- row 3 =       d2 i2 s1\n\
	- row 4 =       d2 i2 s2\n\
	- row 5 =       d2 i3 s1\n\
	- row 6 =       d2 i3 s2\n\
	- ...\n\
	- row (r*2-1) = d2 i(r) s1\n\
	- row (r*2) =   d2 i(r) s2\n"
    affiche 790 110 $avis
    
    ## fin de la FENETRE PRINCIPALE
}
#
# Procedure PRINCIPAL --- fin

# Au debut du programme, j'envoie l'interpreteur a la procedure 'PRINCIPAL'
# Cette procedure 'PRINCIPAL' fait appel a toutes les autres procedures du script
#
principal
